/*******************************************************************************
  
  Copyright(c) 2004 Intel Corporation. All rights reserved.

  Portions of this file are based on the WEP enablement code provided by the
  Host AP project hostap-drivers v0.1.3
  Copyright (c) 2001-2002, SSH Communications Security Corp and Jouni Malinen
  <jkmaline@cc.hut.fi>
  Copyright (c) 2002-2003, Jouni Malinen <jkmaline@cc.hut.fi>
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of version 2 of the GNU General Public License as 
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  James P. Ketrenos <ipw2100-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/

#include <linux/compiler.h>
#include <linux/config.h>
#include <linux/errno.h>
#include <linux/if_arp.h>
#include <linux/in6.h>
#include <linux/in.h>
#include <linux/ip.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/pci.h>
#include <linux/proc_fs.h>
#include <linux/skbuff.h>
#include <linux/slab.h>
#include <linux/tcp.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/wireless.h>
#include <linux/etherdevice.h>
#include <asm/uaccess.h>

#include "ieee802_11.h"
#include "ieee80211.h"

#define MODULE_NAME	"ieee80211"

MODULE_DESCRIPTION("802.11 data/management/control stack");
MODULE_AUTHOR("Copyright (C) 2004 Intel Corporation <jketreno@linux.intel.com>");
MODULE_LICENSE("GPL");

struct ieee80211_device *ieee80211_alloc(struct net_device *dev,
					 void *priv)
{
	struct ieee80211_device *ieee = kmalloc(
		sizeof(struct ieee80211_device), GFP_KERNEL);
	if (ieee == NULL)
		return NULL;
	memset(ieee, 0, sizeof(*ieee));
	ieee->dev = dev;
	ieee->priv = priv;

#ifndef CONFIG_IEEE80211_NOWEP
	/* Default to enabling full open WEP with host based encrypt/decrypt */
	ieee->open_wep = 1;
	ieee->host_encrypt = 1;
	ieee->host_decrypt = 1;

	INIT_LIST_HEAD(&ieee->crypt_deinit_list);
	init_timer(&ieee->crypt_deinit_timer);
	ieee->crypt_deinit_timer.data = (unsigned long)ieee;
	ieee->crypt_deinit_timer.function = ieee80211_crypt_deinit_handler;

#else
	ieee->open_wep = 1;
	ieee->host_encrypt = 0;
	ieee->host_decrypt = 0;
#endif

	/* Default fragmentation threshold is maximum payload size */
	ieee->fts = 2342;

	spin_lock_init(&ieee->lock);

	return ieee;
}

void ieee80211_free(struct ieee80211_device *ieee)
{
#ifndef CONFIG_IEEE80211_NOWEP
	int i;
#endif

	if (!ieee)
		return;

#ifndef CONFIG_IEEE80211_NOWEP

	// PR: FIXME: added this "if" because hostap had it, is it really needed?
	if (timer_pending(&ieee->crypt_deinit_timer))
		del_timer(&ieee->crypt_deinit_timer);
	ieee80211_crypt_deinit_entries(ieee, 1);

	for (i = 0; i < WEP_KEYS; i++) {
		struct ieee80211_crypt_data *crypt = ieee->crypt[i];
		if (crypt) {
			if (crypt->ops)
				crypt->ops->deinit(crypt->priv);
			kfree(crypt);
			ieee->crypt[i] = NULL;
		}
	}
#endif

	kfree(ieee);
}

static int __init ieee80211_init(void)
{
	printk(KERN_INFO MODULE_NAME ": loading with WEP "
#ifdef CONFIG_IEEE80211_NOWEP
	       "disabled"
#else
	       "enabled"
#endif
	       ".\n");

	return 0;
}

static void __exit ieee80211_deinit(void)
{
	printk(KERN_INFO MODULE_NAME ": unloading.\n");
}

EXPORT_SYMBOL(ieee80211_alloc);
EXPORT_SYMBOL(ieee80211_free);

module_init(ieee80211_init);
module_exit(ieee80211_deinit);

