/******************************************************************************
  
  Copyright(c) 2003 - 2004 Intel Corporation. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of version 2 of the GNU General Public License as 
  published by the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  James P. Ketrenos <ipw2100-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

******************************************************************************/

/*
 * This module contains the low level functions used for 
 * implementing the deffered processing assosiate with the ISR.
 * Most of the defferred work will be executed on work queues.
 * 
 * mgross@linux.intel.com
 */

#include "ipw2200.h"

void do_rx_transfer(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_rx_transfer \n ");

/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);
	inta &= ~CX2_INTA_BIT_RX_TRANSFER;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_tx_cmd_queue(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_tx_cmd_queue \n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);
	inta &= ~CX2_INTA_BIT_TX_CMD_QUEUE;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_tx_queue_1(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_tx_queue_1\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);
	inta &= ~CX2_INTA_BIT_TX_QUEUE_1;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_tx_queue_2(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_tx_queue_2\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_TX_QUEUE_2;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_tx_queue_3(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_tx_queue_3\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_TX_QUEUE_3;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_tx_queue_4(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_tx_queue_4\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_TX_QUEUE_4;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_status_change(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_status_change\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_STATUS_CHANGE;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_beacon_period_expired(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_beacon_period_expired\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_BEACON_PERIOD_EXPIRED;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_slave_mode_cmd_done(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_slave_mode_cmd_done\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_SLAVE_MODE_HOST_CMD_DONE;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_fw_init_done(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_fw_init_done\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_FW_INITIALIZATION_DONE;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_phy_off_done(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_phy_off_done\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_FW_CARD_DISABLE_PHY_OFF_DONE;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_rf_kill_done(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_rf_kill_done\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_RF_KILL_DONE;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}

void do_fatal_error(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_fatal_errorn\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_FATAL_ERROR;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);

}

void do_parity_error(void *data)
{
	u32 inta;
	u32 inta_mask;
	struct ipw2200_private *priv = (struct ipw2200_private *)data;

	IPW_DEBUG_TRACE("do_parity_error\n");
/* put work logic here*/

/* re-enable interrupts of this type */
	inta_mask = readl(priv->hw_base + CX2_INTA_MASK_R);
	inta = readl(priv->hw_base + CX2_INTA_RW);
	inta &= (CX2_INTA_MASK_ALL & inta_mask);

	inta &= ~CX2_INTA_BIT_PARITY_ERROR;
	writel(~inta & inta_mask, priv->hw_base + CX2_INTA_MASK_R);
}
