/******************************************************************************
  
  Copyright(c) 2003 - 2004 Intel Corporation. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the Free 
  Software Foundation; either version 2 of the License, or (at your option) 
  any later version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  James P. Ketrenos <ipw2100-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

******************************************************************************/

/*
 * This module implements the DMA setup and processing associated with tranfering
 * firmware data from host to device.
 *
 * mgross@linux.intel.com
 */

#include "ipw2200.h"


static int ipw2200_fw_dma_reset_command_blocks(struct ipw2200_private *priv)
{
	int index;
	unsigned long address = CX2_SHARED_SRAM_DMA_CNTRL;
	
	IPW_DEBUG_TRACE(">> : \n");
	for (index = 0; index < CB_NUMBER_OF_ELEMENTS_SMALL * sizeof(struct command_block)/sizeof(u32); index++)
	{
		ipw2200_write_reg32(priv, address, 0);
		address += sizeof(u32);
	}
	IPW_DEBUG_TRACE("<< : \n");
	return 0;
}

int ipw2200_fw_dma_enable(struct ipw2200_private *priv)
{ /* start dma engine but no transfers yet*/

	IPW_DEBUG_TRACE(">> : \n");
    
	/* Start the dma */
	ipw2200_fw_dma_reset_command_blocks(priv);
	
	/*Write CB base address */
	ipw2200_write_reg32(priv, CX2_DMA_I_CB_BASE, CX2_DMA_CONTROL_SHARED_LOWER_BOUND);

	IPW_DEBUG_TRACE("<< : \n");
	return 0;
}

static void ipw2200_fw_dma_abort(struct ipw2200_private *priv)
{
	u32 control = 0;

	IPW_DEBUG_TRACE(">> :\n");
    
	//set the Stop and Abort bit	
	control = DMA_CONTROL_SMALL_CB_CONST_VALUE|DMA_CB_STOP_AND_ABORT;
	ipw2200_write_reg32(priv,CX2_DMA_I_DMA_CONTROL,control);
	priv->sram_desc.last_cb_index = 0;
	
	IPW_DEBUG_TRACE("<< \n");
}

static int ipw2200_fw_dma_write_command_block(struct ipw2200_private *priv, int index, struct command_block *cb)
{
	u32 address = CX2_SHARED_SRAM_DMA_CNTRL + (sizeof(struct command_block) * index); 
	IPW_DEBUG_TRACE(">> :\n");

	ipw2200_write(priv, address,  (u8*) cb, sizeof(struct command_block));

	IPW_DEBUG_TRACE("<< :\n");
	return 0;

}

int ipw2200_fw_dma_kick(struct ipw2200_private *priv)
{
	u32 control = 0;
	u32 index=0;

	IPW_DEBUG_TRACE(">> :\n");
    
	for (index =0; index < priv->sram_desc.last_cb_index; index++)
	{
		ipw2200_fw_dma_write_command_block( priv, index, &priv->sram_desc.cb_list[index]);
	}

	/*Enable the DMA in the CSR register*/
	ipw2200_clear_bit(priv, CX2_RESET_REG,CX2_RESET_REG_MASTER_DISABLED | CX2_RESET_REG_STOP_MASTER);
	
	
        //Set the Start bit.
	control = DMA_CONTROL_SMALL_CB_CONST_VALUE | DMA_CB_START;
	ipw2200_write_reg32(priv, CX2_DMA_I_DMA_CONTROL, control);

	IPW_DEBUG_TRACE("<< :\n");
	return 0;
}




static void ipw2200_fw_dma_dump_command_block(struct ipw2200_private *priv)
{
	u32 address;
	u32 register_value=0;
	u32 cb_fields_address=0;

	IPW_DEBUG_TRACE(">> :\n");
	address = ipw2200_read_reg32(priv,CX2_DMA_I_CURRENT_CB);
	IPW_DEBUG_INFO("Current CB is 0x%x \n",address);

	/* Read the DMA Controlor register */
	register_value = ipw2200_read_reg32(priv, CX2_DMA_I_DMA_CONTROL);
	IPW_DEBUG_INFO("CX2_DMA_I_DMA_CONTROL is 0x%x \n",register_value);

	/* Print the CB values*/
	cb_fields_address = address;
	register_value = ipw2200_read_reg32(priv, cb_fields_address);
	IPW_DEBUG_INFO("Current CB ControlField is 0x%x \n",register_value);

	cb_fields_address += sizeof(u32);
	register_value = ipw2200_read_reg32(priv, cb_fields_address);
	IPW_DEBUG_INFO("Current CB Source Field is 0x%x \n",register_value);

	cb_fields_address += sizeof(u32);
	register_value = ipw2200_read_reg32(priv, cb_fields_address);
	IPW_DEBUG_INFO("Current CB Destination Field is 0x%x \n",register_value);

	cb_fields_address += sizeof(u32);
	register_value = ipw2200_read_reg32(priv, cb_fields_address);
	IPW_DEBUG_INFO("Current CB Status Field is 0x%x \n",register_value);

	IPW_DEBUG_TRACE(">> :\n");
}

static int ipw2200_fw_dma_command_block_index(struct ipw2200_private *priv)
{
	u32 current_cb_address = 0;
	u32 current_cb_index = 0;

	IPW_DEBUG_TRACE("<< :\n");
	current_cb_address= ipw2200_read_reg32(priv, CX2_DMA_I_CURRENT_CB);
	
	current_cb_index = (current_cb_address - CX2_DMA_CONTROL_SHARED_LOWER_BOUND )/
		sizeof (struct command_block);
	
	IPW_DEBUG_INFO("Current CB index 0x%x address = 0x%X \n",current_cb_index, current_cb_address );

	IPW_DEBUG_TRACE(">> :\n");
	return current_cb_index;

}

static int ipw2200_fw_dma_add_command_block(struct ipw2200_private *priv,
	u32 src_address,
	u32 dest_address,
	u32 length,
	int interrupt_enabled,
	int is_last)
{

	u32 control = CB_VALID|CB_SRC_LE|CB_DEST_LE|CB_SRC_AUTOINC|CB_SRC_IO_GATED|
	CB_DEST_AUTOINC|CB_SRC_SIZE_LONG|CB_DEST_SIZE_LONG;
	struct command_block *cb;
	u32 last_cb_element=0;

	IPW_DEBUG_TRACE(">>:\n");

	IPW_DEBUG_INFO(">>: src_address=0x%x dest_address=0x%x length=0x%x\n",
		 src_address, dest_address, length);

	if (priv->sram_desc.last_cb_index >= CB_NUMBER_OF_ELEMENTS_SMALL)
	{
		return -1;
	}

	last_cb_element = priv->sram_desc.last_cb_index;
	cb = &priv->sram_desc.cb_list[last_cb_element];
	priv->sram_desc.last_cb_index ++;

	/* Calculate the new CB control word */
	if (interrupt_enabled )
	{
	    control |= CB_INT_ENABLED;
	}

	if (is_last)
	{
		    control |= CB_LAST_VALID;
	}
	
	control |= length;

	/* Calculate the CB Element's checksum value */
	cb->status = control ^src_address ^dest_address;

	/* Copy the Source and Destination addresses */
	cb->dest_addr = (void *)dest_address;
	cb->source_addr = (void *)src_address;

	/* Copy the Control Word last */
	cb->control = control;

	IPW_DEBUG_TRACE("<< : \n");
	return 0;
}

int ipw2200_fw_dma_add_buffer(struct ipw2200_private *priv,
	u32 src_phys,
	u32 dest_address,
	u32 length)
{
	u32 bytes_left = length;
	u32 src_offset=0;
	u32 dest_offset=0;
	int status = 0;
	IPW_DEBUG_TRACE(">> \n");
	IPW_DEBUG_INFO("src_phys=0x%x dest_address=0x%x length=0x%x\n",
		src_phys, dest_address, length);
	while (bytes_left > CB_MAX_LENGTH  )
	{
		status = ipw2200_fw_dma_add_command_block( priv,
			src_phys + src_offset,
			dest_address + dest_offset,
			CB_MAX_LENGTH,
			0,
			0);
		if (status)
		{
			IPW_DEBUG_INFO(": Failed\n");
			return -1;
		}
		else
		{
			IPW_DEBUG_INFO(": Added new cb\n");

		}

		src_offset += CB_MAX_LENGTH;
		dest_offset += CB_MAX_LENGTH;
		bytes_left -= CB_MAX_LENGTH;
	}

	/* add the buffer tail */
	if (bytes_left > 0)
	{
		status = ipw2200_fw_dma_add_command_block( priv,	
			src_phys + src_offset,
			dest_address + dest_offset,
			bytes_left,
			0,
			0);
		if (status)
		{
			IPW_DEBUG_INFO(": Failed on the buffer tail\n");
			return -1;
		}
		else
		{
			IPW_DEBUG_INFO(": Adding new cb - the buffer tail\n");

		}
	}


    IPW_DEBUG_TRACE("<< \n");
	return 0;

}

int ipw2200_fw_dma_wait(struct ipw2200_private *priv)
{
	u32 current_index = 0;
	u32 watchdog = 0;

	IPW_DEBUG_TRACE(">> : \n");

	current_index = ipw2200_fw_dma_command_block_index(priv);
	IPW_DEBUG_INFO("sram_desc.last_cb_index:0x%8X\n", 
		       (int) priv->sram_desc.last_cb_index);
	while (current_index < priv->sram_desc.last_cb_index)
	{
		udelay(50); // wtf is this? NdisStallExecution(50);
		current_index = ipw2200_fw_dma_command_block_index(priv);

		watchdog++;
		if (watchdog > 400)
		{
			IPW_DEBUG_INFO("Timeout\n");
			ipw2200_fw_dma_dump_command_block(priv);
			ipw2200_fw_dma_abort(priv);
			return -1;
		}
	}

	ipw2200_fw_dma_abort(priv);
	/*Disable the DMA in the CSR register*/
 	ipw2200_set_bit(priv, CX2_RESET_REG, CX2_RESET_REG_MASTER_DISABLED | CX2_RESET_REG_STOP_MASTER);


	IPW_DEBUG_TRACE("<< dmaWaitSync \n");
	return 0;
}


