/******************************************************************************
  
  Copyright(c) 2003 - 2004 Intel Corporation. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of version 2 of the GNU General Public License as 
  published by the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  James P. Ketrenos <ipw2100-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

******************************************************************************/

/*
 * This is the main driver entry file for the 802.11bg 
 * hardware driver.  This file implements the following 
 * driver init, open, close, ioctl, and sysfs.
 * It provides a simple driver shell that will grow to implement the 
 * rest of the driver.....
 *
 * mgross@linux.intel.com
 */

#include "ipw2200.h"
#include "ipw2200_wx.h"

#define DRV_VERSION	"0.1"
#define DRV_DESCRIPTION	"Intel(R) PRO/Wireless 2200 Network Driver"
#define DRV_COPYRIGHT	"Copyright(c) 2003-2004 Intel Corporation"

MODULE_LICENSE("GPL");

#ifdef CONFIG_IPW_DEBUG
int ipw2200_debug_level;
#endif

/*
 * The following adds a new attribute to the sysfs representation
 * of this device driver (i.e. a new file in /sys/bus/pci/drivers/ipw2200/)
 * used for controling the debug level.
 * 
 * See the level definitions in ipw2200 for details.
 */
static ssize_t show_debug_level(struct device_driver *d, char *buf)
{
	return sprintf(buf, "0x%0x\n", ipw2200_debug_level);
}
static ssize_t store_debug_level(struct device_driver *d, const char *buf, 
				 size_t count)
{
	sscanf(buf, "%x", &ipw2200_debug_level);
	return strnlen(buf, count);
}
static DRIVER_ATTR(debug_level, S_IWUSR|S_IRUGO, 
		   show_debug_level,store_debug_level);

/*
 * The following adds a new attribute to the sysfs representation
 * of the device (i.e. a new file called "eeprom" in 
 * /sys/bus/pci/drivers/ipw2200/XXXX:XX:XX.X/) that when read will
 * trigger a dump of priv->eeprom 
 *
 * To use this, simply cat the "eeprom" file, and then "dmesg" to
 * see the contents of the eeprom
 */
static ssize_t show_eeprom(struct device *d, char *buf)
{
	int i;
	u8 eeprom[0x100];

	memcpy(eeprom,((struct ipw2200_private*)d->driver_data)->eeprom,0x100);
	for ( i=0; i<256; i+=8 ) {
		printk(KERN_INFO "0x%2x 0x%2x 0x%2x 0x%2x "
		       "0x%2x 0x%2x 0x%2x 0x%2x\n",
		       eeprom[i+0], 
		       eeprom[i+1],
		       eeprom[i+2],
		       eeprom[i+3],
		       eeprom[i+4],
		       eeprom[i+5],
		       eeprom[i+6],
		       eeprom[i+7]);
	}
	
	return 0;
}
static DEVICE_ATTR(eeprom, S_IWUSR|S_IRUGO, show_eeprom, NULL);

/*
 * Add a device attribute to view/control the delay between eeprom
 * operations.
 */
static ssize_t show_eeprom_delay(struct device *d, char *buf)
{
	int n = ((struct ipw2200_private*)d->driver_data)->eeprom_delay;
	return sprintf(buf, "%i\n", n);
}
static ssize_t store_eeprom_delay(struct device *d, const char *buf, 
				  size_t count)
{
	struct ipw2200_private *p = (struct ipw2200_private*)d->driver_data;
	sscanf(buf, "%i", &p->eeprom_delay);
	return strnlen(buf, count);
}
static DEVICE_ATTR(eeprom_delay, S_IWUSR|S_IRUGO, 
		   show_eeprom_delay,store_eeprom_delay);

/* net device stuff */
static int ipw2200_net_open(struct net_device *dev)
{
	/* we should be verifying the device is ready to be opened */
	netif_start_queue(dev);
	return 0;
}

static int ipw2200_net_stop(struct net_device *dev)
{
	netif_stop_queue(dev);
	return 0;
}

static int ipw2200_net_hard_start_xmit(struct sk_buff *skb,
				       struct net_device *dev)
{
	return -1;
}

#ifdef NOT_YET
static struct net_device_stats *ipw2200_net_get_stats(struct net_device *dev)
{
	return NULL;
}
#endif
static void ipw2200_net_set_multicast_list(struct net_device *dev)
{

}
static int ipw2200_net_set_mac_address(struct net_device *dev, void *addr)
{
	return -1;
}

static int ipw2200_net_change_mtu(struct net_device *dev, int new_mtu)
{
	return -1;
}

static int ipw2200_net_do_ioctl(struct net_device *dev, struct ifreq *ifr,
				int cmd)
{
	return -1;
}

#ifdef NOT_YET
static struct iw_statistics *ipw2200_net_get_wireless_stats(struct net_device
							    *dev)
{
	return NULL;
}
#endif
/* Interrupt stuff */

static irqreturn_t ipw2200_isr(int irq, void *dev_id, struct pt_regs *regs)
{
	struct ipw2200_private *ipw = (struct ipw2200_private *)dev_id;
	u32 inta;
	u32 inta_mask;
	u32 handled = 0;

	spin_lock(&ipw->lock);

	inta = readl(ipw->hw_base + CX2_INTA_RW);
	inta_mask = readl(ipw->hw_base + CX2_INTA_MASK_R);

	if (!ipw->interrupts_enabled || inta == CX2_INTA_NONE || inta == 0xFFFFFFFF) {
		spin_unlock(&ipw->lock);
		return IRQ_NONE;
	}

	IPW_DEBUG_ERROR("inta_mask 0x%08x\n", inta_mask);
	IPW_DEBUG_ERROR("inta 0x%08x\n", inta);

	inta &= (CX2_INTA_MASK_ALL & inta_mask);
	IPW_DEBUG_ERROR("inta masked 0x%08x\n", inta);

	/* tell the device to stop sending interrupts */
	writel(0, ipw->hw_base + CX2_INTA_MASK_R);

	/* ack current interrupts */
	writel(inta, ipw->hw_base + CX2_INTA_RW);

	/* handle all the justifications for the interrupt */
	if (inta & CX2_INTA_BIT_RX_TRANSFER) {
		schedule_work(&ipw->rx_transfer_work);
		handled &= CX2_INTA_BIT_RX_TRANSFER;
	}

	if (inta & CX2_INTA_BIT_TX_CMD_QUEUE) {
		schedule_work(&ipw->tx_cmd_queue_work);
		handled &= CX2_INTA_BIT_TX_CMD_QUEUE;
	}

	if (inta & CX2_INTA_BIT_TX_QUEUE_1) {
		schedule_work(&ipw->tx_queue_1_work);
		handled &= CX2_INTA_BIT_TX_QUEUE_1;
	}

	if (inta & CX2_INTA_BIT_TX_QUEUE_2) {
		schedule_work(&ipw->tx_queue_2_work);
		handled &= CX2_INTA_BIT_TX_QUEUE_2;
	}

	if (inta & CX2_INTA_BIT_TX_QUEUE_3) {
		schedule_work(&ipw->tx_queue_3_work);
		handled &= CX2_INTA_BIT_TX_QUEUE_3;
	}

	if (inta & CX2_INTA_BIT_TX_QUEUE_4) {
		schedule_work(&ipw->tx_queue_4_work);
		handled &= CX2_INTA_BIT_TX_QUEUE_4;
	}

	if (inta & CX2_INTA_BIT_STATUS_CHANGE) {
		schedule_work(&ipw->status_change_work);
		handled &= CX2_INTA_BIT_STATUS_CHANGE;
	}

	if (inta & CX2_INTA_BIT_BEACON_PERIOD_EXPIRED) {
		schedule_work(&ipw->beacon_period_expired_work);
		handled &= CX2_INTA_BIT_BEACON_PERIOD_EXPIRED;
	}

	if (inta & CX2_INTA_BIT_SLAVE_MODE_HOST_CMD_DONE) {
		schedule_work(&ipw->slave_mode_cmd_done_work);
		handled &= CX2_INTA_BIT_SLAVE_MODE_HOST_CMD_DONE;
	}

	if (inta & CX2_INTA_BIT_FW_INITIALIZATION_DONE) {
		schedule_work(&ipw->fw_init_done_work);
		handled &= CX2_INTA_BIT_FW_INITIALIZATION_DONE;
	}

	if (inta & CX2_INTA_BIT_FW_CARD_DISABLE_PHY_OFF_DONE) {
		schedule_work(&ipw->phy_off_done_work);
		handled &= CX2_INTA_BIT_FW_CARD_DISABLE_PHY_OFF_DONE;
	}

	if (inta & CX2_INTA_BIT_RF_KILL_DONE) {
		schedule_work(&ipw->rf_kill_done_work);
		handled &= CX2_INTA_BIT_RF_KILL_DONE;
	}

	if (inta & CX2_INTA_BIT_FATAL_ERROR) {
		IPW_DEBUG_ERROR("Fatal error\n");
		schedule_work(&ipw->fatal_error_work);
		handled &= CX2_INTA_BIT_FATAL_ERROR;
	}

	if (inta & CX2_INTA_BIT_PARITY_ERROR) {
		IPW_DEBUG_ERROR("Parity error\n");
		schedule_work(&ipw->parity_error_work);
		handled &= CX2_INTA_BIT_PARITY_ERROR;
	}

	spin_unlock(&ipw->lock);

	if (handled != inta) {
		IPW_DEBUG_ERROR("Unhandled INTA bits 0x%08x\n", inta & ~handled);
	}

	/* enable interrupts not handled on this device */
	writel(~handled & inta_mask, ipw->hw_base + CX2_INTA_MASK_R);

	return IRQ_HANDLED;
}

static int ipw2200_setup_deferred_work(struct ipw2200_private *priv)
{
	int ret = 0;

	/* initaly I'm coding all the defferd processing on the work queue */
	/* however; its easy to change to tasklet (if they don't sleep) */

	INIT_WORK(&priv->rx_transfer_work, do_rx_transfer, priv);
	INIT_WORK(&priv->tx_cmd_queue_work, do_tx_cmd_queue, priv);
	INIT_WORK(&priv->tx_queue_1_work, do_tx_queue_1, priv);
	INIT_WORK(&priv->tx_queue_2_work, do_tx_queue_2, priv);
	INIT_WORK(&priv->tx_queue_3_work, do_tx_queue_3, priv);
	INIT_WORK(&priv->tx_queue_4_work, do_tx_queue_4, priv);
	INIT_WORK(&priv->status_change_work, do_status_change, priv);
	INIT_WORK(&priv->beacon_period_expired_work, do_beacon_period_expired,
		  priv);
	INIT_WORK(&priv->slave_mode_cmd_done_work, do_slave_mode_cmd_done,
		  priv);
	INIT_WORK(&priv->fw_init_done_work, do_fw_init_done, priv);
	INIT_WORK(&priv->phy_off_done_work, do_phy_off_done, priv);
	INIT_WORK(&priv->rf_kill_done_work, do_rf_kill_done, priv);
	INIT_WORK(&priv->fatal_error_work, do_fatal_error, priv);
	INIT_WORK(&priv->parity_error_work, do_parity_error, priv);

	return ret;
}

/* PCI driver stuff */
static struct pci_device_id card_ids[] = {
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2701, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2702, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2711, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2712, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2721, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2722, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2731, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2732, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2741, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x103c, 0x2741, 0, 0, 0},/* very old pre-releas hw */
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2742, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2751, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2752, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2753, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2754, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2761, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x1043, 0x8086, 0x2762, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x104f, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
	{PCI_VENDOR_ID_INTEL, 0x4220, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0},
	
	/* required last entry */
	{0,}
};

MODULE_DEVICE_TABLE(pci, card_ids);

static int ipw2200_pci_probe(struct pci_dev *pdev,
			     const struct pci_device_id *ent)
{
	int err = 0;
	struct net_device *net_dev;
	void *base;
	u32 length, val;
	struct ipw2200_private *ipw_priv;

	net_dev = alloc_etherdev(sizeof(struct ipw2200_private));
	if (net_dev == NULL) {
		err = -ENOMEM;
		goto out;
	}

	ipw_priv = net_dev->priv;
	ipw_priv->net_dev = net_dev;
	ipw_priv->pci_dev = pdev;
#ifdef CONFIG_IPW_DEBUG
	ipw2200_debug_level = IPW_DL_ERROR | IPW_DL_WARNING | IPW_DL_INFO | IPW_DL_TRACE;
#else
	ipw2200_debug_level = 0;
#endif
	spin_lock_init(&ipw_priv->lock);

	if (pci_enable_device(pdev)) {
		err = -ENODEV;
		goto out_free_netdev;
	}

	pci_set_master(pdev);
	pci_set_drvdata(pdev, ipw_priv);

	err = pci_request_regions(pdev, DRV_NAME);
	if (err) 
		goto out_pci_disable_device;

	/* We disable the RETRY_TIMEOUT register (0x41) to keep 
	 * PCI Tx retries from interfering with C3 CPU state */
	pci_read_config_dword(pdev, 0x40, &val); 
	if ((val & 0x0000ff00) != 0) 
		pci_write_config_dword(pdev, 0x40, val & 0xffff00ff);
	
	length = pci_resource_len(pdev, 0);
	ipw_priv->hw_len = length;
	
	base = ioremap_nocache(pci_resource_start(pdev, 0), length);
	if (!base) {
		err = -ENODEV;
		goto out_pci_release_regions;
	}

	ipw_priv->hw_base = (unsigned long)base;
	IPW_DEBUG_ERROR("pci_resource_len = 0x%08x\n", length);
	IPW_DEBUG_ERROR("pci_resource_base = %p\n", base);

	pci_set_power_state(pdev, 0);
	pci_restore_state(pdev, NULL);

	/* call more HW inits here */

	ipw_priv->interrupts_enabled = 0;
	writel(0, ipw_priv->hw_base + CX2_INTA_MASK_R);
	if (ipw2200_setup_deferred_work(ipw_priv)) {
		err = -1; 	/* TODO: get a good error code */
		goto out_iounmap;
	}

	err = request_irq(pdev->irq, ipw2200_isr, SA_SHIRQ, DRV_NAME, ipw_priv);
	if (err) {
		IPW_DEBUG_ERROR("Error allocating IRQ %d\n", pdev->irq);
		goto out_pci_release_regions;
	}

	writel(~CX2_INTA_MASK_ALL, ipw_priv->hw_base + CX2_INTA_MASK_R);
	ipw_priv->interrupts_enabled = 0;

	ipw2200_load(ipw_priv);	/*TODO: check return code before shipping! just
				  debugging the hot plug while waiting on other bits to fall into place... */

	/* read in the eeprom data and use it to */
	/* further initialize the private data   */
	ipw_priv->eeprom_delay = 1;
	ipw2200_eeprom_read(ipw_priv);
	ipw2200_eeprom_parse_mac(ipw_priv,ipw_priv->net_dev->dev_addr);
	/* setup the net device call backs */

	SET_MODULE_OWNER(net_dev);
/*	SET_NETDEV_DEV(net_dev, net_dev->card->mdev->ofdev.dev);*/

	strcpy(net_dev->name, "eth%d");
	net_dev->open = ipw2200_net_open;
	net_dev->stop = ipw2200_net_stop;

	net_dev->hard_start_xmit = ipw2200_net_hard_start_xmit;
	net_dev->get_stats = NULL; /*ipw2200_net_get_stats;*/
	net_dev->set_multicast_list = ipw2200_net_set_multicast_list;
	net_dev->set_mac_address = ipw2200_net_set_mac_address;
	net_dev->change_mtu = ipw2200_net_change_mtu;
	net_dev->do_ioctl = ipw2200_net_do_ioctl;
	net_dev->get_wireless_stats = NULL; /*ipw2200_net_get_wireless_stats;*/
	net_dev->wireless_handlers = &ipw2200_wx_handler_def;

	if (register_netdev(net_dev)) {
		err = -ENODEV;
		goto out_release_irq;
	}

	ipw_priv->hw_id = *ent;

	device_create_file(&pdev->dev, &dev_attr_eeprom);
	device_create_file(&pdev->dev, &dev_attr_eeprom_delay);
	return 0;

 out_release_irq:
	free_irq(pdev->irq, ipw_priv);
 out_iounmap:
	iounmap((void *)ipw_priv->hw_base);
 out_pci_release_regions:
	pci_release_regions(pdev);
 out_pci_disable_device:
	pci_disable_device(pdev);
	pci_set_drvdata(pdev, NULL);
 out_free_netdev:
	free_netdev(ipw_priv->net_dev);
 out:
	return err;
}

static void ipw2200_pci_remove(struct pci_dev *pdev)
{
	struct ipw2200_private *ipw = pci_get_drvdata(pdev);
	if (!ipw)
		return;

	/* tell the device to stop sending interrupts */
	writel(0, ipw->hw_base + CX2_INTA_MASK_R);

	device_create_file(&pdev->dev, &dev_attr_eeprom_delay);	
	device_remove_file(&pdev->dev, &dev_attr_eeprom);
	free_irq(pdev->irq, ipw);
	iounmap((void *)ipw->hw_base);
	pci_release_regions(pdev);
	pci_disable_device(pdev);
	pci_set_drvdata(pdev, NULL);
	
	unregister_netdev(ipw->net_dev);
	free_netdev(ipw->net_dev);
}

static int ipw2200_pci_suspend(struct pci_dev *pdev, u32 state)
{
	int err = 0;

	return err;
}

static int ipw2200_pci_resume(struct pci_dev *pdev)
{
	int err = 0;

	return err;
}

/* driver initialization stuff */
static struct pci_driver ipw2200_driver = {
	.name = DRV_NAME,
	.id_table = card_ids,
	.probe = ipw2200_pci_probe,
	.remove = __devexit_p(ipw2200_pci_remove),
	.suspend = ipw2200_pci_suspend,
	.resume = ipw2200_pci_resume,
};

static int __init ipw2200_init(void)
{
	int ret;

	printk(KERN_INFO "load address :%p\n", ipw2200_init);
	printk(KERN_INFO DRV_NAME ": %s, %s\n", DRV_DESCRIPTION, DRV_VERSION);
	printk(KERN_INFO DRV_NAME ":%s\n", DRV_COPYRIGHT);

	ret = pci_module_init(&ipw2200_driver);
	if (ret) {
		IPW_DEBUG_ERROR("Unable to initialize PCI module\n");
		return ret;
	}

	ret = driver_create_file(&ipw2200_driver.driver, 
				 &driver_attr_debug_level);
	if (ret) {
		IPW_DEBUG_ERROR("Unable to create driver sysfs file\n");
		pci_unregister_driver(&ipw2200_driver);
		return ret;
	}

	return ret;
}

static void __exit ipw2200_exit(void)
{
	driver_remove_file(&ipw2200_driver.driver, &driver_attr_debug_level);
	pci_unregister_driver(&ipw2200_driver);
}

module_exit(ipw2200_exit);
module_init(ipw2200_init);
