/******************************************************************************
  
  Copyright(c) 2003 - 2004 Intel Corporation. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of version 2 of the GNU General Public License as 
  published by the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  James P. Ketrenos <ipw2100-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

******************************************************************************/

/*
 * This file contains the declarations for the RX/TX ring buffer / DMA
 * processing that the driver needs for working with the firmware.
 *
 * mgross@linux.intel.com
 */

/* shared memory defines */

struct cyclic_queue
{
	u32 head_index;
	u32 tail_index;
	u32 packet_count;
	u32 size;
};

struct shared_mem_desc
{
	void *virtual_address;
	u32 physical_address;
	u32 length;
	spinlock_t direct_memory_lock;
	spinlock_t indirect_memory_lock;
};


struct shared_mem_buffer
{
	struct shared_mem_desc mem_orig; 
	struct shared_mem_desc mem;
	u32 cached;
};

struct shared_mem_buffer_pool
{
	struct shared_mem_buffer buffer;
	struct shared_mem_buffer *buffer_array;
	u32 buffer_array_length;
};



/*  TX QUEUE defines */

struct txrx_control_flags
{
	u8 message_type;
	u8 rx_seq_num;
	u8 control_bits;
	u8 reserved;
};

struct host_command_desc 
{
	u8 host_cmd_index;
	u8 host_cmd_lenght;
	u16 reserved;
};

struct tfd_host_cmd_header
{
	struct txrx_control_flags control_flags;
	struct host_command_desc host_cmd_desc;
};


#define  TFD_SIZE                       128
#define  RFD_SIZE                       4

#define  NUMBER_TFD_FRAME_QUEUES        4
#define  NUMBER_TX_QUEUES               5

#define  NUM_TFD_CHUNKS                          6
#define  TFD_CMD_IMMEDIATE_PAYLOAD_LENGTH        (TFD_SIZE - sizeof(struct txrx_control_flags)) //108 bytes

#define  RFDS_BUFFER_LEN                32
#define RFD_BUFFER_NUM          ( RFDS_BUFFER_LEN * 4 )
#define TX_QUEUE_SIZE       (1<<5)

struct tfd_command
{
	struct tfd_host_cmd_header cmd_header;
	u8 tfd_cmd_payload[TFD_CMD_IMMEDIATE_PAYLOAD_LENGTH];
};

#define DCT_WEP_KEY_FIELD_LENGTH 16
struct dcmd_tx
{  
	u8 cmd_id;
	u8 seq_num;    
	u16 length;  
	u8 priority;
	u8 tx_flags;
	u8 tx_flags_ext;
	u8 key_index;
	u8 wepkey[DCT_WEP_KEY_FIELD_LENGTH];
	u8 rate;
	u8 antenna;

	u16 next_packet_duration;
	u16 next_frag_len;  

	u16 txop;
	u8 retrylimit;
	u16 cwcurrent;  
	u8 reserved3;
};

struct tx_frame_header
{
	struct txrx_control_flags control_flags;
	u32 work_area_ptr;
	u8 station_number;
	u8 reserved1;
	u8 reserved2;
	struct dcmd_tx dino_params; 
};

#define  MACADRR_BYTE_LEN 6

struct machdr32
{
	u8 ctrl1;
	u8 ctrl2;
	u16 duration;     // watch out for endians!
	u8 addr1[ MACADRR_BYTE_LEN ];
	u8 addr2[ MACADRR_BYTE_LEN ];
	u8 addr3[ MACADRR_BYTE_LEN ];
	u16 seq_ctrl;     // more endians!
	u8 addr4[ MACADRR_BYTE_LEN ];
	u16 qos_ctrl;
} machdr32;

struct machdr30
{
	u8 ctrl1;
	u8 ctrl2;
	u16 duration;     // watch out for endians!
	u8 addr1[ MACADRR_BYTE_LEN ];
	u8 addr2[ MACADRR_BYTE_LEN ];
	u8 addr3[ MACADRR_BYTE_LEN ];
	u16 seq_ctrl;     // more endians!
	u8 addr4[ MACADRR_BYTE_LEN ];
} machdr30;


struct machdr26
{
	u8 ctrl1;
	u8 ctrl2;
	u16 duration;     // watch out for endians!
	u8 addr1[ MACADRR_BYTE_LEN ];
	u8 addr2[ MACADRR_BYTE_LEN ];
	u8 addr3[ MACADRR_BYTE_LEN ];
	u16 seq_ctrl;     // more endians!
	u16 qos_ctrl;
} machdr26;



struct machdr24
{
	u8 ctrl1;
	u8 ctrl2;
	u16 duration;     // watch out for endians!
	u8 addr1[ MACADRR_BYTE_LEN ];
	u8 addr2[ MACADRR_BYTE_LEN ];
	u8 addr3[ MACADRR_BYTE_LEN ];
	u16 seq_ctrl;     // more endians!
} machdr24;



// TX TFD with 32 byte MAC Header
struct tx_tfd_32
{   
	struct machdr32    mchdr;                      // 32
	u32                uivplaceholder[2];          // 8
};

// TX TFD with 30 byte MAC Header
struct tx_tfd_30
{
	struct machdr30    mchdr;                      // 30
	u8                 reserved[2];                // 2
	u32                uivplaceholder[2];          // 8
};

// tx tfd with 26 byte mac header
struct tx_tfd_26
{
	struct machdr26    mchdr;                      // 26
	u8                 reserved1[2];               // 2
	u32                uivplaceholder[2];          // 8
	u8                 reserved2[4];               // 4
};

// tx tfd with 24 byte mac header
struct tx_tfd_24
{
	struct machdr24    mchdr;                      // 24
	u32                uivplaceholder[2];          // 8
	u8                 reserved[8];                // 8
};


struct tx_tfd
{
	struct tx_frame_header tf_header;
	union
	{
		struct tx_tfd_24 tfd_24;
		struct tx_tfd_26 tfd_26;
		struct tx_tfd_30 tfd_30;
		struct tx_tfd_32 tfd_32;
	} tfd;
};



struct tfd_frame
{
	struct tx_tfd tfd;
	u32 num_chuncks;
	u32 chunk_prt[NUM_TFD_CHUNKS];
	u16 chunk_len[NUM_TFD_CHUNKS];
};

union transmit_frame_descripter
{
	struct tfd_frame data;
	struct tfd_command cmd;
};


typedef enum {
	TX_QUEUE_CMD            = 0,
	TX_QUEUE_0              = 1,
	TX_QUEUE_1              = 2,
	TX_QUEUE_2              = 3,
	TX_QUEUE_3              = 4,
	TX_QUEUE_MAX            = 5
} TX_QUEUE_ID;

struct tx_queue
{
	TX_QUEUE_ID			    identifier;
//palon_context           paloncontext;
	struct cyclic_queue           *pqueue;
	struct shared_mem_buffer       queuebuffer;
	struct shared_mem_buffer      *pheadbufferarray;
	struct shared_mem_buffer      *ptailbufferarray;
	struct shared_mem_buffer      *pstagingbufferarray;
//pndis_packet*		    ppacketarray;
	struct shared_mem_desc              *pmmiocontrol;
//pmapped_serialized_info pmappedserializedinfo;
//pscatter_gather_list	pserializedsgl;
	spinlock_t                     txspinlock;
	u32                            watchdogcounter;
};

/*  RX QUEUE defines */


struct rx_queue 
{
	spinlock_t		    rxspinlock;
	struct cyclic_queue         *phwqueue;
	struct cyclic_queue         *pswqueue;
	struct shared_mem_desc        *pmmiocontrol;
	struct shared_mem_buffer_pool	bufferspool;
	struct shared_mem_desc	    *memdescarray;  
	struct shared_mem_desc	    *hwmemdescptrs[RFDS_BUFFER_LEN]; 
	struct shared_mem_desc	    *swmemdescptrs[RFD_BUFFER_NUM];  
};

