/*******************************************************************************
  
  Copyright(c) 2003 - 2004 Intel Corporation. All rights reserved.

  Portions of this file are based on the sample_* files provided by Wireless
  Extensions 0.26 package and copyright (c) 1997-2003 Jean Tourrilhes 
  <jt@hpl.hp.com>

  Portions of this file are based on the Host AP project, 
  Copyright (c) 2001-2002, SSH Communications Security Corp and Jouni Malinen
    <jkmaline@cc.hut.fi>
  Copyright (c) 2002-2003, Jouni Malinen <jkmaline@cc.hut.fi>

  This program is free software; you can redistribute it and/or modify it 
  under the terms of version 2 of the GNU General Public License as 
  published by the Free Software Foundation.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information:
  James P. Ketrenos <ipw2200-admin@linux.intel.com>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

******************************************************************************/

/*
 * This file defines the Wireless Extension handlers.  It does not
 * define any methods of hardware manipulation and relies on the
 * functions defined in ipw2200_main to provide the HW interaction.
 * 
 * The exception to this is the use of the ipw2200_get_ordinal() 
 * function used to poll the hardware vs. making unecessary calls.
 *
 */

#include "ipw2200.h"

static int ipw2200_wx_get_name(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{
	strcpy(wrqu->name, "IEEE 802.11bg");
	IPW_DEBUG_WX("Name: %s\n", wrqu->name);
	return 0;
}


static int ipw2200_wx_set_freq(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra) 
{
	return -EOPNOTSUPP;
}


static int ipw2200_wx_get_freq(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{
	return -EOPNOTSUPP;
}

static int ipw2200_wx_set_mode(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{
	return -EOPNOTSUPP;
}

static int ipw2200_wx_get_mode(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{
	/* We currently only support managed mode */
	wrqu->mode = IW_MODE_MONITOR;
	return 0;
}

static int ipw2200_wx_get_range(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{
	return -EOPNOTSUPP;
}

static int ipw2200_wx_set_wap(struct net_device *dev, 
			      struct iw_request_info *info, 
			      union iwreq_data *wrqu, char *extra)
{
	return -EOPNOTSUPP;
}

static int ipw2200_wx_get_wap(struct net_device *dev, 
			      struct iw_request_info *info, 
			      union iwreq_data *wrqu, char *extra)
{
	return -EOPNOTSUPP;
}

static int ipw2200_wx_set_essid(struct net_device *dev, 
				struct iw_request_info *info, 
				union iwreq_data *wrqu, char *extra)
{
	return -EOPNOTSUPP;
}

static int ipw2200_wx_get_essid(struct net_device *dev, 
				struct iw_request_info *info, 
				union iwreq_data *wrqu, char *extra)
{
	return  -EOPNOTSUPP;
}

static int ipw2200_wx_set_nick(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_get_nick(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_set_rate(struct net_device *dev,
			       struct iw_request_info *info,
			       union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}



static int ipw2200_wx_get_rate(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_set_rts(struct net_device *dev, 
			      struct iw_request_info *info, 
			      union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_get_rts(struct net_device *dev, 
			      struct iw_request_info *info, 
			      union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_set_txpow(struct net_device *dev, 
				struct iw_request_info *info, 
				union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_get_txpow(struct net_device *dev, 
				struct iw_request_info *info, 
				union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_set_frag(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_get_frag(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_set_retry(struct net_device *dev, 
				struct iw_request_info *info, 
				union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_get_retry(struct net_device *dev, 
				struct iw_request_info *info, 
				union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_set_scan(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{
	return -EOPNOTSUPP;
}

static int ipw2200_wx_get_scan(struct net_device *dev, 
			       struct iw_request_info *info, 
			       union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}

static int ipw2200_wx_set_encode(struct net_device *dev, 
				 struct iw_request_info *info, 
				 union iwreq_data *wrqu, char *key)
{
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_get_encode(struct net_device *dev, 
				 struct iw_request_info *info, 
				 union iwreq_data *wrqu, char *key)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_set_power(struct net_device *dev, 
			        struct iw_request_info *info, 
			        union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}

static int ipw2200_wx_get_power(struct net_device *dev, 
			        struct iw_request_info *info, 
			        union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}

#ifdef CONFIG_IPW2200_PROMISC
static int ipw2200_wx_set_promisc(struct net_device *dev, 
				  struct iw_request_info *info, 
				  union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}


static int ipw2200_wx_reset(struct net_device *dev, 
			    struct iw_request_info *info, 
			    union iwreq_data *wrqu, char *extra)
{ 
	return -EOPNOTSUPP; 
}
#endif // CONFIG_IPW2200_PROMISC

static iw_handler ipw2200_wx_handlers[] =
{
        NULL,                     /* SIOCSIWCOMMIT */
        ipw2200_wx_get_name,      /* SIOCGIWNAME */
        NULL,                     /* SIOCSIWNWID */
        NULL,                     /* SIOCGIWNWID */
        ipw2200_wx_set_freq,      /* SIOCSIWFREQ */
        ipw2200_wx_get_freq,      /* SIOCGIWFREQ */
        ipw2200_wx_set_mode,      /* SIOCSIWMODE */
        ipw2200_wx_get_mode,      /* SIOCGIWMODE */
        NULL,                     /* SIOCSIWSENS */
        NULL,                     /* SIOCGIWSENS */
        NULL,                     /* SIOCSIWRANGE */
        ipw2200_wx_get_range,     /* SIOCGIWRANGE */
        NULL,                     /* SIOCSIWPRIV */
        NULL,                     /* SIOCGIWPRIV */
        NULL,                     /* SIOCSIWSTATS */
        NULL,                     /* SIOCGIWSTATS */
        NULL,                     /* SIOCSIWSPY */
        NULL,                     /* SIOCGIWSPY */
        NULL,                     /* SIOCGIWTHRSPY */
        NULL,                     /* SIOCWIWTHRSPY */
        ipw2200_wx_set_wap,       /* SIOCSIWAP */
        ipw2200_wx_get_wap,       /* SIOCGIWAP */
        NULL,                     /* -- hole -- */
        NULL,                     /* SIOCGIWAPLIST -- depricated */
        ipw2200_wx_set_scan,      /* SIOCSIWSCAN */
        ipw2200_wx_get_scan,      /* SIOCGIWSCAN */
        ipw2200_wx_set_essid,     /* SIOCSIWESSID */
        ipw2200_wx_get_essid,     /* SIOCGIWESSID */
        ipw2200_wx_set_nick,      /* SIOCSIWNICKN */
        ipw2200_wx_get_nick,      /* SIOCGIWNICKN */
        NULL,                     /* -- hole -- */
        NULL,                     /* -- hole -- */
        ipw2200_wx_set_rate,      /* SIOCSIWRATE */
        ipw2200_wx_get_rate,      /* SIOCGIWRATE */
        ipw2200_wx_set_rts,       /* SIOCSIWRTS */
        ipw2200_wx_get_rts,       /* SIOCGIWRTS */
        ipw2200_wx_set_frag,      /* SIOCSIWFRAG */
        ipw2200_wx_get_frag,      /* SIOCGIWFRAG */
        ipw2200_wx_set_txpow,     /* SIOCSIWTXPOW */
        ipw2200_wx_get_txpow,     /* SIOCGIWTXPOW */
        ipw2200_wx_set_retry,     /* SIOCSIWRETRY */
        ipw2200_wx_get_retry,     /* SIOCGIWRETRY */
        ipw2200_wx_set_encode,    /* SIOCSIWENCODE */
        ipw2200_wx_get_encode,    /* SIOCGIWENCODE */
        ipw2200_wx_set_power,     /* SIOCSIWPOWER */
        ipw2200_wx_get_power,     /* SIOCGIWPOWER */
};

#ifdef CONFIG_IPW2200_PROMISC

static const struct iw_priv_args ipw2200_private_args[] = { 
	{
		SIOCIWFIRSTPRIV + 0x0, 
		IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 2, 0, "monitor" 
	}, 
	{
		SIOCIWFIRSTPRIV + 0x1, 
		IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 0, 0, "reset" 
	},
};

static iw_handler ipw2200_private_handler[] = {
	ipw2200_wx_set_promisc,  /* SIOCIWFIRSTPRIV */
	ipw2200_wx_reset,        /* SIOCIWFIRSTPRIV + 1*/ 
};

#endif /* CONFIG_IPW2200_PROMISC */

struct iw_handler_def ipw2200_wx_handler_def = 
{
	.standard = ipw2200_wx_handlers,
	.num_standard = sizeof(ipw2200_wx_handlers) / sizeof(iw_handler),
#ifdef CONFIG_IPW2200_PROMISC
	.num_private = sizeof(ipw2200_private_handler) / sizeof(iw_handler),
 	.num_private_args = sizeof(ipw2200_private_args) /
	sizeof(struct iw_priv_args),
	.private = (iw_handler *)ipw2200_private_handler,
	.private_args = (struct iw_priv_args *)ipw2200_private_args,	
#else
	.num_private = 0,
	.num_private_args = 0,
	.private = NULL,
 	.private_args = NULL,
#endif /* CONFIG_IPW2200_PROMISC */
};

struct iw_statistics* ipw2200_get_wireless_stats(struct net_device *dev)
{
	struct ipw2200_private * priv = dev->priv;
	return &priv->wstats;
}


