/*
 * iSCSI connection daemon
 * Copyright (C) 2002 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: init.c,v 1.2.2.11 2004/01/16 12:11:10 naveenb Exp $
 *
 */

#include <unistd.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <sys/mount.h>
#include <scsi/scsi.h>
#include <linux/unistd.h>
#include <errno.h>
#include <dirent.h>

#include "iscsi-ioctl.h"

#define BINDING_FILE "/sysroot/var/lib/iscsi/bindings"

static inline _syscall2(int, pivot_root, const char *, one, const char *, two);
extern void establish_iscsi_network_boot_session(int bus_id, int target_id,
						 struct sapiNBP *nbp);
extern void get_inbp_info(struct sapiNBP *nbp);
extern int bring_up_network(struct sapiNBP *nbp);
extern void set_iscsi_network_boot(void);

static int bid, tid;
static int varpart = 0, rootpart = 0;	/* default value 0 indicate complete 
					 * disk instead of a partition.
					 */

struct scsi_idlun {
	int dev_id;
	int host_unique_id;
};

struct device_info {
	int bus;
	int target;
	int lun;
	int part;
};

int
scan_cmdline(void)
{
	FILE *f;
	char line[512], *res, *rbuf, *cp, *tmp = NULL;
	char buf[256];

	f = fopen("/proc/cmdline", "r");
	if (!f) {
		printf("Cannot open control path to the driver");
		exit(1);
	}

	rbuf = fgets(line, sizeof (buf), f);

	cp = strtok(rbuf, " ");
	do {
		sscanf(cp, "%s", buf);
		res = strchr(buf, '=');
		if (res) {
			*res = '\0';
			tmp = res + 1;
		}
		if (strcasecmp(buf, "varpart") == 0) {
			varpart = atoi(tmp);
		}
		if (strcasecmp(buf, "rootpart") == 0) {
			rootpart = atoi(tmp);
		}
		cp = strtok(NULL, " ");
	} while (cp);

	fclose(f);
	return 0;
}

void
read_sysfs_file(char *sysfs_file, char *buf)
{
	int fd;
	
	fd = open(sysfs_file, O_RDONLY);
	if (fd < 0)
		printf("error in opening %s file\n", sysfs_file);

	read(fd, buf, sizeof (buf));
	
	close(fd);
}

void
write_sysfs_file(char *sysfs_file, char *cmd)
{
	int fd;

	fd = open(sysfs_file, O_WRONLY);
	if (fd < 0)
		printf("error in opening %s file\n", sysfs_file);

	write(fd, cmd, strlen(cmd));
	write(fd, "\n", 1);

	close(fd);
}

void
generate_rootdev(int *rdev_id, struct device_info *info, char *devname)
{
	struct scsi_idlun idlun;
	struct stat s;
	char fname[64];
	char buf[8], ebuf[256], hbuf[3];
	int fd, in, r, i, ret, hid;

	read_sysfs_file("/sys/class/scsi_host/iscsi/host_no", hbuf);
	hid = atoi(hbuf);
	
	for (i = 0; i <= 26; i++) {
		strcpy(fname, "/dev/sd");
		if (info->part != 0)
			sprintf(buf, "%c%d", ('a' + (char) i), info->part);
		else
			sprintf(buf, "%c", ('a' + (char) i));

		strcat(fname, buf);

		fd = open(fname, O_RDWR);
		if (fd < 0) {
			if (errno == EBUSY) {
				printf("%s : device busy\n", fname);
				continue;
			} else
				break;
		}

		r = ioctl(fd, SCSI_IOCTL_GET_IDLUN, &idlun);
		if (r < 0) {
			sprintf(ebuf, "device %s failed on scsi ioctl, skip",
				fname);
			perror(ebuf);
			continue;
		}
		if (((idlun.dev_id >> 24 & 0xff) == hid) &&
		    ((idlun.dev_id >> 16 & 0xff) == info->bus) &&
		    ((idlun.dev_id & 0xff) == info->target) &&
		    ((idlun.dev_id >> 8 & 0xff) == info->lun)) {
			in = open(fname, O_RDONLY);
			ret = fstat(in, &s);
			*rdev_id = s.st_rdev;
			if (devname)
				strcpy(devname, fname);
			close(in);
			break;
		}
	}
	if (fd)
		close(fd);
}

int
mkdevices(char *prefix)
{
	int fd;
	char buf[32768];
	int i;
	char *start, *chptr;
	int major, minor;
	char old;
	char devName[128];

	if ((fd = open("/proc/partitions", O_RDONLY)) < 0) {
		printf("failed to open /proc/partitions: %d\n", errno);
		return 1;
	}

	i = read(fd, buf, sizeof (buf));
	if (i < 1) {
		close(fd);
		printf("failed to read /proc/partitions: %d\n", errno);
		return 1;
	}
	buf[i] = '\0';
	close(fd);

	start = strchr(buf, '\n');
	if (start) {
		start++;
		start = strchr(buf, '\n');
	}
	if (!start)
		return 1;

	start = start + 1;
	while (*start) {
		while (*start && isspace(*start))
			start++;
		major = strtol(start, &chptr, 10);

		if (start != chptr) {
			start = chptr;
			while (*start && isspace(*start))
				start++;
			minor = strtol(start, &chptr, 10);

			if (start != chptr) {
				start = chptr;
				while (*start && isspace(*start))
					start++;
				while (*start && !isspace(*start))
					start++;
				while (*start && isspace(*start))
					start++;

				if (*start) {
					char *ptr, *deviceDir;
					int i;

					chptr = start;
					while (!isspace(*chptr))
						chptr++;
					old = *chptr;
					*chptr = '\0';

					sprintf(devName, "%s/%s", prefix,
						start);
					unlink(devName);

					ptr = devName;
					i = 0;
					while (*ptr)
						if (*ptr++ == '/')
							i++;
					if (i > 2) {
						deviceDir =
						    alloca(strlen(devName) + 1);
						strcpy(deviceDir, devName);
						ptr =
						    deviceDir +
						    (strlen(devName) - 1);
						while (*ptr != '/')
							*ptr-- = '\0';

						if (access(deviceDir, X_OK)
						    && mkdir(deviceDir, 0644)) {
							printf("cannot create "
							       "directory %s: "
							       "%d\n", 
							       deviceDir, 
							       errno);
						}
					}
					if (mknod(devName, S_IFBLK | 0600,
						  makedev(major, minor))) {
						printf("failed to create %s\n",
						       devName);
					}
					*chptr = old;
					start = chptr;
				}

			}
		}

		start = strchr(start, '\n');
		if (!*start)
			return 1;
		start = start + 1;
	}

	return 0;
}

void 
rm_udev_iscsi_entries(void)
{
	DIR *dp;
	struct dirent *d;
	char buf[64];
	
	if((dp = opendir("/sysroot/udev")) == NULL) {
		printf("unable to open /udev");
		return;
	}
	
	while((d = readdir(dp))) {
		if (strncmp(d->d_name, "iscsi", 5) == 0) {
			sprintf(buf, "%s%s", "/sysroot/udev/", d->d_name);
			unlink(buf);
		}
	}
	
	closedir(dp);
}

void
mkudevices(int lun)
{
	struct device_info info;
	char devname[64], buf[64];
	int rdev_id = 0;
	int i, l, fd;
	char tmp[64];
	struct stat s;
	
	info.bus = bid;
	info.target = tid;
	info.lun = lun;
	info.part = rootpart;
	
	memset(devname, 0, sizeof (devname));
	generate_rootdev(&rdev_id, &info, devname);
	
	if (rootpart == 0) {
		sprintf(tmp, "/sysroot/udev/iscsib%dt%dl%dsd", 
			info.bus, info.target, info.lun);
		unlink(tmp);
		if (mknod(tmp, S_IFBLK | 0660, rdev_id)) {
			printf("failed to create %s\n", tmp);
		}
	}
	else {
		sprintf(tmp, "%d", (char) rootpart);
		l = strlen(tmp);
		devname[strlen(devname) - l] = '\0';	
		
		for (i = 0; i < 16; i++) {
			if (i == 0)
				sprintf(buf, "%s", devname);
			else
				sprintf(buf, "%s%d", devname, i);
			
			fd = open(buf, O_RDONLY);
			if (fd < 0)
				continue;
			fstat(fd, &s);
			rdev_id = s.st_rdev;
			
			if (i == 0)
				sprintf(tmp, 
					"/sysroot/udev/iscsib%dt%dl%dsd",
					info.bus, info.target, info.lun);
			else 
				sprintf(tmp, 
					"/sysroot/udev/iscsib%dt%dl%dsd%d",
					info.bus, info.target, info.lun, i);
			unlink(tmp);
			if (mknod(tmp, S_IFBLK | 0660, rdev_id)) {
				printf("failed to create %s\n", tmp);
			}
			close(fd);
		}
	}
}


int
write_device_number(int rdev)
{
	FILE *f = NULL;
	char buf[10];

	f = fopen("/proc/sys/kernel/real-root-dev", "w");
	if (!f) {
		printf("Cannot open /proc/sys/kernel/real-root-dev\n");
		return -1;
	}
	sprintf(buf, "0x%x", rdev);
	fprintf(f, buf, strlen(buf));

	fclose(f);
	return 1;
}

void
get_bt_from_binding_file(char *tgt_name)
{
	FILE *f = NULL;
	char buffer[4096];
	char *line;
	unsigned int bus_id;
	unsigned int target_id;
	int b = -1, t = -1;
	int bus[ISCSI_MAX_CHANNELS_PER_HBA];
	int i;
	char *c;
	char *TargetName = NULL;
	char buf[256];

	memset(bus, -1, sizeof (bus));
	f = fopen(BINDING_FILE, "r+");

	if (!f)
		printf("Error in opening iSCSI target bindings file\n");

	if (f) {
		unsigned long line_nr = 0;

		while ((line = fgets(buffer, sizeof (buffer), f))) {
			c = line;

			/* strip comments */
			if ((c = (char *) strchr(line, '#'))) {
				*c = '\0';
			}

			line_nr++;

			/* each binding line should have at least: 
			 * bus_id target_id TargetName 
			 */

			c = (char *) strtok(line, "# \t\r\n");

			if (c == NULL)
				continue;	/* don't log, 
						 * it's just a blank line 
						 */

			if (sscanf(c, "%du", &bus_id) != 1) {
				printf("ignoring malformed line %lu of "
				       "bindings file,no bus id", line_nr);
				continue;
			}

			c = (char *) strtok(NULL, "# \t\r\n");
			if ((c == NULL) || 
			    (sscanf(c, "%du", &target_id) != 1)) {
				printf("ignoring malformed line %lu of "
				       "bindings file,no target id", line_nr);
				continue;
			}

			TargetName = (char *) strtok(NULL, "# \t\r\n");
			if (TargetName == NULL) {
				printf("ignoring malformed line %lu of "
				       "bindings file,no iSCSI TargetName", 
				       line_nr);
				continue;
			}

			if (strcmp(TargetName, tgt_name) == 0) {
				b = bus_id;
				t = target_id;
				break;
			}
			bus[bus_id] = target_id;
		}
	}
	if (b > -1 && t > -1) {
		bid = bus_id;
		tid = target_id;
	} else {
		memset(buf, 0, sizeof (buf));
		for (i = 0; i < ISCSI_MAX_CHANNELS_PER_HBA; i++) {
			if (bus[i] < ISCSI_MAX_CHANNELS_PER_HBA) {
				bid = i;
				tid = bus[i] + 1;
				break;
			}
		}
		sprintf(buf, "%d\t%d\t%s\n", bid, tid, tgt_name);
		fprintf(f, buf, strlen(buf));
	}
	fclose(f);
}

void
create_symlink(char *devname, int rootpart)
{
	char buf1[64];
	char buf2[256];
	int i, len;
	char *c, diskname[256];

	unlink("/sysroot/dev/inbpdisk");

	if (rootpart != 0) {
		len = strlen(devname);
		c = devname;
		c += strlen(devname);
		c--;
		i = atoi(c);
		
		for(;;) {
			if (i == rootpart) {
				memset(diskname, 0, sizeof(diskname));
				strncpy(diskname, devname, c-devname); 
				break;
			} else {
				c--;
				i = atoi(c);
			}
		}
	} else {
		memset(diskname, 0, sizeof(diskname));
		strcpy(diskname, devname);
	}
	
	symlink(diskname, "/sysroot/dev/inbpdisk");
	
	for (i = 1; i < 16; i++) {
		sprintf(buf1, "/sysroot/dev/inbpdisk%d", i);
		unlink(buf1);
		sprintf(buf2, "%s%d", diskname, i);
		symlink(buf2, buf1);
	}
}

void
generate_rootrc(char *rootfs, int lun)
{
	int rdev_id = 0;
	unsigned long mf = 0;
	char buf[256];
	struct device_info info;
	char devname[64];

	info.bus = bid;
	info.target = tid;
	info.lun = lun;
	info.part = rootpart;

	memset(devname, 0, sizeof (devname));
	generate_rootdev(&rdev_id, &info, devname);
	if (rdev_id) {
		write_device_number(rdev_id);
	}

	if (mknod("/dev/root", S_IFBLK | 0700, rdev_id))
		printf("mknod failed: %d\n", errno);

	write_device_number(256);

	mf = 0;
	mount("/dev/root", "/sysroot", rootfs, mf, NULL);

	sprintf(buf, "%d %d -", bid, tid);
	write_sysfs_file("/sys/class/scsi_host/iscsi/scan", buf);

	create_symlink(devname, rootpart);
	rm_udev_iscsi_entries();
	
	mkudevices(lun);
	umount("/sysroot");

	mf = 0;
	mf |= MS_RDONLY;
	mount("/dev/root", "/sysroot", rootfs, mf, NULL);

	if (pivot_root("/sysroot", "/sysroot/initrd"))
		printf("pivotroot: pivot_root(/sysroot, /sysroot/initrd) "
		       "failed: %d\n", errno);

	umount("/initrd/proc");
	umount("/initrd/sys");
}

void
get_target_bindings(char *rootfs, char *varfs, struct sapiNBP *nbp)
{
	int bus = 0;
	int target = 0;
	struct device_info info;
	int rdev_id;
	unsigned long mf = 0;
	char rootdevname[64];

	establish_iscsi_network_boot_session(bus, target, nbp);

	mkdevices("/dev");

	info.bus = bus;
	info.target = target;
	info.lun = nbp->slun;
	info.part = rootpart;

	memset(rootdevname, 0, sizeof (rootdevname));
	generate_rootdev(&rdev_id, &info, rootdevname);
	if (rdev_id)
		write_device_number(rdev_id);

	if (mknod("/dev/tmproot", S_IFBLK | 0700, rdev_id))
		printf("mknod failed: %d\n", errno);

	write_device_number(256);

	mf = 0;
	mount("/dev/tmproot", "/sysroot", rootfs, mf, NULL);

	if (varpart != rootpart) {
		int l;
		char vardevname[64];
		char tmp[64];

		sprintf(tmp, "%d", (char) rootpart);
		l = strlen(tmp);
		rootdevname[strlen(rootdevname) - l] = '\0';
		sprintf(vardevname, "%s", rootdevname);
		sprintf(tmp, "%d", (char) varpart);
		strcat(vardevname, tmp);

		if (access("/sysroot/var", F_OK) != 0)
			if (mkdir("/sysroot/var", 0755) < 0)
				printf("Error in creating var directory\n");

		if (mount(vardevname, "/sysroot/var", varfs, mf, NULL) < 0)
			printf("Error in mounting var partition\n");
	}

	get_bt_from_binding_file(nbp->targetstring);

	if (varpart != rootpart)
		umount("/sysroot/var");

	umount("/sysroot");

	write_sysfs_file("/sys/class/scsi_host/iscsi/shutdown", "");
}

int
main(int argc, char *argv[])
{
	char *rootfs = NULL;
	char *varfs = NULL;
	struct sapiNBP nbp;

	if (argc > 1)
		rootfs = argv[1];

	if (argc > 2)
		varfs = argv[2];

	scan_cmdline();

	get_inbp_info(&nbp);

	while(!bring_up_network(&nbp)) {
		printf("iSCSI: Error in bringing up the network interface\n");
	}
	
	get_target_bindings(rootfs, varfs, &nbp);

	set_iscsi_network_boot();

	establish_iscsi_network_boot_session(bid, tid, &nbp);

	mkdevices("/dev");

	generate_rootrc(rootfs, nbp.slun);

	return 0;
}
