#! /bin/sh
#
# Install the iSCSI driver

# root directory for the system we're installing for.
# May not be / for diskless clients.
if [ "$ROOT" ] ; then
    export ROOT
fi

# base dir relative to ROOT into which relocatable files should be installed
BASEDIR=${BASEDIR:=/}
export BASEDIR

# save the path to be used by the client
CLIENT_BASEDIR=$BASEDIR
export CLIENT_BASEDIR

# make the BASEDIR the full path
BASEDIR=${ROOT}${CLIENT_BASEDIR}
export BASEDIR

KVER=`uname -r`

# the Makefile passes the object dirs, put default them 
# in case someone calls the script by hand.
OBJDIR=${OBJDIR:=`uname`-`uname -m`/obj}
KOBJDIR=${KOBJDIR:=`pwd`}

if [ `whoami` != root ]; then
	echo "You must be root to install the iSCSI distribution"
	exit 1
fi


PATH="/usr/bin:/bin:/usr/sbin:/sbin:$PATH"
export PATH

#
# figure out where to install the init.d script and symlinks
#
INITDIR=
RCROOT=
RELDIR=
if [ -d ${ROOT}/etc/rc.d/init.d ] ; then
    INITDIR=${ROOT}/etc/rc.d/init.d
    if [ -d ${ROOT}/etc/rc.d/init.d/rc2.d ] ; then
	RCROOT=${ROOT}/etc/rc.d/init.d
	RELDIR=..
    elif [ -d ${ROOT}/etc/rc.d/rc2.d ] ; then
	RCROOT=${ROOT}/etc/rc.d
	RELDIR=../init.d
    elif [ -d ${ROOT}/etc/rc2.d ] ; then
	RCROOT=${ROOT}/etc
	RELDIR=../rc.d/init.d
    fi
elif [ -d ${ROOT}/etc/init.d ] ; then
    INITDIR=${ROOT}/etc/init.d
    if [ -d ${ROOT}/etc/init.d/rc2.d ]; then 
	RCROOT=${ROOT}/etc/init.d
	RELDIR=..
    elif [ -d ${ROOT}/etc/rc.d/rc2.d ] ; then
	RCROOT=${ROOT}/etc/rc.d
	RELDIR=../../init.d
    elif [ -d ${ROOT}/etc/rc2.d ] ; then
	RCROOT=${ROOT}/etc
	RELDIR=../init.d
    fi
fi

printf "Installing iSCSI driver for Linux $KVER\n\n"

# try to remove the module, in case one is installed and loaded already.
# we try to unload it here and in remove.sh, so that people updating drivers
# don't end up running a new daemon against an old driver.
rmmod iscsi > /dev/null 2>&1

# If old device links still persists, remove them forever.

if [ -d /dev/iscsi ]; then
    rm -rf /dev/iscsi
fi

# delete any existing iSCSI kernel modules, since we don't want
# modprobe loading the wrong one, and some distributions now ship
# older modules that we may need to replace.  Since the module subdir
# varies, use find to look in every module dir for this kernel.
find ${ROOT}/lib/modules/$KVER -name 'iscsi\.ko.*' -exec rm {} \;

# install files
if [ ! -d $BASEDIR/sbin ] ; then
    mkdir -p $BASEDIR/sbin
fi

install -D ${KOBJDIR}/iscsi.ko ${ROOT}/lib/modules/$KVER/misc/iscsi.ko 
install -b -s ${OBJDIR}/iscsid ${OBJDIR}/iscsi-device ${OBJDIR}/iscsi-iname ${OBJDIR}/iscsi-ls ${OBJDIR}/iscsi-id $BASEDIR/sbin
install -D -b -m 644 iscsid.8 $BASEDIR/man/man8/iscsid.8
install -D -b -m 644 iscsi.conf.5 $BASEDIR/man/man5/iscsi.conf.5
install -D -b -m 644 iscsi.bindings.5 $BASEDIR/man/man5/iscsi.bindings.5
install -D -b -m 644 iscsi-ls.1 $BASEDIR/man/man1/iscsi-ls.1

# update scripts with the proper BASEDIR
for util in iscsi-umountall iscsi-mountall 
do
    sed -e "s|^BASEDIR=.*|BASEDIR=$CLIENT_BASEDIR|" < $util > $BASEDIR/sbin/$util
    chmod 744 $BASEDIR/sbin/$util
done


if [ "$INITDIR" != "" ] ; then
    # update the BASEDIR in the rc file based on the BASEDIR we're actually installing to.
    sed -e "s|^BASEDIR=.*|BASEDIR=$CLIENT_BASEDIR|" < rc.iscsi > $INITDIR/iscsi
    chmod 744 $INITDIR/iscsi
    echo "The initialization script has been installed as $INITDIR/iscsi." | fmt -
    echo 
else
    # Couldn't find an init.d dir to put the script in
    INITDIR=${ROOT}/etc/rc.d
    if [ ! -f ${INITDIR}/iscsi ]; then
	echo "The initialization script has been installed as ${INITDIR}/iscsi."
	echo

	if [ ! -d $INITDIR ]; then 
	    mkdir -p $INITDIR
	fi

	# update the BASEDIR in the rc file based on the BASEDIR we're actually installing to.
	sed -e "s|^BASEDIR=.*|BASEDIR=$CLIENT_BASEDIR|" < rc.iscsi > $INITDIR/iscsi
	chmod 744 $INITDIR/iscsi
    fi
fi

if [ "$RCROOT" != "" ] ; then
    # usually make it start automatically at boot
    if [ ! -f ./.iscsi.no.autostart ] ; then
        if [ -f /etc/SuSE-release ]; then
            # For SuSE, insserv program provides both start/stop links
            # based on the information provided in rc.iscsi
            /sbin/insserv iscsi
        else
            ln -sf $RELDIR/iscsi $RCROOT/rc2.d/S11iscsi
            ln -sf $RELDIR/iscsi $RCROOT/rc3.d/S11iscsi
            ln -sf $RELDIR/iscsi $RCROOT/rc4.d/S11iscsi
            ln -sf $RELDIR/iscsi $RCROOT/rc5.d/S11iscsi
        fi
	echo "iSCSI has been set up to run automatically when you reboot."
	echo
    fi
    # always make is stop automatically
    if [ ! -f /etc/SuSE-release ]; then
        ln -sf $RELDIR/iscsi $RCROOT/rc0.d/K20iscsi
        ln -sf $RELDIR/iscsi $RCROOT/rc1.d/K20iscsi
        ln -sf $RELDIR/iscsi $RCROOT/rc6.d/K20iscsi
    fi

else
    echo "You will need to edit your rc files to run"
    echo "$INITDIR/iscsi when you reboot."
    echo 
fi

# iSCSI requires a unique InitiatorName.  Since it may be used for
# access control, it should never change once created.  We store it in
# a separate file, and check for it in the iSCSI daemon.  
if [ "$ROOT" ] ; then
    if ! grep -q "^GenerateName=yes" /etc/initiatorname.iscsi 2> /dev/null ; then
	# the installer is running on a different machine than
	# the one that needs the file.
	# the init.d file will create it on that machine later.
	echo "GenerateName=yes" > ${ROOT}/etc/initiatorname.iscsi
	chmod 600 ${ROOT}/etc/initiatorname.iscsi
    fi
else
    if ! grep -q "^InitiatorName=[^ \t\n]" /etc/initiatorname.iscsi 2>/dev/null ; then
	# Make sure we have the iname utility
	if [ ! -x $BASEDIR/sbin/iscsi-iname ] ; then
	    echo "Error: $BASEDIR/sbin/iscsi-iname does not exist, driver not successfully installed"
	    exit 1;
	fi 
	# Generate a unique InitiatorName and save it
	INAME=`$BASEDIR/sbin/iscsi-iname`
	if [ "$INAME" != "" ] ; then
	    echo "## DO NOT EDIT OR REMOVE THIS FILE!" > /etc/initiatorname.iscsi
	    echo "## If you remove this file, the iSCSI daemon will not start." >> /etc/initiatorname.iscsi
	    echo "## If you change the InitiatorName, existing access control lists" >> /etc/initiatorname.iscsi
	    echo "## may reject this initiator.  The InitiatorName must be unique">> /etc/initiatorname.iscsi
	    echo "## for each iSCSI initiator.  Do NOT duplicate iSCSI InitiatorNames." >> /etc/initiatorname.iscsi
	    printf "InitiatorName=$INAME\n"  >> /etc/initiatorname.iscsi
	    chmod 600 /etc/initiatorname.iscsi
	    printf "InitiatorName $INAME was generated and written to /etc/initiatorname.iscsi.\n\n" | fmt -
	else
	    echo "Error: couldn't generate an iSCSI InitiatorName"
	    exit 1;
	fi
    else
	INAME=`awk -F= '/^InitiatorName=/ { print $2} ' < /etc/initiatorname.iscsi`
        printf "InitiatorName $INAME has already been generated and written to /etc/initiatorname.iscsi.\n\n" | fmt -
    fi
fi

config_file="iscsi.conf"

if [ -e ${ROOT}/etc/iscsi.conf ]; then
    egrep -q -v '^[ \t]*([#].*)?$' ${ROOT}/etc/iscsi.conf
    if [ $? -eq 1 ] ; then
	# the existing config file has only whitespace and comments, replace it
	echo "Reinstalling configuration file ${ROOT}/etc/iscsi.conf";
	rm -f ${ROOT}/etc/iscsi.conf
	install -m 600 iscsi.conf ${ROOT}/etc
	echo "Make sure you check and edit the ${ROOT}/etc/iscsi.conf file!"
    else
	if [ -e ./iscsi-version.h ]; then
	    # don't overwrite the existing config, just make a new one for the user to compare to
	    major=`grep DRIVER_MAJOR_VERSION iscsi-version.h |awk -F " " '{print $3}'`
	    minor=`grep DRIVER_MINOR_VERSION iscsi-version.h |awk -F " " '{print $3}'`
	    patch=`grep DRIVER_PATCH_VERSION iscsi-version.h |awk -F " " '{print $3}'`
	    internal=`grep DRIVER_INTERNAL_VERSION iscsi-version.h |awk -F " " '{print $3}'`
	    if [ $major ]; then
		config_file="${config_file}.$major"
	    fi
	    if [ $minor ]; then
		config_file="${config_file}.$minor"
	    fi
	    if [ $patch ]; then
		config_file="${config_file}.$patch"
	    fi
	    if [ $internal ]; then
		config_file="${config_file}.$internal"
	    fi
	else
	    config_file="${config_file}.new" 
	fi

	install -m 600 iscsi.conf ${ROOT}/etc/${config_file}
	chmod 600 ${ROOT}/etc/${config_file}
	echo "The config file ${ROOT}/etc/iscsi.conf already exists."
	echo "A new config file ${ROOT}/etc/${config_file} was installed."
	echo "Check this file for any new options you may wish to use."
	echo
    fi
else
    install -m 600 iscsi.conf ${ROOT}/etc
    echo "Make sure you check and edit the ${ROOT}/etc/iscsi.conf file!"
    echo
fi

# Udev package needs to be installed for installation to progress.

if [ ! -f /etc/udev/udev.rules ]; then
    echo "Udev package need to be installed for devices to be created"
    echo "Aborting iSCSI installtion process"
    exit 2
fi


# Fill up the iSCSI specific entry for device creation in /etc/udev/udev.rules
# if it does not exist already 

present=0
while read rule bus program name
do
    case $rule
    in
    \#*) continue ;; #  ignore comments
    '')  continue ;; # ignore empty lines
    esac

    BUS=`echo $bus |awk -F= '/BUS=/ { print $2} '`
    PROGRAM=`echo $program|awk -F= '/PROGRAM=/ {print $2} '`

    if [ $rule == "CALLOUT," ] && [ $BUS == "\"scsi\"," ] && [ $PROGRAM == "\"/sbin/iscsi-id\"," ]; then
        present=1
        break
    fi

done < /etc/udev/udev.rules

if [ $present -eq 0 ]; then
    echo "CALLOUT, BUS=\"scsi\", PROGRAM=\"/sbin/iscsi-id\", ID=\"*\", NAME=\"%c%D\"" >> /etc/udev/udev.rules
fi

# /etc/fstab.iscsi needs to contain an entry to automount sysfs filesystem
# if it is not already present.

found=0
while read dev mountp fstype options dummy1 dummy2
do
    case $dev
    in
    \#*) continue ;; #  ignore comments
    '')  continue ;; # ignore empty lines
    esac
    
    if [ $mountp = "/sys" ]; then
         found=1
         break
    fi 
done < /etc/fstab

if [ $found -eq 0 ]; then
    mkdir -p /sys
    echo "none                    /sys                   sysfs    defaults        0 0" >> /etc/fstab
fi

if [ ! -f /etc/fstab.iscsi ]; then
    echo "# /etc/fstab.iscsi file for filesystems built on iscsi devices." > /etc/fstab.iscsi
    echo "#" >> /etc/fstab.iscsi
    echo "# A typical entry here would look like:" >> /etc/fstab.iscsi
    echo "# /udev/iscsib0t0l0disk /mnt ext2 defaults 0 0" >> /etc/fstab.iscsi
    echo "#" >> /etc/fstab.iscsi
    echo "# Where /udev/iscsib0t0l0disk is an iscsi device" >> /etc/fstab.iscsi
    echo "#" >> /etc/fstab.iscsi
    echo "# See fstab(5) for further details on configuring devices." >> /etc/fstab.iscsi 
    echo "#" >> /etc/fstab.iscsi
else
    count=0
    while read line
    do
        echo $line |grep udev > /dev/null
        if [ $? -eq 0 ]; then
           count=1
           break;
        fi
    done < /etc/fstab.iscsi

    if [ $count -eq 0 ]; then
        echo "# /etc/fstab.iscsi file for filesystems built on iscsi devices." >> /etc/fstab.iscsi.tmp
        echo "#" >> /etc/fstab.iscsi.tmp
        echo "# A typical entry here would look like:" >> /etc/fstab.iscsi.tmp
        echo "# /udev/iscsib0t0l0disk /mnt ext2 defaults 0 0" >> /etc/fstab.iscsi.tmp
        echo "#" >> /etc/fstab.iscsi.tmp
        echo "# Where /udev/iscsib0t0l0disk is an iscsi device" >> /etc/fstab.iscsi.tmp
        echo "#" >> /etc/fstab.iscsi.tmp
        echo "# See fstab(5) for further details on configuring devices." >> /etc/fstab.iscsi.tmp
        echo "#" >> /etc/fstab.iscsi.tmp
        echo " " >> /etc/fstab.iscsi.tmp
        while read check line
        do
            case $check
            in
             \#*) continue ;; #  ignore comments
              '')  continue ;; # ignore empty lines
            esac
            echo $check $line >> /etc/fstab.iscsi.tmp
        done < /etc/fstab.iscsi
        mv /etc/fstab.iscsi.tmp /etc/fstab.iscsi
        echo "The iSCSI filesystem entries in fstab.iscsi needs to be changed"
        echo "to new iSCSI device format like /udev/iscsib0t0l0disk."
        echo "Refer README for the new iSCSI device name format."
        echo " "

    fi
fi 

# this assumes that we're installing on this box, but that's usually 
# the case, and this is harmless if it's not.
# ignore errors on anything  except our iscsi module.
depmod -a 2>&1 | grep iscsi\.o
exit 0
