#ifndef ISCSI_HOOKS_H_
#define ISCSI_HOOKS_H_

/*
 * iSCSI connection daemon
 * Copyright (C) 2002 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsi-hooks.h,v 1.4.8.4 2003/12/16 05:59:52 smhatre Exp $
 *
 *  hooks into platform-specific code 
 */

#include "iscsi-config.h"
#include "iscsid.h"

/* these are typically just checking a variable set by signal handlers */
extern int iscsi_should_reload_config(void);
extern int iscsi_process_should_exit(void);

/* probably set signal handlers, among other things */
extern void iscsi_daemon_starting(int argc, char **argv);
extern void iscsi_daemon_stopping(void);

/* set config defaults to use if the config file is empty */
extern void iscsi_init_config_defaults(struct iscsi_config_defaults *defaults);

/* implement session process, and be able to kill it */
extern int iscsi_session_process(struct iscsi_session_process *process,
				 struct iscsi_session_config *config);
extern void iscsi_kill_session(struct iscsi_session_process *process);

/* notifications from the main daemon that may effect existing sessions */
extern void iscsi_lun_inventory_changed(struct iscsi_session_process *process);
extern void iscsi_config_changed(struct iscsi_session_process *process,
				 struct iscsi_session_config *new_config,
				 struct iscsi_session_config *old_config);

/* notifications that don't effect any particular session, but may prompt
 * platform-specific activity.
 */
extern void iscsi_session_count_changed(int num_sessions,
					int discovery_in_progress,
					int discovery_not_in_progress);

/* locking */
extern int iscsi_lock_file(int fd);
extern int iscsi_unlock_file(int fd);
extern int lock_pid_file(void);
#endif
