/*
 * iSCSI driver for Linux
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi@cisco.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsi-kernel-event.c,v 1.1.4.5 2003/10/30 14:49:51 smhatre Exp $
 */

#include <linux/fs.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/wait.h>
#include <linux/signal.h>

#include <linux/slab.h>
#include <asm/unistd.h>
#include <asm/semaphore.h>
#include <asm/uaccess.h>

#include "iscsi-kernel-event.h"
#include "iscsi-platform.h"

static struct kernel_request *r;	
				/* send_request does memory allocation, and 
				 * wait_for_kernel_event free the allocated 
				 * memory after sending the request.
				 */
static struct rsp_status *rs;
static struct semaphore sem;
static wait_queue_head_t req_queue;
static wait_queue_head_t rsp_queue;

static void
p(struct semaphore *sem)
{
	down(sem);
}

static void
v(struct semaphore *sem)
{
	up(sem);
}

int
response_status(char *rd)
{
	char *c, *start;
	unsigned long status = -1;

	start = rd;
	c = start;
	c = strchr(c, '\n');
	*c = '\0';
	c++;
	status = iscsi_strtoul(start, NULL, 0);

	return status;
}

void
process_response(struct response * rp)
{
	int status = -1;

	if (rp) {
		switch (rp->req_type) {
		case REMOVE_LUN:
			status = response_status(rp->data);
			rs->status = status;
			break;

		case CLEAR_LUNS:
			status = response_status(rp->data);
			rs->status = status;
			break;

		case PROBE_LUNS:
			status = response_status(rp->data);
			rs->status = status;
			break;

		case START_EVENT_HANDLER:
			/*return; */
			break;

		default:
			printk("Invalid response\n");
			if (rs)
				rs->status = -1;
			break;
		}
		if (rp->req_type != START_EVENT_HANDLER)
			wake_up(&rsp_queue);
	} else
		printk("NO RESPONSE\n");

}

int
send_request(int req_type, void *req, struct rsp_status * rp_status)
{
	char *c;
	int i;

	p(&sem);

	r = kmalloc(sizeof (struct kernel_request), GFP_KERNEL);
	if (!r) {
		printk("\niSCSI: send_request : failed to allocate memory "
		       "for kernel request\n");
		return -1;
	}

	memset(r, 0, sizeof (*r));

	switch (req_type) {
	case REMOVE_LUN:
		r->req_type = REMOVE_LUN;
		c = r->data;

		for (i = 0; i < (ISCSI_MAX_LUNS + 31) / 32; i++) {
			sprintf(c, "%lu\n",
				(((struct remove_lun *) req)->
							luns_activated[i]));
			c += strlen(c);
		}

		sprintf(c, "%s\n", ((struct remove_lun *) req)->
							target_link_dir);
		c += strlen(c);

		sprintf(c, "%d\n", ((struct remove_lun *) req)->host_no);
		c += strlen(c);

		sprintf(c, "%d\n", ((struct remove_lun *) req)->channel);
		c += strlen(c);

		sprintf(c, "%d\n", ((struct remove_lun *) req)->target_id);
		c += strlen(c);

		sprintf(c, "%d\n", ((struct remove_lun *) req)->lun);

		rs = rp_status;
		break;

	case CLEAR_LUNS:
		r->req_type = CLEAR_LUNS;

		c = r->data;
		for (i = 0; i < 2; i++) {
			sprintf(c, "%d\n", ((struct clear_luns *) req)->
					    host[i]);
			c += strlen(c);
		}
		for (i = 0; i < 2; i++) {
			sprintf(c, "%d\n", ((struct clear_luns *) req)->
					    chan[i]);
			c += strlen(c);
		}
		for (i = 0; i < 2; i++) {
			sprintf(c, "%d\n", ((struct clear_luns *) req)->tid[i]);
			c += strlen(c);
		}
		for (i = 0; i < 2; i++) {
			sprintf(c, "%d\n", ((struct clear_luns *) req)->lun[i]);
			c += strlen(c);
		}
		c -= strlen(c);

		rs = rp_status;
		break;

	case PROBE_LUNS:
		r->req_type = PROBE_LUNS;
		c = r->data;

		for (i = 0; i < (ISCSI_MAX_LUNS + 31) / 32; i++) {
			sprintf(c, "%ld\n",
				(((struct probe_devices *) req)->
				  luns_found[i]));
			c += strlen(c);
		}

		for (i = 0; i < (ISCSI_MAX_LUNS + 31) / 32; i++) {
			sprintf(c, "%ld\n",
				(((struct probe_devices *) req)->
				  luns_detected[i]));
			c += strlen(c);
		}

		for (i = 0; i < (ISCSI_MAX_LUNS + 31) / 32; i++) {
			sprintf(c, "%ld\n",
				(((struct probe_devices *) req)->
				  luns_activated[i]));
			c += strlen(c);
		}

		sprintf(c, "%d\n", ((struct probe_devices *) req)->host_no);
		c += strlen(c);

		sprintf(c, "%d\n", ((struct probe_devices *) req)->channel);
		c += strlen(c);
		sprintf(c, "%d\n", ((struct probe_devices *) req)->target_id);
		c += strlen(c);

		sprintf(c, "%u\n", ((struct probe_devices *) req)->
				     lun_bitmap[0]);
		c += strlen(c);

		sprintf(c, "%s\n", ((struct probe_devices *) req)->
				     target_link_dir);
		c += strlen(c);

		sprintf(c, "%d\n", ((struct probe_devices *) req)->
				     max_sd_devices);
		c += strlen(c);

		sprintf(c, "%d\n", ((struct probe_devices *) req)->
				     max_sg_devices);
		c += strlen(c);

		sprintf(c, "%d\n", ((struct probe_devices *) req)->
				     max_sr_devices);
		c += strlen(c);

		sprintf(c, "%d\n", ((struct probe_devices *) req)->
				     max_st_devices);

		rs = rp_status;
		break;

	case STOP_EVENT_HANDLER:
		r->req_type = STOP_EVENT_HANDLER;
		memset(r->data, 0, sizeof (r->data));
		rs = NULL;
		break;

	default:
		rs = NULL;
		printk("iSCSI: send_request: illegal request/event\n");
		break;

	}
	wake_up(&req_queue);

	return 0;
}

int
wait_for_response(void)
{
	sleep_on(&rsp_queue);

	return 0;
}

int
wait_for_request(unsigned long *arg)
{
	int rc;

	v(&sem);
	sleep_on(&req_queue);

	if ((rc = copy_to_user((void *) arg, r, sizeof (*r))))
		printk("\n copy to user failed %d\n", rc);
	kfree(r);

	return 1;
}

int
initialise_event_handler()
{
	init_MUTEX_LOCKED(&sem);	/* Initialise semaphore */
	init_waitqueue_head(&req_queue);	/* Initialise wait queue */
	init_waitqueue_head(&rsp_queue);	/* Initialise wait queue */

	return 1;
}
