/*
 * iSCSI driver for Linux
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi@cisco.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsi-kernel-event.h,v 1.1.4.1 2003/10/21 13:40:33 krishmnc Exp $
 *
 */

#ifndef ISCSI_KERNEL_EVENT_H_
#define ISCSI_KERNEL_EVENT_H_

/* states of event handler */
#define START 0
#define STOP 1

/* Types of requests/events */
#define START_EVENT_HANDLER 1
#define STOP_EVENT_HANDLER 2
#define PROBE_LUNS 3
#define CLEAR_LUNS 4
#define REMOVE_LUN 5
#define ISCSI_MAX_LUNS	256

struct remove_lun {
	unsigned int host_no;
	unsigned int channel;
	unsigned int target_id;
	int lun;
	unsigned long luns_activated[(ISCSI_MAX_LUNS + 31) / 32];
	unsigned char *target_link_dir;
};

struct remove_luns {
	unsigned int host_no;
	unsigned int channel;
	unsigned int target_id;
	unsigned int lun;
	unsigned char *target_link_dir;
	unsigned long luns_activated[(ISCSI_MAX_LUNS + 31) / 32];
};

struct clear_luns {
	unsigned int host[2];
	unsigned int chan[2];
	unsigned int tid[2];
	unsigned int lun[2];
};

struct probe_devices {
	unsigned long luns_found[(ISCSI_MAX_LUNS + 31) / 32];
	unsigned long luns_detected[(ISCSI_MAX_LUNS + 31) / 32];
	unsigned long luns_activated[(ISCSI_MAX_LUNS + 31) / 32];
	unsigned int host_no;
	unsigned int channel;
	unsigned int target_id;
	unsigned int *lun_bitmap;
	unsigned char *target_link_dir;
	unsigned int max_sd_devices;
	unsigned int max_sg_devices;
	unsigned int max_sr_devices;
	unsigned int max_st_devices;
};

/* Request/Response packets */
struct kernel_request {
	int req_type;
	char data[1024];
};

struct response {
	int req_type;
	char data[1024];
};

struct rsp_status {
	int status;
};

#define WAIT_FOR_EVENT 0x0047030c
#define RELEASE_EVENT_HANDLER 0x0047030d

void process_response(struct response * r);
int send_request(int req_type, void *req, struct rsp_status * rp_status);
int wait_for_response(void);
int wait_for_request(unsigned long *arg);
int initialise_event_handler(void);

void start_event_handler(void);
int stop_event_handler(void);

#endif
