/*
 * iSCSI driver for Linux
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsi-network.c,v 1.1.2.6 2003/12/23 11:04:18 krishmnc Exp $
 */


#include <linux/tcp.h>

#include "iscsi-protocol.h"
#include "iscsi-session.h"
#include "iscsi.h"

extern int this_is_iscsi_boot;
extern int iscsi_system_is_rebooting;

int
iscsi_reboot_notifier_function(struct notifier_block *this, unsigned long code,
			       void *unused)
{

	if (code == SYS_DOWN) {
		DEBUG_INIT ("\niSCSI: iscsi_reboot_notifier_function "
			    "called with code SYS_DOWN = 0x%lu\n", code);
	} else if (code == SYS_RESTART) {
		DEBUG_INIT ("\niSCSI: iscsi_reboot_notifier_function "
			    "called with code SYS_RESTART = 0x%lu\n", code);
	} else if (code == SYS_HALT) {
		DEBUG_INIT ("\niSCSI: iscsi_reboot_notifier_function "
			    "called with code SYS_HALT = 0x%lu\n", code);
	} else if (code == SYS_POWER_OFF) {
		DEBUG_INIT ("\niSCSI: iscsi_reboot_notifier_function "
			    "called with code SYS_POWER_OFF = 0x%lu\n", code);
	} else {
		printk ("\niSCSI: iscsi_reboot_notifier_function "
			"called with unknown code = 0x%lu !!!\n", code);
	}

	DEBUG_INIT
	    ("\niSCSI: Setting iscsi_system_is_rebooting, current->pid = %d "
	     "this->next = 0x%p iscsi_reboot_notifier_function = 0x%p\n",
	     current->pid, this->next, &iscsi_reboot_notifier_function);

	iscsi_system_is_rebooting = 1;

	if (!iscsi_shutdown()) {
		printk("iSCSI: driver shutdown failed\n");
	}
	printk ("\niSCSI: iscsi_reboot_notifier_function: "
		"driver shutdown succeeded\n");

	return NOTIFY_DONE;
}


/* create and connect a new socket for this session */
int
iscsi_connect(struct iscsi_session *session)
{
	mm_segment_t oldfs;
	struct socket *socket;
	struct sockaddr_in addr;
	int window_size;
	int arg = 1, arglen;
	int rc, ret = 0;

	if (session->socket) {
		printk("iSCSI: session iscsi bus %d target id %d already has "
		       "socket \n", session->iscsi_bus, session->target_id);
		return 0;
	}

	oldfs = get_fs();
	set_fs(get_ds());

	rc = sock_create(PF_INET, SOCK_STREAM, IPPROTO_TCP, &socket);
	if (rc < 0) {
		printk ("iSCSI: session iscsi bus %d target id %d failed "
			"to create socket, rc %d\n", 
			session->iscsi_bus, session->target_id, rc);
		set_fs(oldfs);
		return 0;
	}

	/* no delay in sending */
	rc = socket->ops->setsockopt(socket, IPPROTO_TCP, TCP_NODELAY,
				     (char *) &arg, sizeof (arg));
	if (rc < 0) {
		printk ("iSCSI: session iscsi bus %d target id %d failed to "
			"setsockopt TCP_NODELAY, rc %d\n",
			session->iscsi_bus, session->target_id, rc);
		goto done;
	}

	/* try to ensure a reasonably sized TCP window */
	arglen = sizeof (window_size);
	if (sock_getsockopt
	    (socket, SOL_SOCKET, SO_RCVBUF, (char *) &window_size,
	     &arglen) >= 0) {
		DEBUG_FLOW ("iSCSI: session iscsi bus %d target id %d TCP "
			    "recv window size %u\n",
			    session->iscsi_bus, session->target_id, 
			    window_size);

		if (session->tcp_window_size
		    && (window_size < session->tcp_window_size)) {
			window_size = session->tcp_window_size;
			rc = sock_setsockopt(socket, SOL_SOCKET, SO_RCVBUF,
					     (char *) &window_size,
					     sizeof (window_size));
			if (rc < 0) {
				printk
				    ("iSCSI: session iscsi bus %d target id %d "
				     "failed to set TCP recv window size to %u,"
				     " rc %d\n",
				     session->iscsi_bus, session->target_id,
				     window_size, rc);
			} else
			    if (sock_getsockopt
				(socket, SOL_SOCKET, SO_RCVBUF,
				 (char *) &window_size, &arglen) >= 0) {
				DEBUG_INIT
				    ("iSCSI: session iscsi bus %d target id %d"
				     " set TCP recv window size to %u,"
				     " actually got %u\n",
				     session->iscsi_bus, session->target_id,
				     session->tcp_window_size, window_size);
			}
		}
	} else {
		printk ("iSCSI: session iscsi bus %d target id %d getsockopt"
			" RCVBUF failed\n", 
			session->iscsi_bus, session->target_id);
	}

	if (sock_getsockopt
	    (socket, SOL_SOCKET, SO_SNDBUF, (char *) &window_size,
	     &arglen) >= 0) {
		DEBUG_FLOW
		    ("iSCSI: session iscsi bus %d target id %d TCP send window "
		     "size %u\n",
		     session->iscsi_bus, session->target_id, window_size);

		if (session->tcp_window_size
		    && (window_size < session->tcp_window_size)) {
			window_size = session->tcp_window_size;
			rc = sock_setsockopt(socket, SOL_SOCKET, SO_SNDBUF,
					     (char *) &window_size,
					     sizeof (window_size));
			if (rc < 0) {
				printk
				    ("iSCSI: session iscsi bus %d target id %d"
				     " failed to set TCP send window size to "
				     "%u, rc %d\n",
				     session->iscsi_bus, session->target_id,
				     window_size, rc);
			} else
			    if (sock_getsockopt
				(socket, SOL_SOCKET, SO_SNDBUF,
				 (char *) &window_size, &arglen) >= 0) {
				DEBUG_INIT
				    ("iSCSI: session iscsi bus %d target id %d"
				     " set TCP send window size to %u, "
				     "actually got %u\n",
				     session->iscsi_bus, session->target_id,
				     session->tcp_window_size, window_size);
			}
		}
	} else {
		printk ("iSCSI: session iscsi bus %d target id %d getsockopt "
			"SNDBUF failed\n",
			session->iscsi_bus, session->target_id);
	}

	/* connect to the target */
	addr.sin_family = AF_INET;
	addr.sin_port = htons(session->port);
	if (session->ip_length == 4) {
		memcpy(&addr.sin_addr.s_addr, session->ip_address,
		       MIN(sizeof (addr.sin_addr.s_addr),
			  (unsigned int) session->ip_length));
	} else {
		/* FIXME: IPv6 */
		printk
		    ("iSCSI: session iscsi bus %d target id %d unable to "
		     "handle IPv6 address, length %u, addr %u.%u.%u.%u\n",
		     session->iscsi_bus, session->target_id, session->ip_length,
		     session->ip_address[0], session->ip_address[1],
		     session->ip_address[2], session->ip_address[3]);
		goto done;
	}
	rc = socket->ops->connect(socket, (struct sockaddr *) &addr,
				  sizeof (addr), 0);

	if (signal_pending(current))
		goto done;

	if (rc < 0) {
		char *error = iscsi_strerror(-rc);
		if (error && error[0] != '\0') {
			printk ("iSCSI: session iscsi bus %d target id %d to %s"
				" failed to connect, rc %d, %s\n",
				session->iscsi_bus, session->target_id,
				session->log_name, rc, error);
		} else {
			printk ("iSCSI: session iscsi bus %d target id %d to %s"
				" failed to connect, rc %d\n",
				session->iscsi_bus, session->target_id,
				session->log_name, rc);
		}

	} else {
		if (LOG_ENABLED(ISCSI_LOG_LOGIN))
			printk ("iSCSI: session iscsi bus %d target id %d to %s"
				" connected at %lu\n",
				session->iscsi_bus, session->target_id,
				session->log_name, jiffies);
		ret = 1;
	}

      done:
	if (ret) {
		/* save the socket pointer for later */
		session->socket = socket;
	} else {
		/* close the socket */
		sock_release(socket);
		session->socket = NULL;
	}
	smp_mb();
	set_fs(oldfs);
	return ret;
}

void
iscsi_disconnect(struct iscsi_session *session)
{
	if (session->socket) {
#if TCP_ABORT_ON_DROP
		if (test_and_clear_bit(SESSION_DROPPED, &session->control_bits)
		    && !test_bit(SESSION_LOGGED_OUT, &session->control_bits)) {
			/* setting linger on and lingertime to 0 before closing
			 * the socket will trigger a TCP abort (abort all sends
			 * and receives, possibly send RST, connection to 
			 * CLOSED), which is probably what we want if we're 
			 * dropping and restarting a session.  A TCP Abort will 
			 * discard TCP data, which is probably a bunch of 
			 * commands and data we'll resend on a new session 
			 * anyway.  This frees up skbuffs, and makes the VM 
			 * livelock less likely.  When we relogin again to the 
			 * target with the same ISID, the target will kill off 
			 * the old connections on it's side, so the FIN 
			 * handshake should be unnecessary, and there
			 * are cases where network failures may prevent the FIN
			 * handshake from completing, so the connection wouldn't
			 * get cleaned up unless the TCP stack has timeouts for
			 * some of the TCP states.
			 */
			struct linger ling;
			mm_segment_t oldfs;

			memset(&ling, 0, sizeof (ling));
			ling.l_onoff = 1;
			ling.l_linger = 0;

			/* we could adjust the socket linger values directly, 
			 * but using the sockopt call is less likely to break 
			 * if someone overhauls the socket structure.
			 */
			oldfs = get_fs();
			set_fs(get_ds());

			if (sock_setsockopt
			    (session->socket, IPPROTO_TCP, SO_LINGER,
			     (char *) &ling, sizeof (ling)) < 0) {
				printk
				    ("iSCSI: session iscsi bus %d target id %d"
				     " couldn't set lingertime to zero after "
				     "session drop\n",
				     session->iscsi_bus, session->target_id);
			} else {
				DEBUG_INIT
				    ("iSCSI: session iscsi bus %d target id %d"
				     " set lingertime to zero because of "
				     "session drop\n",
				     session->iscsi_bus, session->target_id);
			}

			set_fs(oldfs);
		}
#endif

		/* close the socket, triggering either a TCP close or 
		 * a TCP abort 
		 */
		sock_release(session->socket);

		session->socket = NULL;
		smp_mb();
	}
}

int
iscsi_sendmsg(struct iscsi_session *session, struct msghdr *msg, int len)
{
	mm_segment_t oldfs;

	if (session->socket) {
		int rc;
		oldfs = get_fs();
		set_fs(get_ds());

		/* Try to avoid resource acquisition deadlocks by using 
		 * GFP_ATOMIC. 
		 */
		session->socket->sk->sk_allocation = GFP_ATOMIC;

		/* FIXME: ought to loop handling short writes, 
		 * unless a signal occurs 
		 */
		rc = sock_sendmsg(session->socket, msg, len);

		set_fs(oldfs);

		return rc;
	}

	return -1;
}

int
iscsi_recvmsg(struct iscsi_session *session, struct msghdr *msg, int len)
{
	mm_segment_t oldfs;

	if (session->socket) {
		int rc; 
		oldfs = get_fs();
		set_fs(get_ds());

		/* Try to avoid memory allocation deadlocks by using GFP_ATOMIC.
		 */
		session->socket->sk->sk_allocation = GFP_ATOMIC;

		rc = sock_recvmsg(session->socket, msg, len, MSG_WAITALL);

		set_fs(oldfs);

		return rc;
	}

	return -1;
}
