#ifndef ISCSI_PORTAL_H_
#define ISCSI_PORTAL_H_
/*
 * iSCSI driver for Linux
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsi-portal.h,v 1.4.10.3 2003/12/16 05:59:53 smhatre Exp $
 *
 * portal info structure used in ioctls and the kernel module
 *
 */

#include "iscsi-session.h"

struct iscsi_portal_info {
	int login_timeout;
	int auth_timeout;
	int active_timeout;
	int idle_timeout;
	int ping_timeout;
	int abort_timeout;
	int reset_timeout;
	int replacement_timeout;	/* FIXME: should this be per-session 
					 * rather than per-portal? 
					 */
	int min_disk_command_timeout;	/* FIXME: should this be per-session 
					 * rather than per-portal? 
					 */
	int max_disk_command_timeout;	/* FIXME: should this be per-session 
					 * rather than per-portal? 
					 */
	int InitialR2T;
	int ImmediateData;
	int MaxRecvDataSegmentLength;
	int FirstBurstLength;
	int MaxBurstLength;
	int DefaultTime2Wait;
	int DefaultTime2Retain;
	int HeaderDigest;
	int DataDigest;
	int ip_length;
	unsigned char ip_address[16];
	int port;
	int tag;
	int tcp_window_size;
	int type_of_service;
	int preference;		/* preference relative to other portals, 
				 * higher is better 
				 */
};

#define MAX_PORTALS 32		/* 
				 * 32 is the sizeof(unsigned int).
				 * If max portals to a target exceeds 32 then
				 * we need to change the 
				 * preferred_portal_bitmap,
				 * preferred_subnet_bitmap from unsigned int to
				 * an array of unsigned int's.
				 */

unsigned int find_portal(struct iscsi_session *session, 
			 unsigned char *ip_address, int ip_length, int port);
void set_portal_config(struct iscsi_session *session, unsigned int p);
int set_portal(struct iscsi_session *session, unsigned int p);
void set_preferred_subnet_bitmap(struct iscsi_session *session);
void set_preferred_portal_bitmap(struct iscsi_session *session);
int get_appropriate_portal(struct iscsi_session *session);
void next_portal(struct iscsi_session *session);
int same_network_portal(struct iscsi_portal_info *p1, 
			struct iscsi_portal_info *p2);
#endif
