#ifndef ISCSI_H_
#define ISCSI_H_

/*
 * iSCSI driver for Linux
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsi.h,v 1.40.8.13 2003/12/23 10:49:00 krishmnc Exp $ 
 *
 * iscsi.h
 *
 *    include for iSCSI kernel module
 * 
 */
#include <linux/list.h>
#include <linux/socket.h>
#include "iscsiAuthClient.h"
#include "iscsi-common.h"
#include "iscsi-limits.h"
#ifndef MIN
# define MIN(x, y)        min(x,y)
#endif

#ifndef MAX
# define MAX(x, y)    max(x,y)
#endif

struct iscsi_hba {
	struct Scsi_Host *host;
	unsigned int host_no;
	unsigned long flags;
	spinlock_t session_lock;
	struct list_head session_head;
	atomic_t num_sessions;
	kmem_cache_t *task_cache;
};

/* HBA flags */
#define ISCSI_HBA_ACTIVE          0
#define ISCSI_HBA_SHUTTING_DOWN   1
#define ISCSI_HBA_RELEASING       2

struct iscsi_dev_data {
	struct iscsi_session *session;
	unsigned long flags;
};
/* driver entry points needed by the probing code */
int iscsi_queue(struct iscsi_session * session, Scsi_Cmnd * sc,
		void (*done) (Scsi_Cmnd *));
int iscsi_squash_cmnd(struct iscsi_session * session, Scsi_Cmnd * sc);

/* flags we set on Scsi_Cmnds */
#define COMMAND_TIMEDOUT 1

/* run-time controllable logging */
#define ISCSI_LOG_ERR   1
#define ISCSI_LOG_SENSE 2
#define ISCSI_LOG_INIT  3
#define ISCSI_LOG_QUEUE 4
#define ISCSI_LOG_ALLOC 5
#define ISCSI_LOG_EH    6
#define ISCSI_LOG_FLOW  7
#define ISCSI_LOG_RETRY 8
#define ISCSI_LOG_LOGIN 9
#define ISCSI_LOG_TIMEOUT 10

#define LOG_SET(flag) (1U << (flag))
#define LOG_ENABLED(flag) (iscsi_log_settings & (1U << (flag)))

extern volatile unsigned int iscsi_log_settings;

#ifdef DEBUG
/* compile in all the debug messages and tracing */
# define INCLUDE_DEBUG_ERROR  1
# define INCLUDE_DEBUG_TRACE  1
# define INCLUDE_DEBUG_INIT   1
# define INCLUDE_DEBUG_QUEUE  1
# define INCLUDE_DEBUG_FLOW   1
# define INCLUDE_DEBUG_ALLOC  1
# define INCLUDE_DEBUG_EH     1
# define INCLUDE_DEBUG_RETRY  1
# define INCLUDE_DEBUG_TIMEOUT 1
#else
/* leave out the tracing and most of the debug messages */
# define INCLUDE_DEBUG_ERROR  1
# define INCLUDE_DEBUG_TRACE  0
# define INCLUDE_DEBUG_INIT   0
# define INCLUDE_DEBUG_QUEUE  0
# define INCLUDE_DEBUG_FLOW   0
# define INCLUDE_DEBUG_ALLOC  0
# define INCLUDE_DEBUG_EH     0
# define INCLUDE_DEBUG_RETRY  0
# define INCLUDE_DEBUG_TIMEOUT 0
#endif

#if INCLUDE_DEBUG_INIT
#  define DEBUG_INIT(fmt, args...) do { if (LOG_ENABLED(ISCSI_LOG_INIT)) printk(fmt , ## args); } while (0)
#else
#  define DEBUG_INIT(fmt, args...) do { } while (0)
#endif

#if INCLUDE_DEBUG_QUEUE
#  define DEBUG_QUEUE(fmt, args...) do { if (LOG_ENABLED(ISCSI_LOG_QUEUE)) printk(fmt , ## args); } while (0)
#else
#  define DEBUG_QUEUE(fmt, args...) do { } while (0)
#endif

#if INCLUDE_DEBUG_FLOW
#  define DEBUG_FLOW(fmt, args...) do { if (LOG_ENABLED(ISCSI_LOG_FLOW)) printk(fmt , ## args); } while (0)
#else
#  define DEBUG_FLOW(fmt, args...) do { } while (0)
#endif

#if INCLUDE_DEBUG_ALLOC
#  define DEBUG_ALLOC(fmt, args...) do { if (LOG_ENABLED(ISCSI_LOG_ALLOC)) printk(fmt , ## args); } while (0)
#else
#  define DEBUG_ALLOC(fmt, args...) do { } while (0)
#endif

#if INCLUDE_DEBUG_RETRY
#  define DEBUG_RETRY(fmt, args...) do { if (LOG_ENABLED(ISCSI_LOG_RETRY)) printk(fmt , ## args); } while (0)
#else
#  define DEBUG_RETRY(fmt, args...) do { } while (0)
#endif

#if INCLUDE_DEBUG_EH
#  define DEBUG_EH(fmt, args...) do { if (LOG_ENABLED(ISCSI_LOG_EH)) printk(fmt , ## args); } while (0)
#else
#  define DEBUG_EH(fmt, args...) do { } while (0)
#endif

#if INCLUDE_DEBUG_ERROR
#  define DEBUG_ERR(fmt, args...) do { if (LOG_ENABLED(ISCSI_LOG_ERR)) printk(fmt , ## args); } while (0)
#else
#  define DEBUG_ERR(fmt, args...) do { } while (0)
#endif

#if INCLUDE_DEBUG_TIMEOUT
#  define DEBUG_TIMEOUT(fmt, args...) do { if (LOG_ENABLED(ISCSI_LOG_TIMEOUT)) printk(fmt , ## args); } while (0)
#else
#  define DEBUG_TIMEOUT(fmt, args...) do { } while (0)
#endif

/* the Scsi_Cmnd's request_bufflen doesn't always match the actual amount of data
 * to be read or written.  Try to compensate by decoding the cdb.
 */
extern unsigned int iscsi_expected_data_length(Scsi_Cmnd * sc);

/* extract parts of the sense data from an (unsigned char *) to the beginning of sense data */
#define SENSE_KEY(sensebuf) ((sensebuf)[2] & 0x0F)
#define ASC(sensebuf)       ((sensebuf)[12])
#define ASCQ(sensebuf)       ((sensebuf)[13])

#endif
#define SNA32_CHECK 2147483648UL
unsigned long * command_flags(Scsi_Cmnd * sc);
int iscsi_handle_signals(struct iscsi_session *session);
void wake_tx_thread(int control_bit, struct iscsi_session *session);
void iscsi_request_logout(struct iscsi_session *session, int logout,
		     	  int logout_response);
int update_address(struct iscsi_session *session, char *address);
int iscsi_rx_thread(void *vtaskp);
int iscsi_tx_thread(void *vtaskp);
int add_lun_line(struct iscsi_session *session, int lun, off_t start, 
		 off_t finish, off_t * current_offset, char **buffer);
int internally_retryable(Scsi_Cmnd * sc);
void iscsi_daemonize(char *daemon_name);
int sna_lt(uint32_t n1, uint32_t n2);
int sna_lte(uint32_t n1, uint32_t n2);
void request_command_retries(unsigned long arg);
void requeue_deferred_commands(struct iscsi_session *session, unsigned int lun);
int add_cmnd(Scsi_Cmnd * sc, Scsi_Cmnd ** head, Scsi_Cmnd ** tail);
int iscsi_xmit_task_mgmt(struct iscsi_session *session, uint8_t func_type,
			 struct iscsi_task *task, uint32_t mgmt_itt);
void iscsi_xmit_queued_cmnds(struct iscsi_session *session);
void iscsi_xmit_data(struct iscsi_task *task, uint32_t ttt, 
		     uint32_t data_offset, uint32_t data_length);
void iscsi_xmit_r2t_data(struct iscsi_session *session);
void iscsi_xmit_nop_reply(struct iscsi_session *session,
			  struct iscsi_nop_info *nop_info);
void iscsi_xmit_nop_replys(struct iscsi_session *session);
void iscsi_xmit_logout(struct iscsi_session *session, uint32_t itt, int reason);
void iscsi_xmit_ping(struct iscsi_session *session, uint32_t itt,
		     unsigned char *data, int length);
unsigned int iscsi_command_attr(Scsi_Cmnd * cmd);
void print_cmnd(Scsi_Cmnd * sc);
int find_number(char *start, char *end, int *number);
int find_ip(char *start, char *end, char *addr);
int find_keyword(char *start, char *end, char *key);
void iscsi_ping_test_session(struct iscsi_session *session, int total_data_length);
void * kmap_sg(struct scatterlist *sg);
void kunmap_sg(struct scatterlist *sg);
void * sg_virtual_address(struct scatterlist *sg);
int iscsi_sendmsg(struct iscsi_session *session, struct msghdr *msg, int len);
int iscsi_recvmsg(struct iscsi_session *session, struct msghdr *msg, int len);
int iscsi_shutdown(void);
int iscsi_set_if_addr(void);
int iscsi_reboot_notifier_function(struct notifier_block *this, 
				   unsigned long code, void *unused);
void iscsi_recv_logout(struct iscsi_session *session, 
		       struct IscsiLogoutRspHdr *stlh);
int iscsi_recv_nop_data(struct iscsi_session *session, unsigned char *buffer,
			int data_length);
void iscsi_recv_nop(struct iscsi_session *session, struct IscsiNopInHdr *stnih);
void iscsi_recv_cmd(struct iscsi_session *session, 
		    struct IscsiScsiRspHdr *stsrh, unsigned char *sense_data);
void iscsi_recv_r2t(struct iscsi_session *session, struct IscsiRttHdr *strh);
void iscsi_recv_data(struct iscsi_session *session, 
		     struct IscsiDataRspHdr *stdrh);
void iscsi_recv_task_mgmt(struct iscsi_session *session,
			  struct IscsiScsiTaskMgtRspHdr *ststmrh);
void lun_reset_occured(struct iscsi_session *session, unsigned int lun);
void iscsi_recv_reject(struct iscsi_session *session,
		       struct IscsiRejectRspHdr *reject, unsigned char *xbuf);
void iscsi_recv_async_event(struct iscsi_session *session,
			    struct IscsiAsyncEvtHdr *staeh, 
			    unsigned char *xbuf);
void process_inquiry_data(struct iscsi_session *session, Scsi_Cmnd * sc,
			  uint8_t * data);
int iscsi_lun_thread(void *vtaskp);
void iscsi_retry_commands(struct iscsi_session *session);
int process_timedout_commands(struct iscsi_session *session);
char * iscsi_strerror(int errno);
void add_completion_timer(Scsi_Cmnd * sc);
void add_command_timer(struct iscsi_session *session, Scsi_Cmnd * sc,
		       void (*fn) (unsigned long));
void add_task_timer(struct iscsi_task *task, void (*fn) (unsigned long));
int del_command_timer(Scsi_Cmnd * sc);
int del_task_timer(struct iscsi_task *task);
void iscsi_command_times_out(unsigned long arg);
void iscsi_task_times_out(unsigned long arg);
int iscsi_timer_thread(void *vtaskp);

#if DEBUG_TRACE
void
iscsi_fill_trace(unsigned char type, Scsi_Cmnd * sc, struct iscsi_task *task,
		 unsigned long data1, unsigned long data2);
# define ISCSI_TRACE(P_TYPE, P_CMND, P_TASK, P_DATA1, P_DATA2) \
           iscsi_fill_trace((P_TYPE), (P_CMND), (P_TASK), (P_DATA1), (P_DATA2))
#else
# define ISCSI_TRACE(P_TYPE, P_CMND, P_TASK, P_DATA1, P_DATA2)
#endif
