#ifndef ISCSIAUTHCLIENT_H
#define ISCSIAUTHCLIENT_H

/*
 * iSCSI connection daemon
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi-devel@lists.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: iscsiAuthClient.h,v 1.13.8.5 2003/12/16 05:59:53 smhatre Exp $
 */

/*
 * This file is the include file for for iscsiAuthClient.c
 */

#ifdef __cplusplus
extern "C" {    
#endif  

	enum { iscsiAuthStringMaxLength = 256 };
	enum { iscsiAuthStringBlockMaxLength = 1024 };
	enum { iscsiAuthLargeBinaryMaxLength = 1024 };

	enum { iscsiAuthRecvEndMaxCount = 10 };

	enum { iscsiAuthClientSignature = 0x5984B2E3 };

	enum { iscsiAuthChapResponseLength = 16 };

/*
 * Note: The ordering of these values are chosen to match
 *       the ordering of the keys as shown in the iSCSI spec.
 *       The table IscsiAuthClientKeyInfo in iscsiAuthClient.c
 *       must also match this order.
 */
	enum iscsiAuthKeyType {
		iscsiAuthKeyTypeNone = -1,
		iscsiAuthKeyTypeFirst = 0,
		iscsiAuthKeyTypeAuthMethod = iscsiAuthKeyTypeFirst,
		iscsiAuthKeyTypeChapAlgorithm,
		iscsiAuthKeyTypeChapUsername,
		iscsiAuthKeyTypeChapResponse,
		iscsiAuthKeyTypeChapIdentifier,
		iscsiAuthKeyTypeChapChallenge,
		iscsiAuthKeyTypeMaxCount,
		iscsiAuthKeyTypeLast = iscsiAuthKeyTypeMaxCount - 1
	};

	enum {
		/* Common options for all keys. */
		iscsiAuthOptionReject = -2,
		iscsiAuthOptionNotPresent = -1,
		iscsiAuthOptionNone = 1,

		iscsiAuthMethodChap = 2,
		iscsiAuthMethodMaxCount = 2,

		iscsiAuthChapAlgorithmMd5 = 5,
		iscsiAuthChapAlgorithmMaxCount = 2
	};

	enum iscsiAuthNegRole {
		iscsiAuthNegRoleOriginator = 1,
		iscsiAuthNegRoleResponder = 2
	};

/*
 * Note: These values are chosen to map to the values sent
 *       in the iSCSI header.
 */
	enum iscsiAuthVersion {
		iscsiAuthVersionDraft8 = 2,
		iscsiAuthVersionRfc = 0
	};

	enum iscsiAuthStatus {
		iscsiAuthStatusNoError = 0,
		iscsiAuthStatusError,
		iscsiAuthStatusPass,
		iscsiAuthStatusFail,
		iscsiAuthStatusContinue,
		iscsiAuthStatusInProgress
	};

	enum iscsiAuthDebugStatus {
		iscsiAuthDebugStatusNotSet = 0,

		iscsiAuthDebugStatusAuthPass,
		iscsiAuthDebugStatusAuthRemoteFalse,

		iscsiAuthDebugStatusAuthFail,

		iscsiAuthDebugStatusAuthMethodBad,
		iscsiAuthDebugStatusChapAlgorithmBad,
		iscsiAuthDebugStatusPasswordDecryptFailed,
		iscsiAuthDebugStatusPasswordTooShortWithNoIpSec,
		iscsiAuthDebugStatusAuthServerError,
		iscsiAuthDebugStatusAuthStatusBad,
		iscsiAuthDebugStatusAuthPassNotValid,
		iscsiAuthDebugStatusSendDuplicateSetKeyValue,
		iscsiAuthDebugStatusSendStringTooLong,
		iscsiAuthDebugStatusSendTooMuchData,

		iscsiAuthDebugStatusAuthMethodExpected,
		iscsiAuthDebugStatusChapAlgorithmExpected,
		iscsiAuthDebugStatusChapIdentifierExpected,
		iscsiAuthDebugStatusChapChallengeExpected,
		iscsiAuthDebugStatusChapResponseExpected,
		iscsiAuthDebugStatusChapUsernameExpected,

		iscsiAuthDebugStatusAuthMethodNotPresent,
		iscsiAuthDebugStatusAuthMethodReject,
		iscsiAuthDebugStatusAuthMethodNone,
		iscsiAuthDebugStatusChapAlgorithmReject,
		iscsiAuthDebugStatusChapChallengeReflected,
		iscsiAuthDebugStatusPasswordIdentical,

		iscsiAuthDebugStatusLocalPasswordNotSet,

		iscsiAuthDebugStatusChapIdentifierBad,
		iscsiAuthDebugStatusChapChallengeBad,
		iscsiAuthDebugStatusChapResponseBad,
		iscsiAuthDebugStatusUnexpectedKeyPresent,
		iscsiAuthDebugStatusTbitSetIllegal,
		iscsiAuthDebugStatusTbitSetPremature,

		iscsiAuthDebugStatusRecvMessageCountLimit,
		iscsiAuthDebugStatusRecvDuplicateSetKeyValue,
		iscsiAuthDebugStatusRecvStringTooLong,
		iscsiAuthDebugStatusRecvTooMuchData
	};

	enum iscsiAuthNodeType {
		iscsiAuthNodeTypeInitiator = 1,
		iscsiAuthNodeTypeTarget = 2
	};

	enum iscsiAuthPhase {
		iscsiAuthPhaseConfigure = 1,
		iscsiAuthPhaseNegotiate,
		iscsiAuthPhaseAuthenticate,
		iscsiAuthPhaseDone,
		iscsiAuthPhaseError
	};

	enum iscsiAuthLocalState {
		iscsiAuthLocalStateSendAlgorithm = 1,
		iscsiAuthLocalStateRecvAlgorithm,
		iscsiAuthLocalStateRecvChallenge,
		iscsiAuthLocalStateDone,
		iscsiAuthLocalStateError
	};

	enum iscsiAuthRemoteState {
		iscsiAuthRemoteStateSendAlgorithm = 1,
		iscsiAuthRemoteStateSendChallenge,
		iscsiAuthRemoteStateRecvResponse,
		iscsiAuthRemoteStateAuthRequest,
		iscsiAuthRemoteStateDone,
		iscsiAuthRemoteStateError
	};

	typedef void IscsiAuthClientCallback(void *, void *, int);

	struct iscsiAuthClientGlobalStats {
		unsigned long requestSent;
		unsigned long responseReceived;
	};

	struct iscsiAuthBufferDesc {
		unsigned int length;
		void *address;
	};

	struct iscsiAuthKey {
		unsigned int present:1;
		unsigned int processed:1;
		unsigned int valueSet:1;
		char *string;
	};

	struct iscsiAuthLargeBinaryKey {
		unsigned int length;
		unsigned char *largeBinary;
	};

	struct iscsiAuthKeyBlock {
		unsigned int transitBit:1;
		unsigned int duplicateSet:1;
		unsigned int stringTooLong:1;
		unsigned int tooMuchData:1;
		unsigned int blockLength:16;
		char *stringBlock;
		struct iscsiAuthKey key[iscsiAuthKeyTypeMaxCount];
	};

	struct iscsiAuthStringBlock {
		char stringBlock[iscsiAuthStringBlockMaxLength];
	};

	struct iscsiAuthLargeBinary {
		unsigned char largeBinary[iscsiAuthLargeBinaryMaxLength];
	};

	struct iscsiAuthClient {
		unsigned long signature;

		void *glueHandle;
		struct iscsiAuthClient_t *next;
		unsigned int authRequestId;

		enum iscsiAuthNodeType nodeType;
		unsigned int authMethodCount;
		int authMethodList[iscsiAuthMethodMaxCount];
		enum iscsiAuthNegRole authMethodNegRole;
		unsigned int chapAlgorithmCount;
		int chapAlgorithmList[iscsiAuthChapAlgorithmMaxCount];
		int authRemote;
		char username[iscsiAuthStringMaxLength];
		int passwordPresent;
		unsigned int passwordLength;
		unsigned char passwordData[iscsiAuthStringMaxLength];
		char methodListName[iscsiAuthStringMaxLength];
		enum iscsiAuthVersion version;
		unsigned int chapChallengeLength;
		int ipSec;
		int base64;

		unsigned int authMethodValidCount;
		int authMethodValidList[iscsiAuthMethodMaxCount];
		int authMethodValidNegRole;
		const char *rejectOptionName;
		const char *noneOptionName;

		int recvInProgressFlag;
		int recvEndCount;
		IscsiAuthClientCallback *callback;
		void *userHandle;
		void *messageHandle;

		enum iscsiAuthPhase phase;
		enum iscsiAuthLocalState localState;
		enum iscsiAuthRemoteState remoteState;
		enum iscsiAuthStatus remoteAuthStatus;
		enum iscsiAuthDebugStatus debugStatus;
		int negotiatedAuthMethod;
		int negotiatedChapAlgorithm;
		int authResponseFlag;
		int authServerErrorFlag;
		int transitBitSentFlag;

		unsigned int sendChapIdentifier;
		struct iscsiAuthLargeBinaryKey sendChapChallenge;
		char chapUsername[iscsiAuthStringMaxLength];

		int recvChapChallengeStatus;
		struct iscsiAuthLargeBinaryKey recvChapChallenge;

		char scratchKeyValue[iscsiAuthStringMaxLength];

		struct iscsiAuthKeyBlock recvKeyBlock;
		struct iscsiAuthKeyBlock sendKeyBlock;
	};

#ifdef __cplusplus
}
#endif
#include "iscsiAuthClientGlue.h"
#ifdef __cplusplus
extern "C" {
#endif

	extern struct iscsiAuthClientGlobalStats iscsiAuthClientGlobalStats;

	extern int iscsiAuthClientInit(int, int, struct iscsiAuthBufferDesc *);
	extern int iscsiAuthClientFinish(struct iscsiAuthClient *);

	extern int iscsiAuthClientRecvBegin(struct iscsiAuthClient *);
	extern int iscsiAuthClientRecvEnd(struct iscsiAuthClient *,
					  IscsiAuthClientCallback *, void *,
					  void *);

	extern const char *iscsiAuthClientGetKeyName(int);
	extern int iscsiAuthClientGetNextKeyType(int *);
	extern int iscsiAuthClientKeyNameToKeyType(const char *);
	extern int iscsiAuthClientRecvKeyValue(struct iscsiAuthClient *, int,
					       const char *);
	extern int iscsiAuthClientSendKeyValue(struct iscsiAuthClient *, int, 
					       int *, char *, unsigned int);
	extern int iscsiAuthClientRecvTransitBit(struct iscsiAuthClient *, int);
	extern int iscsiAuthClientSendTransitBit(struct iscsiAuthClient *, 
						 int *);

	extern int iscsiAuthClientSetAuthMethodList(struct iscsiAuthClient *,
						    unsigned int, const int *);
	extern int iscsiAuthClientSetAuthMethodNegRole(struct iscsiAuthClient *,
						       int);
	extern int iscsiAuthClientSetChapAlgorithmList(struct iscsiAuthClient *,
						       unsigned int,
						       const int *);
	extern int iscsiAuthClientSetUsername(struct iscsiAuthClient *, 
					      const char *);
	extern int iscsiAuthClientSetPassword(struct iscsiAuthClient *,
					      const unsigned char *,
					      unsigned int);
	extern int iscsiAuthClientSetAuthRemote(struct iscsiAuthClient *, int);
	extern int iscsiAuthClientSetGlueHandle(struct iscsiAuthClient *, 
					        void *);
	extern int iscsiAuthClientSetMethodListName(struct iscsiAuthClient *,
						    const char *);
	extern int iscsiAuthClientSetIpSec(struct iscsiAuthClient *, int);
	extern int iscsiAuthClientSetBase64(struct iscsiAuthClient *, int);
	extern int iscsiAuthClientSetChapChallengeLength
						(struct iscsiAuthClient *,
						 unsigned int);
	extern int iscsiAuthClientSetVersion(struct iscsiAuthClient *, int);
	extern int iscsiAuthClientCheckPasswordNeeded(struct iscsiAuthClient *, 
						      int *);

	extern int iscsiAuthClientGetAuthPhase(struct iscsiAuthClient *, int *);
	extern int iscsiAuthClientGetAuthStatus(struct iscsiAuthClient *, 
					        int *);
	extern int iscsiAuthClientAuthStatusPass(int);
	extern int iscsiAuthClientGetAuthMethod(struct iscsiAuthClient *, 
					        int *);
	extern int iscsiAuthClientGetChapAlgorithm(struct iscsiAuthClient *, 
						   int *);
	extern int iscsiAuthClientGetChapUsername(struct iscsiAuthClient *, 
						  char *, unsigned int);

	extern int iscsiAuthClientSendStatusCode(struct iscsiAuthClient *, 
						 int *);
	extern int iscsiAuthClientGetDebugStatus(struct iscsiAuthClient *, 
						 int *);
	extern const char *iscsiAuthClientDebugStatusToText(int);

/*
 * The following is called by platform dependent code.
 */
	extern void iscsiAuthClientAuthResponse(struct iscsiAuthClient *, int);

/*
 * The following routines are considered platform dependent,
 * and need to be implemented for use by iscsiAuthClient.c.
 */

	extern int iscsiAuthClientChapAuthRequest(struct iscsiAuthClient *, 
						  char *, unsigned int, 
						  unsigned char *, unsigned int,
						  unsigned char *,
						  unsigned int);
	extern void iscsiAuthClientChapAuthCancel(struct iscsiAuthClient *);

	extern int iscsiAuthClientTextToNumber(const char *, unsigned long *);
	extern void iscsiAuthClientNumberToText(unsigned long, char *,
						unsigned int);

	extern void iscsiAuthRandomSetData(unsigned char *, unsigned int);
#ifndef __KERNEL__
	extern void iscsiAuthMd5Init(struct MD5Context *);
	extern void iscsiAuthMd5Update(struct MD5Context *, unsigned char *,
				       unsigned int);
	extern void iscsiAuthMd5Final(unsigned char *, struct MD5Context *);
#endif

	extern int iscsiAuthClientData(unsigned char *, unsigned int *,
				       unsigned char *, unsigned int);

#ifdef __cplusplus
}
#endif
#endif				/* #ifndef ISCSIAUTHCLIENT_H */
