#! /bin/sh
#
# This script is responsible for uninstalling the iSCSI driver
#

BINDIRS="${ROOT}/sbin ${ROOT}/sbin"
MANDIRS="${ROOT}/man ${ROOT}/man"

KVER=`uname -r`

if [ `whoami` != root ]; then
	echo "You must be root to remove the device driver!"
	exit 1
fi

echo "Removing iSCSI driver for Linux $KVER"
echo " "

# find dirs that might contain driver files
for initdir in ${ROOT}/etc/rc.d/init.d ${ROOT}/etc/init.d
do
    if [ -x $initdir/iscsi ]; then
	# try to find where the relocatable files were installed
	DIR=`awk -F= '$1=="BASEDIR" { print $2}' $initdir/iscsi`
	BINDIRS="$BINDIRS ${ROOT}$DIR/sbin"
	MANDIRS="$MANDIRS ${ROOT}$DIR/man"
    fi
done

# stop the driver if it's running
for initdir in ${ROOT}/etc/rc.d/init.d ${ROOT}/etc/init.d
do
    if [ -x $initdir/iscsi ]; then
	$initdir/iscsi stop
	break
    fi
done

# shutdown all sessions in the kernel module
if [ -d /sys/class/scsi_host/iscsi ] ; then
    echo "shutdown" > /sys/class/scsi_host/iscsi/shutdown
fi

# ensure all the kernel threads are really dead
killall -9 iscsi-rx > /dev/null 2>&1
killall -9 iscsi-timer > /dev/null 2>&1

# Unload and remove the iSCSI kernel module
rmmod iscsi > /dev/null 2>&1
for dir in ${ROOT}/lib/modules ${ROOT}/usr/lib/modules ${ROOT}/lib/modules/$KVER/misc
do
    if [ -f $dir/iscsi.ko ] ; then
	rm -f $dir/iscsi.ko
    fi
    # remove the old name as well
    if [ -f $dir/iscsi_mod.o ] ; then
	rm -f $dir/iscsi_mod.o
    fi
done
#
# Remove the init.d script and links
#
for initdir in ${ROOT}/etc/rc.d/init.d ${ROOT}/etc/init.d ${ROOT}/etc/rc.d
do
    if [ -d $initdir ]; then
	for dir in `ls -d $initdir/rc*.d 2> /dev/null`
	do
	    if [ -d $dir ] ; then
		for file in `ls $dir/*iscsi 2> /dev/null | egrep '(S|K)[0-9]+iscsi'`
		do
		    rm -f $file
		done
	    fi
	done
    fi
done

for initdir in ${ROOT}/etc/rc.d/init.d ${ROOT}/etc/init.d ${ROOT}/etc/rc.d
do
    if [ -f $initdir/iscsi ] ; then
        rm -f $initdir/iscsi
    fi
    if [ -f $initdir/rc.iscsi ] ; then
        rm -f $initdir/rc.iscsi
    fi
done

# Remove the iSCSI CALLOUT entry from udev.rules file

sed '/iscsi-id/d' /etc/udev/udev.rules > /tmp/udev.rules
mv /tmp/udev.rules /etc/udev/udev.rules

# Remove iSCSI permissions entry from udev.permissions

sed '/iscsi/d' /etc/udev/udev.permissions > /tmp/udev.permissions
mv /tmp/udev.permissions /etc/udev/udev.permissions

#
# Remove the iscsi daemon
#
for dir in $BINDIRS
do
    if [ -f $dir/iscsid ] ; then
	rm -f $dir/iscsid
    fi
done

#
# Remove the iscsi LUN activator
#
for dir in $BINDIRS
do
    if [ -f $dir/iscsilun ] ; then
	rm -f $dir/iscsilun
    fi
done


#
# Remove the iscsi-device utility
#
for dir in $BINDIRS
do
    if [ -f $dir/iscsi-device ] ; then
	rm -f $dir/iscsi-device
    fi
done

#
# Remove the iscsi-iname utility
#
for dir in $BINDIRS
do
    if [ -f $dir/iscsi-iname ] ; then
	rm -f $dir/iscsi-iname
    fi
done

#
# Remove the iscsi-ls utility
#
for dir in $BINDIRS
do
    if [ -f $dir/iscsi-ls ]; then
	rm -f $dir/iscsi-ls
    fi
done

#
# Remove the iscsi-id utility
#
for dir in $BINDIRS
do
    if [ -f $dir/iscsi-id ]; then
	rm -f $dir/iscsi-id
    fi
done

#
# Remove the iscsi-umountall utility
#
for dir in $BINDIRS
do
    if [ -f $dir/iscsi-umountall ] ; then
	rm -f $dir/iscsi-umountall
    fi
done

#
# Remove the iscsi-mountall utility
#
for dir in $BINDIRS
do
    if [ -f $dir/iscsi-mountall ] ; then
	rm -f $dir/iscsi-mountall
    fi
done

# Remove the man pages
for dir in $MANDIRS
do
    if [ -f $dir/man8/iscsid.8 ] ; then
	rm -f $dir/man8/iscsid.8
    fi
    if [ -f $dir/man5/iscsi.conf.5 ] ; then
	rm -f $dir/man5/iscsi.conf.5
    fi
    if [ -f $dir/man5/iscsi.bindings.5 ] ; then
	rm -f $dir/man5/iscsi.bindings.5
    fi
    if [ -f $dir/man1/iscsi-ls.1 ] ; then
	rm -f $dir/man1/iscsi-ls.1
    fi
done
