#ifndef TEXTKEY_H_
#define TEXTKEY_H_

/*
 * iSCSI connection daemon
 * Copyright (C) 2001 Cisco Systems, Inc.
 * maintained by linux-iscsi@cisco.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * See the file COPYING included with this distribution for more details.
 *
 * $Id: textkey.hpp,v 1.2 2001/08/08 15:13:59 smferris Exp $ 
 *
 * textkey.hpp
 *
 *    iSCSI text command block
 * 
 */

typedef unsigned char   UINT8;
typedef unsigned short  UINT16;
typedef unsigned int    UINT32;                                                 

// Proto
struct TextKeyPair;

//
// iSCSI Text Command Block
//

class TextKey {

public:

    // Construct a text command block from a rcvd packet
    TextKey(char *buf, int textBlockLen);

    // Construct an empty text holder
    TextKey();

    // Destructor
    ~TextKey();

    // Search for the specified key and return the value if found (NULL if not)
    char *find(char *key);

    // Add an entry
    void add(char *key, char *val);

    // Get a copy of the text command block.  The length of the text block
    // is returned in len.  If len == 0, NULL is returned.  If len > 0,
    // the caller is responsible for freeing the memory returned.
    char *get(int &len);

    // Compare two strings.  Returns 0 if the same, -1 if not.
    static int cmp(char *t1, char *t2);

    // Show the key:val pairs stored in this text block
    void show(int indent);

    // The number of key:val pairs store in this text block
    int count_m;

private:

    char *textBlock_m;

    struct TextKeyPair *first_m;
    struct TextKeyPair *last_m;
};

#endif
