#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/videodev.h>
#include <linux/interrupt.h>

#include "../drivers/video/matrox/matroxfb_base.h"
#include "../drivers/video/matrox/matroxfb_maven.h"
#include "mgacore.h"

#include "mgabes.h"
#include "mgacodec.h"

#include "media/audiochip.h" //hmmm..
#include "media/tuner.h"  //hmmm..


#ifndef	MAP_NR
#define	MAP_NR(x)	virt_to_page(x)
#endif



MODULE_PARM(mgacore_debug, "i");

int	mgacore_debug = 0;

#undef dprintk
#define dprintk if (mgacore_debug) printk

static struct list_head mgadev;

/* /proc/mgavideo directory */
static struct proc_dir_entry *mga_procdir;

static struct mga_pci_id_s
{
	unsigned short dev_id;
	int mga_type;
	const char *name;
} mga_pci_ids[] = {
	{ PCI_DEVICE_ID_MATROX_G400,		MGA_G400,	"MGA G400" },
	{ PCI_DEVICE_ID_MATROX_G200_AGP,	MGA_G200,	"MGA G200 AGP" },
	{ PCI_DEVICE_ID_MATROX_G200_PCI,	MGA_G200,	"MGA G200 PCI" },
};



/* * PRIVATE FUNCTIOM *
 *	mga_interrupt_handler -	main interrupt handler
 *	@mga:		mga instance
 *	@dev_id:	
 *	@regs:
 *
 *	This function handles interrupt requests. It will call
 *	registered sub-handlers for each specific interrupt.
 *	
 *	See also:
 *	mga_request_irq()
 *
 *	mga_release_irq()
 *
 *	Notes:
 *	mga_interrupt_hw() is responsible for clearing the
 *	interrupt. Sub-handlers should not worry about it.
 */
static irqreturn_t 
mga_interrupt_handler(int irq, void *dev_id, struct pt_regs *regs)
{
	u32 status, vstatus;
	struct mga_dev_s *mga = dev_id;
	int i, j, k;

	if (mga == NULL) {
		printk("mgacore: mga interrupt handler says *mga is NULL!\n");
		return IRQ_NONE;
	}

	spin_lock(&mga->irq_lock);

	for (i=0; i<8; i++) {

		/* read status register */
		vstatus = mga_read(mga, VSTATUS);
		status  = mga_read(mga, STATUS);

		if ( ((vstatus & mga->enabled_irq) == 0) && ((status & mga->enabled_core_irq) == 0) ) {
			/* from other dev on shared irq line? */
			spin_unlock(&mga->irq_lock);
			return IRQ_NONE;
		}
		mga->status = vstatus;
		
		if (i!=0) {
			dprintk( "mgacore: isr iteration %i, %x, %x\n", i, vstatus, status );
		}

		/* handle VIN irqs */
		for (j=0; j<MGA_MAX_VIRQ; j++) {
			if ((vstatus & (1<<j)) & (mga->enabled_irq & (1<<j))) {
			
				mga_write(mga, VICLEAR, (1<<j));

				for ( k=0; k<MGA_MAX_ISR; k++ ) {
					if (mga->irq_handler[j][k])
						(*mga->irq_handler[j][k])(mga, regs);
				}
			}
		}
				

		if (vstatus & (1<<MGA_IRQ_DCMPEOI)) {
			printk("dmcpi interrupt seen\n");
		}
		

		/* handle core irqs */
		for (j=0; j<MGA_MAX_IRQ; j++) {
			if ((status & (1<<j)) & (mga->enabled_core_irq & (1<<j))) {
				dprintk( "handle core irq 0x%x\n", 1<<j );
				for ( k=0; k<MGA_MAX_ISR; k++ ) {
					if (mga->core_irq_handler[j][k])
						(*mga->core_irq_handler[j][k])(mga, regs);
				}
				/* clear it, except when reserved bit */
				mga_write(mga, ICLEAR, (1<<j) & 0x0fa5 );
			}
		}
		
		if (status & (1<<MGA_CORE_IRQ_EXTERNAL)) {
			/* for now, disable external ints, since they
			won't shut up properly yet. (both i33 and i34 would
			need exec_commands to do that, but that doesn't
			work from within isr's yet */
			mga_write(mga, IEN,
				(mga->enabled_core_irq & ~(1<<MGA_CORE_IRQ_EXTERNAL))
				| mga->saved_core_irq );
			
			printk( "external int\n" );
		}
		
	}
	      
	spin_unlock(&mga->irq_lock);

        return IRQ_HANDLED;
}

/**
 *	mga_request_irq - install an interrupt handler for PCI IRQ
 *	@mga:		mga instance
 *	@handler:	interrupt handler function
 *
 *	This function requests the PCI IRQ used by the card from
 *	the kernel and installs an interrupt handler to manage
 *	all of the various IRQ events that may occur.
 *
 *	All card interrupts are disabled BEFORE the request so that
 *	no spurious interrupts will occur before any of the
 *	sub-handlers have been initialized.
 *	
 *	If another handler is already installed, this function will
 *	NOT overwrite it. The other handler must be released before.
 *
 *	IRQs:
 *	%MGA_IRQ_VINVSYNC
 *
 *	%MGA_IRQ_DCMPEOI
 *
 *	%MGA_IRQ_DCMPEOI
 *
 *	%MGA_IRQ_EXTERNAL
 *
 *	Return value:
 *	Returns 0 on success.
 *
 *	Returns %-ENODEV on failure.
 *
 *	See also:
 *			mga_release_irq()
 *			mga_enable_irq()
 *			mga_disable_irq()
 */
static int 
mga_request_irq(struct mga_dev_s *mga)
{
	unsigned long flags;
	u32 vien_chk;

	spin_lock_irqsave(&mga->irq_lock, flags);

	if (!mga->irq_inuse)
	{
		/* see G200 spec, Table 4-8 on p 4-115
		   clear and disable the 4 MGA video IRQ sources. this may
		   be overkill but better too much caution than not enough.
		   -inkling */
		mga_write(mga, VIEN, 0);
		mga_write(mga, VICLEAR, MGA_IRQ_ALLMASK);

		/* all MGA Video interrupts should be disabled now. we
		   need to check them and if they are not disabled,
		   return an error for total abort of process
		   -inkling */
		vien_chk = mga_read(mga, VIEN) & MGA_IRQ_ALLMASK; /* mask reserved bits */

		if (vien_chk)
		{
			spin_unlock_irqrestore(&mga->irq_lock, flags);
			printk("mgacore: PCI IRQ request could not clear VIEN (0x0%1X)\n",
				vien_chk);
			return -ENODEV;
		}

		mga->saved_core_irq = mga_read(mga, IEN) & 0x0fe5;
		mga->enabled_core_irq = 0;

		if (request_irq(mga->dev->irq, mga_interrupt_handler,
				SA_INTERRUPT | SA_SHIRQ, "mgacore", mga) < 0)
		{
			spin_unlock_irqrestore(&mga->irq_lock, flags);

			printk("mgacore: denied PCI IRQ %d\n",  mga->dev->irq);
			return -ENODEV;
		}
		mga->irq_inuse = 1; /* set the IRQ In Use counter to 1 */
		dprintk("mgacore: PCI IRQ %d handler registered\n",
			mga->dev->irq);
	} else {
		printk("mgacore: PCI IRQ %d handler is already registered\n",
			mga->dev->irq);
	}
	spin_unlock_irqrestore(&mga->irq_lock, flags);

	return 0;
}

/**
 *	mga_release_irq - release a PCI IRQ interrupt handler
 *	@mga:	mga instance
 *
 *	This function releases the PCI IRQ handler used by the card
 *	after disabling all enabled interrupt sources on the card.
 *	Any associated sub-handlers are indicated upon release.
 *
 *	See also:
 *			mga_request_irq()
 *			mga_enable_irq()
 *			mga_disable_irq()
 *
 *	FIXME: all the sub-handler pointers should also be set to NULL.
 * 		it's not really a crucial step, but more a neatness thing
 *		-inkling
 */
static int 
mga_release_irq(struct mga_dev_s *mga)
{
	unsigned long flags;
	u32 vien_chk;

	spin_lock_irqsave(&mga->irq_lock, flags);

	if (mga->irq_inuse)
	{
		dprintk("mgacore: releasing PCI IRQ %d handler\n",
			mga->dev->irq);

		if (mga->irq_inuse > 1)
		{
			printk("mgacore: %d IRQ sub-handlers will be affected!\n",
				mga->irq_inuse - 1);
		}
		/* see G200 spec, Table 4-8 on p 4-115
		   clear and disable the 4 MGA video IRQ sources. this may
		   be overkill but better too much caution than not enough.
		   -inkling */
		mga_write(mga, VIEN, 0);
		mga_write(mga, VICLEAR, MGA_IRQ_ALLMASK);

		/* all MGA Video interrupts should be disabled now. now we
		   need to check them and if they are not disabled,
		   return an error
		   -inkling */
		
		vien_chk = mga_read(mga, VIEN) & MGA_IRQ_ALLMASK; /* mask reserved bits */

		if (vien_chk)
		{
			spin_unlock_irqrestore(&mga->irq_lock, flags);
			printk("mgacore: PCI IRQ %d release could not clear VIEN (0x0%1X)\n",
				mga->dev->irq, vien_chk);
			return -ENODEV;
		}
		
		/* restore original IEN value */
		mga_write(mga, IEN, mga->saved_core_irq );


		free_irq(mga->dev->irq, mga);
		dprintk("mgacore: PCI IRQ %d released\n", mga->dev->irq);
	} else {
		printk("mgacore: no registered PCI IRQ handler to release!\n");
	}			

	spin_unlock_irqrestore(&mga->irq_lock, flags);

	return 0;
}

/**
 *	mga_enable_irq - enable an interrupt source to use a sub-handler
 *	@mga:		mga instance
 *	@irq:		IRQ to enable
 *
 *	This function enables the hardware to generate interrupts from
 *	the specified source.
 *
 *	This will NOT allow you to replace a sub-handler.  You must disable
 *	it first if you want to replace it with a new sub-handler.
 *
 *	FIXME: We may want to modify this to allow sub-handler replacement.
 *	-inkling	
 *
 *	IRQs:
 *	%MGA_IRQ_VINVSYNC
 *
 *	%MGA_IRQ_BUFLVL
 *
 *	%MGA_IRQ_DCMPEOI
 *
 *	%MGA_IRQ_EXTERNAL
 *
 *	Return value:
 *	Returns 0 on success.
 *
 *	Returns %-ENODEV on failure.
 *
 *	See also:
 *			mga_disable_irq()
 *			mga_request_irq()
 *			mga_release_irq()
 */
int mga_enable_irq(struct mga_dev_s *mga, int irq, mga_interrupt_handler_t sub_handler)
{
	unsigned long flags;
 	u32 vien_chk;
	int	i;
	
	if (irq < 0 || irq >= MGA_MAX_VIRQ) {
		dprintk( "invalid irq %i\n", irq );
		return 1;
	}

	spin_lock_irqsave(&mga->irq_lock, flags);

 
	/* next we assign the sub-handler pointer and increment in_use count.
	   no change and no error (except a klog msg) if a sub-handler
	   is already registered and try to re-enable the MGA Video IRQ.
	   -inkling */
	for(i=0; i < MGA_MAX_ISR; i++) {
		if (mga->irq_handler[irq][i] == 0)
			break;
	}

	if (i < MGA_MAX_ISR) {
		mga->irq_handler[irq][i] = sub_handler;
		mga->enabled_irq |= (1<<irq) & MGA_IRQ_ALLMASK;
		mga->irq_inuse++;
	} else {
		printk("mgacore: sub-handler for VIRQ 0x%1X already exists!\n",
			irq);
	}

	/* last step is to clear again any pending video irq (cautious)
	   and enable the interrupt to use the handler
	   -inkling */
	if ( i == 0 )
		mga_write(mga, VICLEAR, (1<<irq));
	mga_write(mga, VIEN, mga->enabled_irq );

	/* check to see if it actually did enable the interrupt.
	   return failure code if it did NOT (re)enable the interrupt.
	   -inkling */
	vien_chk = mga_read(mga, VIEN);

	if ( vien_chk != mga->enabled_irq ) {
		dprintk("mgacore: VIRQs %x enabled, but wanted %x\n", vien_chk, mga->enabled_irq );
	}

	spin_unlock_irqrestore(&mga->irq_lock, flags);

	return 0;
}


void
mga_check_irqs( struct mga_dev_s *mga )
{
	unsigned long flags;
 	u32 vien_chk;
	u32 vien;

	spin_lock_irqsave(&mga->irq_lock, flags);

	vien = mga->enabled_irq;
	mga_write(mga, VIEN, vien );
	
	vien_chk = mga_read(mga, VIEN) & MGA_IRQ_ALLMASK;

	spin_unlock_irqrestore(&mga->irq_lock, flags);
	
	if ( vien_chk != vien ) {
		dprintk("mgacore: check_irqs %x enabled, wanted %x\n", vien_chk, vien );
	}
}


/**
 *	mga_disable_irq - disable an interrupt source
 *	@mga:	mga instance
 *	@irq:	IRQ to disable
 *
 *	This function disables the generation of hardware interrupts from
 *	the specified source.
 *
 *	The associated interrupt handler IS uninstalled by this
 *	function.
 *
 *	Return value:
 *	Returns 0 on success.
 *
 *	Returns %-ENODEV on failure.
 *
 *	See also:
 *			mga_enable_irq()
 *			mga_request_irq()
 *			mga_release_irq()
 */
int mga_disable_irq(struct mga_dev_s *mga, int irq, mga_interrupt_handler_t sub_handler)
{
	unsigned long flags;
	int i;
	u32 vien;
	
	if (irq < 0 || irq >= MGA_MAX_VIRQ) {
		dprintk( "invalid irq %i\n", irq );
		return 1;
	}

	spin_lock_irqsave(&mga->irq_lock, flags);

	for(i=0; i < MGA_MAX_ISR; i++) {
		if (mga->irq_handler[irq][i] == sub_handler)
			break;
	}

	if (i < MGA_MAX_ISR) {
		mga->irq_handler[irq][i] = NULL;
		mga->irq_inuse--;
		for(i=0; i < MGA_MAX_ISR; i++) {
			if (mga->irq_handler[irq][i])
				break;
		}
		if (i == MGA_MAX_ISR) {
			mga->enabled_irq &= ~(1<<irq);
			mga_write(mga, VIEN, mga->enabled_irq );
			mga_write(mga, VICLEAR, (1<<irq));
			vien = mga_read(mga, VIEN);
			if (vien & (1<<irq)) {
				spin_unlock_irqrestore(&mga->irq_lock, flags);
				printk("mgacore: disable VIRQ is unable to clear VIRQ %d\n",
					irq);
				return -ENODEV;
			}		
		}
	} else {
		printk("mgacore: no sub-handler for VIRQ 0x%1X to remove!\n",
			irq);
	}

	spin_unlock_irqrestore(&mga->irq_lock, flags);
	return 0;
}



int mga_enable_core_irq(struct mga_dev_s *mga, int irq, mga_interrupt_handler_t sub_handler)
{
	unsigned long flags;
	int	i;
	
	if ( irq<0 || irq >= MGA_MAX_IRQ ) {
		dprintk( "invalid irq %i\n", irq );
		return 1;
	}

	spin_lock_irqsave(&mga->irq_lock, flags);

	for(i=0; i < MGA_MAX_ISR; i++) {
		if (mga->core_irq_handler[irq][i] == 0)
			break;
	}

	if (i < MGA_MAX_ISR) {
		mga->core_irq_handler[irq][i] = sub_handler;
		mga->enabled_core_irq |= 1 << irq;
		mga->irq_inuse++;
	} else {
		printk("mgacore: sub-handler for IRQ 0x%1X already exists!\n",
			irq);
	}

	if ( i == 0 )
		mga_write(mga, ICLEAR, (1<<irq) & 0x0fa5 );
	mga_write(mga, IEN, mga->enabled_core_irq | mga->saved_core_irq );

	spin_unlock_irqrestore(&mga->irq_lock, flags);

	return 0;
}


int mga_disable_core_irq(struct mga_dev_s *mga, int irq, mga_interrupt_handler_t sub_handler)
{
	unsigned long flags;
	int i;
	
	if ( irq<0 || irq >= MGA_MAX_IRQ ) {
		dprintk( "invalid irq %i\n", irq );
		return 1;
	}

	spin_lock_irqsave(&mga->irq_lock, flags);

	for(i=0; i < MGA_MAX_ISR; i++) {
		if (mga->core_irq_handler[irq][i] == sub_handler)
			break;
	}

	if (i < MGA_MAX_ISR) {
		mga->core_irq_handler[irq][i] = NULL;
		mga->irq_inuse--;
		for(i=0; i < MGA_MAX_ISR; i++) {
			if (mga->core_irq_handler[irq][i])
				break;
		}
		if (i == MGA_MAX_ISR) {
			mga->enabled_core_irq &= ~(1<<irq);
			mga_write(mga, IEN, mga->enabled_core_irq | mga->saved_core_irq);
			mga_write(mga, ICLEAR, (1<<irq) & 0x0fa5 );
		}
	} else {
		printk("mgacore: no sub-handler for IRQ 0x%1X to remove!\n",
			irq);
	}

	spin_unlock_irqrestore(&mga->irq_lock, flags);
	return 0;
}


struct proc_dir_entry	*
mga_create_proc_entry(struct mga_dev_s *mga, char *name,
	read_proc_t *rp, write_proc_t *wp, void *data)
{
	struct proc_dir_entry	*entry;

	if ((entry = create_proc_entry(name, S_IFREG | S_IRUGO | S_IWUSR,
		mga->proc_directory)) == NULL) {
		return NULL;
	}

	entry->data = data;
	entry->read_proc = rp;
	entry->write_proc = wp;

	return entry;
}

void mga_remove_proc_entry(struct mga_dev_s *mga, char *name)
{
	if (mga_procdir) {
		if (mga->proc_directory) {
			remove_proc_entry(name, mga->proc_directory);
		}
	}
}



#define MMALLOC_MAGIC1	((u32)0xfeedcafe)
#define MMALLOC_MAGIC2	((u32)0xdeafbabe)
#define MMA_PUT_U32( addr, u ) writel((u), (u8*)mga->fb + (addr))
#define MMA_GET_U32( addr, u ) *(u) = readl((u8*)mga->fb + (addr))

/**
 *	mga_alloc_buffer - 	allocate a buffer
 *	@mga:	mga instance
 *	@size:	size of the buffer to allocate
 *
 *	This is a really simple memory manager for the framebuffer.
 *	It justs decrements a pointer from the end of the framebuffer
 *	and DOES NOT really manage an array of allocated blocks.
 *	
 *	An allocated block must be deallocated in the SAME ORDER (with
 *	respect to other modules requesting memory) for this to work.
 *
 *	Return value:	returns a pointer to the allocated buffer
 *
 *	See also:	mga_free_buffer()
 *
 *	Notes:	The memory address is expressed as a value *inside*
 *	the framebuffer address space. For example a buffer at address
 *	0x0 represents the beginning of the video framebuffer.
 *
 *
 *	Allocation is now slightly safer. Blocklayout:
 *	{
 *		u32 MAGIC1
 *		u32 requested_size
 *		u32 ptr_to_end_of_block
 *		u8  aligned_user_block[ size ]
 *		u8  alignment_filler[n]
 *		u32 MAGIC2
 *	}
 *		--lvs
 */
u32 mga_alloc_buffer(struct mga_dev_s *mga, unsigned long size, unsigned long alignment)
{
	unsigned long bsize = size + alignment + 4*sizeof(u32);
	u32 rv;
	u32 saved_limit;
	unsigned long flags;

	spin_lock_irqsave( &mga->malloc_lock, flags );

	if (mga->malloc_limit < bsize) {
		spin_unlock_irqrestore( &mga->malloc_lock, flags );
		return 0;
	}

	mga->malloc_limit -= sizeof(u32);
	MMA_PUT_U32( mga->malloc_limit, MMALLOC_MAGIC2 );

	saved_limit = mga->malloc_limit;
	
	mga->malloc_limit -= size;
	mga->malloc_limit &= -alignment;
	rv = mga->malloc_limit;
	
	mga->malloc_limit -= sizeof(u32);
	MMA_PUT_U32( mga->malloc_limit, saved_limit );
	
	mga->malloc_limit -= sizeof(u32);
	MMA_PUT_U32( mga->malloc_limit, size );
	
	mga->malloc_limit -= sizeof(u32);
	MMA_PUT_U32( mga->malloc_limit, MMALLOC_MAGIC1 );

	memset_io((u8*)mga->fb + rv, 0xeb, size );
	
	dprintk("mgacore: alloc %ld (%#lx) at %08x\n", size, size, rv);

	spin_unlock_irqrestore( &mga->malloc_lock, flags );

	return rv;
}

/**
 *	mga_free_buffer - 	deallocate a buffer
 *	@mga:	mga instance
 *	@size:	size of the block to allocate
 *
 *	Increments the allocation counter, to simulate memory
 *	deallocation.
 *
 *	An allocated block must be deallocated in the SAME ORDER (with
 *	respect to other modules requesting memory) for this to work.
 *
 *	See also:	mga_alloc_buffer()
 */
void mga_free_buffer(struct mga_dev_s *mga, unsigned long size)
{
	u32 u;
	unsigned long flags;

	spin_lock_irqsave( &mga->malloc_lock, flags );
	
	MMA_GET_U32( mga->malloc_limit, &u );
	if ( u != MMALLOC_MAGIC1 ) {
		dprintk( "mgacore: block corruption at begin\n" );
		spin_unlock_irqrestore( &mga->malloc_lock, flags );
		return;
	}
	mga->malloc_limit += sizeof(u32);

	MMA_GET_U32( mga->malloc_limit, &u );
	if ( u != size ) {
		dprintk( "mgacore: not freeing own block\n" );
		// not fatal; alloc/free order mixed up
	}
	mga->malloc_limit += sizeof(u32);
	
	MMA_GET_U32( mga->malloc_limit, &u );
	mga->malloc_limit = u;
	
	MMA_GET_U32( mga->malloc_limit, &u );
	if ( u != MMALLOC_MAGIC2 ) {
		dprintk( "mgacore: block corruption at end\n" );
		//maybe we get lucky
	}
	mga->malloc_limit += sizeof(u32);

	dprintk("mgacore: free %ld (%#lx)\n", size, size);

	spin_unlock_irqrestore( &mga->malloc_lock, flags );
}


static void
mga_malloc_init( struct mga_dev_s *mga )
{
	spin_lock_init(&mga->malloc_lock);

	mga->malloc_limit = mga->memory_size;

	mga->malloc_limit -= sizeof(u32);
	MMA_PUT_U32( mga->malloc_limit, MMALLOC_MAGIC2 );
}


static void
mga_malloc_check( struct mga_dev_s *mga )
{
	u32 u;
	u32 limit;
	unsigned long flags;

	spin_lock_irqsave( &mga->malloc_lock, flags );

	limit = mga->malloc_limit;

	while ( 1==1 ) {	
		MMA_GET_U32( limit, &u );
		limit += sizeof(u32);
		if ( u == MMALLOC_MAGIC2 )
			break;
		if ( u == MMALLOC_MAGIC1 ) {
			u32 size;
			
			MMA_GET_U32( limit, &size );
			limit += sizeof(u32);
		
			dprintk( "mgacore: leaked block 0x%x\n", size );

			dprintk( "mgacore: old limit: %x\n", limit );
			MMA_GET_U32( limit, &limit );
			
			dprintk( "mgacore: new limit: %x\n", limit );
			
			MMA_GET_U32( limit, &u );
			limit += sizeof(u32);
			
			if ( u != MMALLOC_MAGIC2 ) {
				dprintk( "mgacore: block corruption at end\n" );
				break;
			}
		}
		else {
			dprintk( "mgacore: malloc corruption\n" );
			break;
		}
	}

	spin_unlock_irqrestore( &mga->malloc_lock, flags );
}

void *mga_rvmalloc(unsigned long size)
{
        void *mem;
        unsigned long adr;

        size = PAGE_ALIGN(size);
        mem = vmalloc_32 (size);
        if (!mem)
                return NULL;

        memset (mem, 0, size);  /* Clear the ram out, no junk to the user */
        adr = (unsigned long) mem;
        while (size > 0) {
                SetPageReserved(vmalloc_to_page((void *)adr));
                adr += PAGE_SIZE;
                size -= PAGE_SIZE;
        }
        return mem;
}

void mga_rvfree(void *mem, unsigned long size)
{
        unsigned long adr;

        if (!mem)
                return;

        adr = (unsigned long) mem;
        while ((long) size > 0) {
                ClearPageReserved(vmalloc_to_page((void *)adr));
                adr += PAGE_SIZE;
                size -= PAGE_SIZE;
        }
        vfree (mem);
}


/**
 *	mga_display_pitch - returns bytes per line
 *	@mga:	mga instance
*/
int
mga_display_pitch(struct mga_dev_s *mga)
{
	int tmp;
	int offs;
	int pitch;
	int inter;
	
	MGA_REGLOCK(mga);
	
	mgax_writeb(mga, CRTCADD, 0x13);		/* offset */
	offs = mgax_readb( mga, CRTCREG );	/* offset 7:0 */

	mgax_writeb(mga, CRTCEXTADD, 0x00);	/* addr gen extensions */
	tmp = mgax_readb( mga, CRTCEXTREG);
	offs |= (tmp&0x30)<<4;			/* offset 9:8 */

	mgax_writeb(mga, CRTCADD, 0x14);		/*underline loc */
	tmp = mgax_readb( mga, CRTCREG );
	if ( tmp & 0x40 ) {			/* dword<6>*/
		pitch = offs * 32;		/*fixme, probably wrong*/
	}
	else {
		mgax_writeb(mga, CRTCADD, 0x17);	/* crtc mode control */
		tmp = mgax_readb( mga, CRTCREG );
		if (tmp & 0x40) {		/* wbmode<6> */
			pitch = offs * 8;
		} else {
			pitch = offs * 16;	/*fixme, probably wrong*/
		}
	}

	mgax_writeb(mga, CRTCEXTADD, 0x00);
        tmp  = mgax_readb(mga, CRTCEXTREG);
	inter = (tmp & 0x80) != 0;
	if (!inter) /* not interlaced */
		pitch *= 2;

	MGA_REGUNLOCK(mga);
	
	dprintk("display pitch %d (offs %d, i%d)\n", pitch, offs, inter);
	
	return pitch;
}


#define BPP_8         0x00
#define BPP_15        0x01
#define BPP_16        0x02
#define BPP_24        0x03
#define BPP_32_DIR    0x04
#define BPP_32_PAL    0x07

void
mga_display_size(struct mga_dev_s *mga, int *width, int *height, int *depth)
{
	int	vtotal;
	int	htotal;
	int	dtotal;
	int	tmp;

	MGA_REGLOCK(mga);

	/* Read HDisplay form hardware (width >> 3) -1 */
	mgax_writeb(mga, CRTCADD, 0x1);
	htotal  = mgax_readb(mga, CRTCREG) + 1;

	mgax_writeb(mga, CRTCEXTADD, 0x01);
        tmp  = mgax_readb(mga, CRTCEXTREG);
	htotal += (tmp & 0x02) ? 0x100 : 0;
	htotal <<= 3;

	mgax_writeb(mga, CRTCADD, 0x12);
        vtotal  = mgax_readb(mga, CRTCREG) + 1;
	mgax_writeb(mga, CRTCADD, 0x07);
        tmp  = mgax_readb(mga, CRTCREG);

	vtotal += (tmp & 0x02) ? 256 : 0;
	vtotal += (tmp & 0x40) ? 512 : 0;
	mgax_writeb(mga, CRTCEXTADD, 0x02);
        tmp  = mgax_readb(mga, CRTCEXTREG);
	vtotal += (tmp & 0x04) ? 1024 : 0;

	mgax_writeb(mga, CRTCEXTADD, 0x00);
        tmp  = mgax_readb(mga, CRTCEXTREG);
	if (tmp & 0x80) /* interlaced */
		vtotal *= 2;

	switch (mga_indirect_readb(mga, XMULCTRL)) {
	case BPP_8:
		dtotal = 8;
		break;
	case BPP_15:
		dtotal = 15;
		break;
	case BPP_16:
		dtotal = 16;
		break;
	case BPP_24:
		dtotal = 24;
		break;
	case BPP_32_DIR:
	case BPP_32_PAL:
		dtotal = 32;
		break;
	default:
		dtotal = 0;
	}
	MGA_REGUNLOCK(mga);

	dprintk("display %d x %d depth %d\n", htotal, vtotal, dtotal);

	*width = htotal;
	*height = vtotal;
	*depth = dtotal;
}



int
mga_i2c_control( struct mga_dev_s *mga, int driver_id, unsigned int cmd, void *arg )
{
        struct list_head  *item;
        struct i2c_client *client;
	struct i2c_adapter *adap = mga->fb_i2c;

        down(&adap->clist_lock);
        list_for_each(item,&adap->clients) {
                client = list_entry(item, struct i2c_client, list);
		if (client->driver->id != driver_id)
                        continue;
                if (!try_module_get(client->driver->owner))
                        continue;
                if (NULL != client->driver->command) {
                        int ret;
                        up(&adap->clist_lock);
                        ret = client->driver->command(client,cmd,arg);
                        module_put(client->driver->owner);
                        return ret;
                }
                module_put(client->driver->owner);
       }
       up(&adap->clist_lock);

       return -1;
}


void
mga_set_audioinput(struct mga_dev_s *mga, int channel)
{
	if (mga->options & MGAVIDEO_HAS_AUDIO) {
		mga_i2c_control(mga,
			I2C_DRIVERID_MSP3400, 
			AUDC_SET_INPUT, 
			&channel);
	}
	mga->msp_input = channel;
}


int
mga_set_defaults(struct mga_dev_s *mga)
{
	mga_set_audioinput(mga, AUDIO_EXTERN);
	return 0;
}

int
mga_get_vipformat(struct mga_dev_s *mga, struct mgabus_video_input_s *fmt)
{
	if (mga->vip_fmt_valid) {
		memcpy(fmt, &mga->vip_format, sizeof(*fmt));
		return 0;
	}
	return 1;
}

void mgagrab_vipnotify(struct mga_grab_s *video); ///

void
mga_set_vipformat(struct mga_dev_s *mga, struct mgabus_video_input_s *fmt)
{
	memcpy(&mga->vip_format, fmt, sizeof(mga->vip_format));
	mga->vip_fmt_valid = 1;
	
	mgagrab_vipnotify(mga->grab); ///
}





/**
 *	mga_get_card -	return a device structure
 *
 *	This function returns a pointer to a &mga_dev_s device.
 */
struct mga_dev_s*
mga_get_card(int num)
{
	struct mga_dev_s *mga;
	struct list_head * h;
	
	list_for_each(h, &mgadev) {
		mga = list_entry(h, struct mga_dev_s, devlist);
		if (num == 0) {
			mga->in_use++; //fixme
			MOD_INC_USE_COUNT;
			return mga;
		}
		num--;
	}
	return NULL;
}


/**
 *	mga_release_card -	reset the usage count of the card
 *	@mga:	mga instance
 *
 *	This function resets the usage counter of the device.
 */
void mga_release_card(struct mga_dev_s *mga)
{
	if (mga) {
		mga->in_use--;
		MOD_DEC_USE_COUNT;
	}
}

int mga_card_lock(struct mga_dev_s *mga)
{
	return test_and_set_bit(0, &mga->card_lock);
}


void mga_card_unlock(struct mga_dev_s *mga)
{
	mga->card_lock = 0;
}


static void
mga_detect_tuner(struct mga_dev_s *mga, unsigned tuner_addr)
{
	int msp, tuner;

	if(mga->options & MGAVIDEO_HAS_TUNER) {
		
		msp = 0;
		if(mga->options & MGAVIDEO_HAS_AUDIO) {
			/* not implemented in msp3400.c
			mga_i2c_control(mga,
				I2C_DRIVERID_MSP3400,
				AUDC_GET_UNIT, &msp);
			*/
		}

		tuner = TUNER_ABSENT;

		switch(tuner_addr) {
		case 0x60:
			tuner = TUNER_PHILIPS_SECAM; /* RRG, PAL version */
			if((msp&0xff) == 0x07) 
			{ 
			/* Usa audio *//* Samsung NTSC - unsupported by tuner.c? */
			} 
			else 
			{
			dprintk("_detect_tvt set the tuner to PAL/SECAM !\n");
			/* 20000525/gz: Supposedly G400:s PAL/SECAM tuner */
			tuner = TUNER_PHILIPS_SECAM; /* RRG, PAL version */
			}; break;

			break;

		case 0x61:
			tuner = TUNER_PHILIPS_NTSC;

			if( (msp&0xff) == 0x07) { /* Usa audio */
				tuner = TUNER_PHILIPS_NTSC;
			} else {
			/* Samsung SECAM - unsupported by tuner.c? */
			}
			break;
	
		case 0x62:
			tuner = TUNER_PHILIPS_PAL_I;
			break;

		case 0x63:
			tuner = TUNER_PHILIPS_SECAM;

			if((msp&0xff) == 0x07) { /* Usa audio */
			/* Samsung NTSC - unsupported by tuner.c? */
			} else {
			tuner = TUNER_PHILIPS_SECAM; /* RRG, PAL version */
			}; break;
		default:
			printk("Unknown tuner address 0x%x - can't autodetect tuner\n", tuner_addr);
	       
		}
		dprintk("_detect_tvt: tuner_addr is 0x%x\n", tuner_addr);
                mga_i2c_control(mga, I2C_DRIVERID_TUNER,
			TUNER_SET_TYPE, &tuner);
        }
}

// Ughhh..  Should not have copy of matrox_fb private data structure..
struct matroxfb_dh_maven_info {
	struct i2c_bit_adapter	maven;
	struct i2c_bit_adapter	ddc1;
	struct i2c_bit_adapter	ddc2;
};

static int
detectI2Cstuff(struct mga_dev_s* mga)
{
	struct matroxfb_dh_maven_info* m2info = NULL;
        struct list_head  *item;
	struct i2c_adapter * adap;
	unsigned i;
	unsigned addr;
	
	for (i=0; i<mga->minfo->drivers_count; i++) {
		if (mga->minfo->drivers[i] != NULL) {
			if (strcmp(mga->minfo->drivers[i]->name, "i2c-matroxfb") == 0) {
				m2info = mga->minfo->drivers_data[i];
				break;
			}
		}
	}
	if (m2info == NULL) {
		printk("m2info not found\n");
		return 1;
	}
	adap = &m2info->maven.adapter;
	
	mga->fb_i2c = adap;


        down(&adap->clist_lock);
        list_for_each(item,&adap->clients) {
                struct i2c_client *client;
                client = list_entry(item, struct i2c_client, list);
		
		addr = client->addr;
		switch (client->driver->id) {

		case I2C_DRIVERID_MSP3400:
			mga->options |= MGAVIDEO_HAS_AUDIO;
			break;

		case I2C_DRIVERID_TUNER:
			mga->options |= MGAVIDEO_HAS_TUNER;
			mga_detect_tuner(mga, addr);
			break;

		case I2C_DRIVERID_KS0127:
			mga->options |= MGAVIDEO_HAS_DECODER;
			if (addr == 0xD8>>1) {
				dprintk("%s: Detected KS0127 in RRG fashion\n", __FUNCTION__);
				mga->options |= MGAVIDEO_HAS_RRG;
			}
			else if (addr == 0xDA>>1) {
				dprintk("%s: Detected KS0127 in Marvel fashion\n", __FUNCTION__);
				mga->options |= MGAVIDEO_HAS_MARVEL;
			}
			break;

		case I2C_DRIVERID_MGATVO:
			mga->options |= MGAVIDEO_HAS_ENCODER;
			break;

		default:
			printk("mgai2c: unknown device@%x ", addr);
			break;
		}
	}
        up(&adap->clist_lock);
	return 0;
}


/**
	matroxfb_mjpg_probe
*/
static void*
matroxfb_mjpg_probe(struct matrox_fb_info* minfo)
{
	struct mga_dev_s* mga;
	int i;
	static int card = 0;
	int type = MGA_G200; // fixme

        mga = (struct mga_dev_s*)kmalloc(sizeof(*mga), GFP_KERNEL);
        if (!mga) {
                printk(KERN_ERR "mga: no mem\n");
                return NULL;
        }
        memset(mga, 0, sizeof(*mga));

	for (i=0; i<sizeof(mga_pci_ids)/sizeof(mga_pci_ids[0]); i++) {
		if (minfo->pcidev->device == mga_pci_ids[i].dev_id) {
			printk("mgacore: Found %s\n", mga_pci_ids[i].name );
			type = mga_pci_ids[i].mga_type;
		}
	}

	mga->minfo = minfo;
	mga->dev = minfo->pcidev;
	mga->card_num = card++;
	mga->mga_type = type;
	mga->memory_size = minfo->video.len;
	
	mga->ctrl = vaddr_va( minfo->mmio.vbase );
	mga->fb = vaddr_va( minfo->video.vbase );

	if (detectI2Cstuff(mga) != 0) {
		printk("detectI2Cstuff failed\n");
		kfree(mga);
		return NULL;
	}

	mga_malloc_init(mga);

        /* Create a directory entry for each card  */
	sprintf(mga->name, "mga%d", mga->card_num);
	mga->proc_directory = create_proc_entry(mga->name, S_IFDIR, mga_procdir);

	spin_lock_init(&mga->irq_lock);
	spin_lock_init(&mga->reg_lock);

	mga->maven_std = -1;
        
	if (mga_request_irq(mga) != 0) {
                printk(KERN_ERR "mga: no irq\n");
		kfree(mga);
		return NULL;
	}
	
	mgabes_attach(mga);
	mgacodec_attach(mga);

	mga_display_size(mga, &i, &i, &i);
	mga_display_pitch(mga);

	mga_set_defaults(mga);

	list_add_tail(&mga->devlist, &mgadev);
	
        return mga;
}


static void
matroxfb_mjpg_remove(struct matrox_fb_info* minfo, void* dta)
{
	struct mga_dev_s* mga = dta;
	
	list_del(&mga->devlist);

	if (mga->codec)
		mgacodec_deattach(mga->codec);
	mgabes_deattach(mga->bes);
		
	mga_release_irq(mga);
	
	if (mga_procdir && mga->proc_directory) {
		remove_proc_entry(mga->name, mga_procdir);
	}
	
	mga_malloc_check(mga);
		
	kfree(mga);
}




static struct matroxfb_driver mjpg = {
	.name =		"Matrox MJPEG",
	.probe =	matroxfb_mjpg_probe,
	.remove =	matroxfb_mjpg_remove
};

static int matroxfb_mjpg_init(void)
{
	INIT_LIST_HEAD(&mgadev);
	
        /* Create a directory entry in /proc  (0 is root of proc_fs) */
        mga_procdir = create_proc_entry("mgavideo", S_IFDIR, 0);

        matroxfb_register_driver(&mjpg);
        return 0;
}

static void matroxfb_mjpg_exit(void)
{
        matroxfb_unregister_driver(&mjpg);
	
	remove_proc_entry("mgavideo", 0);
}


EXPORT_SYMBOL(mga_display_size);
EXPORT_SYMBOL(mga_get_card);
EXPORT_SYMBOL(mga_release_card);
EXPORT_SYMBOL(mga_alloc_buffer);
EXPORT_SYMBOL(mga_free_buffer);
EXPORT_SYMBOL(mga_enable_irq);
EXPORT_SYMBOL(mga_disable_irq);
EXPORT_SYMBOL(mga_create_proc_entry);
EXPORT_SYMBOL(mga_remove_proc_entry);
EXPORT_SYMBOL(mga_rvmalloc);
EXPORT_SYMBOL(mga_rvfree);
EXPORT_SYMBOL(mga_check_irqs);
EXPORT_SYMBOL(mga_display_pitch);
EXPORT_SYMBOL(mga_enable_core_irq);
EXPORT_SYMBOL(mga_disable_core_irq);
EXPORT_SYMBOL(mga_card_lock);
EXPORT_SYMBOL(mga_card_unlock);
EXPORT_SYMBOL(mga_i2c_control);
EXPORT_SYMBOL(mga_set_defaults);
EXPORT_SYMBOL(mga_set_audioinput);

EXPORT_SYMBOL(mga_set_vipformat);
EXPORT_SYMBOL(mga_get_vipformat);

MODULE_AUTHOR("(c) 1999-2003 various <marvel.sf.net>");
MODULE_DESCRIPTION("Matrox MJPEG driver");
MODULE_LICENSE("GPL");

module_init(matroxfb_mjpg_init);
module_exit(matroxfb_mjpg_exit);

