/*
 *
 *    Copyright 1995-1999 Regents of the University of Minnesota
 *    Portions Copyright 2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */
#ifndef _BLKLIST_H
#define _BLKLIST_H

ogfs_rgrpd_t *ogfs_blk2rgrpd(ogfs_sbd_t * sdp, uint64 blk);
ogfs_rgrpd_t *ogfs_rgrpd_get_by_num(ogfs_sbd_t * sdp, unsigned int num);
ogfs_rgrpd_t *ogfs_rgrpd_get_with_hint(ogfs_sbd_t * sdp, uint64 hint);

void __ogfs_rgrpd_put(ogfs_rgrpd_t * rgd);
static __inline__ void
ogfs_rgrpd_put(ogfs_rgrpd_t * rgd)
{
	ENTER(GFN_RGRPD_PUT);
	if (atomic_dec_and_test(&rgd->rd_count))
		__ogfs_rgrpd_put(rgd);
	EXIT(GFN_RGRPD_PUT);
}

void ogfs_clear_rgrpd(ogfs_sbd_t * sdp);

int ogfs_rindex_hold(ogfs_sbd_t * sdp);
void ogfs_rindex_release(ogfs_sbd_t * sdp);

int ogfs_rgrp_read(ogfs_rgrpd_t * rgd);
void ogfs_rgrp_relse(ogfs_rgrpd_t * rgd);
void ogfs_rgrp_verify(ogfs_rgrpd_t * rgd);

void ogfs_rgrp_lvb_fill(ogfs_rgrpd_t * rgd);
int ogfs_rgrp_lvb_init(ogfs_rgrpd_t * rgd, int force);

int ogfs_inplace_reserv(ogfs_inode_t * ip, int accept_partial_fit);
void ogfs_inplace_relse(ogfs_inode_t * ip);

int ogfs_blkalloc(ogfs_inode_t * ip, uint64 * block);
int ogfs_metaalloc(ogfs_inode_t * ip, uint64 * block);
void ogfs_dipick(ogfs_sbd_t * sdp, ogfs_ipreserv_t * res, ogfs_inum_t * inum);
int ogfs_dialloc(ogfs_sbd_t * sdp, ogfs_ipreserv_t * res,
		ogfs_inum_t * expected_inum, ogfs_glock_t * expected_gl);

int ogfs_blkfree(ogfs_inode_t * ip, uint64 bstart, uint32 blen);
int ogfs_metafree(ogfs_inode_t * ip, uint64 bstart, uint32 blen);
int ogfs_difree(ogfs_inode_t * ip, ogfs_rgrpd_t * rgd);

struct ogfs_rgrp_list {
	unsigned int rl_rgrps;
	ogfs_rgrpd_t **rl_list;
	unsigned int rl_space;
};
typedef struct ogfs_rgrp_list ogfs_rgrp_list_t;

int ogfs_rlist_add(ogfs_sbd_t * sdp, ogfs_rgrp_list_t * rlist, uint64 block);
void ogfs_rlist_sort(ogfs_rgrp_list_t * rlist);
void ogfs_rlist_free(ogfs_rgrp_list_t * rlist);
int ogfs_reclaim_one(ogfs_sbd_t *sdp, unsigned int rnum);

#define	BFITNOENT	(0xFFFFFFFF)

void ogfs_setbit(unsigned char *buffer, unsigned int buflen,
		uint32 block, unsigned char new_state);
uint32 ogfs_bitfit(unsigned char *buffer, unsigned int buflen,
		  uint32 goal, unsigned char old_state);
uint32 ogfs_travel_bitmap(unsigned char *buffer, unsigned int buflen,
		    unsigned char old_state, unsigned char new_state);

#endif /* _BLKLIST_H */
