/*
 *
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */

#include "ogfs_debug.h"
#include "ogfs.h"

#ifdef DEBUG_PRINT

/**
 * ogfs_icdb_inc
 * @add:
 *
 */

void
ogfs_icdb_inc(unsigned int add)
{
	ogfs_icdb_point += add;

	if (ogfs_icdb_point > OGFS_ICDB_BUF_LEN)
		osi_panic("OGFS:  debug buffer overflow\n");

	if (ogfs_icdb_point + OGFS_ICDB_BUF_WIDTH > OGFS_ICDB_BUF_LEN) {
		memset(ogfs_icdb_buf + ogfs_icdb_point, 0,
			   OGFS_ICDB_BUF_LEN - ogfs_icdb_point);
		ogfs_icdb_point = 0;
	}
}

/**
 * ogfs_icdb_print
 *
 */

void
ogfs_icdb_print()
{
	unsigned int x;

	printk("\nDebug log:\n\n");

	down(&ogfs_icdb_lock);

	for (x = ogfs_icdb_point; x < OGFS_ICDB_BUF_LEN; x++)
		if (ogfs_icdb_buf[x])
			printk("%c", ogfs_icdb_buf[x]);

	for (x = 0; x < ogfs_icdb_point; x++)
		if (ogfs_icdb_buf[x])
			printk("%c", ogfs_icdb_buf[x]);

	up(&ogfs_icdb_lock);

	printk("\n");
}

#endif				/*  DEBUG_PRINT  */

#ifdef DEBUG_PROFILE

/**
 * ogfs_memcpy - Copy memory
 * @dest: the destination pointer
 * @src: the source pointer
 * @size: the number of bytes to copy
 *
 */

void
ogfs_memcpy(void *dest, const void *src, unsigned int size)
{
	ENTER(GFN_MEMCPY);

	memcpy(dest, src, size);

	EXIT(GFN_MEMCPY);
}

/**
 * ogfs_memset - fill memory with a pattern
 * @str: the memory pointer
 * @pattern: the pattern to fill with
 * @size: the number of bytes to fill
 *
 */

void
ogfs_memset(void *dest, unsigned int pattern, unsigned int size)
{
	ENTER(GFN_MEMSET);

	memset(dest, pattern, size);

	EXIT(GFN_MEMSET);
}

#endif				/*  DEBUG_PROFILE  */
