/*
 *
 *    Copyright 2000-2001 Sistina Software, Inc.
 *    Portions Copyright 2003 The OpenGFS Project
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */
#ifndef _DEBUG_H
#define _DEBUG_H

#ifdef DEBUG_PRINT

#define OGFS_ICDB_BUF_LEN (1048576)
#define OGFS_ICDB_BUF_WIDTH (256)

extern char ogfs_icdb_buf[OGFS_ICDB_BUF_LEN];
extern unsigned int ogfs_icdb_point INIT(0);
extern struct semaphore ogfs_icdb_lock;

void ogfs_icdb_inc(unsigned int add);
void ogfs_icdb_print(void);

#ifdef DEBUG_PRINT_SYSLOG
#define db_printf(fmt, args...) do { printk(fmt, ##args); } while(0)
#else
#define db_printf(fmt, args...) \
do { \
	down(&ogfs_icdb_lock); \
	ogfs_icdb_inc(sprintf(ogfs_icdb_buf + ogfs_icdb_point, fmt, ##args)); \
	up(&ogfs_icdb_lock); \
} while(0)
#endif

#else				/*  DEBUG_PRINT  */
#define db_printf(fmt, args...)
#define ogfs_icdb_print()
#endif				/*  DEBUG_PRINT  */

#if DEBUG_TRACE

#define ENTER_PRINT(x) \
do { \
	if (ogfs_debug_flags[(x) / 8] & (1 << ((x) % 8))) \
		db_printf(\
			"En %s %ld\n", ogfs_debug_flag_names[(x)], \
			(long int)osi_pid()); \
} while (0);

#define EXIT_PRINT(x) \
do { \
	if (ogfs_debug_flags[(x) / 8] & (1 << ((x) % 8))) \
		db_printf( \
			"Ex %s %ld\n", ogfs_debug_flag_names[(x)], \
			(long int)osi_pid()); \
} while (0);

#define IFDB(x, todo) \
do { \
	if (ogfs_debug_flags[(x) / 8] & (1 << ((x) % 8))) \
		{todo} \
} while (0);

extern unsigned char ogfs_debug_flags[DEBUG_FLAG_BYTES];

#else				/*  DEBUG_TRACE  */
#define ENTER_PRINT(x)
#define EXIT_PRINT(x)
#define IFDB(x, todo)
#endif				/*  DEBUG_TRACE  */

#ifdef DEBUG_PROFILE

extern ogfs_profile_t ogfs_prof_stats[DEBUG_FLAGS];
extern uint64 ogfs_prof_neg_count INIT(0);

#define ENTER_PROFILE(x) \
	uint64 ogfs_profile_time = osi_gettimeofday(); \

#define EXIT_PROFILE(x) \
do { \
	uint64 stop_time = osi_gettimeofday(); \
	ogfs_profile_time = ogfs_profile_time; \
	if (stop_time >= ogfs_profile_time) \
		ogfs_profile_time = stop_time - ogfs_profile_time; \
	else { \
		ogfs_profile_time = 0; \
		ogfs_prof_neg_count++; \
	} \
	ogfs_prof_stats[(x)].total_calls++; \
	ogfs_prof_stats[(x)].total_micros += ogfs_profile_time; \
	if (ogfs_prof_stats[(x)].min_micros > ogfs_profile_time) \
		ogfs_prof_stats[(x)].min_micros = ogfs_profile_time; \
	if (ogfs_prof_stats[(x)].max_micros < ogfs_profile_time) \
		ogfs_prof_stats[(x)].max_micros = ogfs_profile_time; \
} while (0);

#define PROFILE_TAP(x) \
do { \
	ogfs_prof_stats[(x)].total_calls++; \
	ogfs_prof_stats[(x)].total_micros = 1; \
	ogfs_prof_stats[(x)].min_micros = 1; \
	ogfs_prof_stats[(x)].max_micros = 1; \
} while (0);

void ogfs_memcpy(void *dest, const void *src, unsigned int size);
void ogfs_memset(void *dest, unsigned int pattern, unsigned int size);

#else				/*  DEBUG_PROFILE  */
#define ENTER_PROFILE(x)
#define EXIT_PROFILE(x)
#define PROFILE_TAP(x)
#define ogfs_memcpy memcpy
#define ogfs_memset memset
#endif				/*  DEBUG_PROFILE  */

#define DBTAG(x)
#define ENTER(x) ENTER_PROFILE((x)); ENTER_PRINT((x));
#define ENTER2(x) ENTER(x)
#define EXIT(x) EXIT_PRINT((x)); EXIT_PROFILE((x));
#define DO_PROF(x, todo) \
do { \
	ENTER_PROFILE((x)); \
	ENTER_PRINT((x)); \
	{todo} \
	EXIT_PRINT((x)); \
	EXIT_PROFILE((x)); \
} while (0);
#define DO_PROF2(x, todo) DO_PROF(x, todo)

#define con_printf(fmt, args...) \
do { \
	printk(fmt, ##args); \
	db_printf(fmt, ##args); \
} while(0)

#endif /* _DEBUG_H */
