/*
 *
 *    Copyright 1995-1999 Regents of the University of Minnesota
 *    Portions Copyright 2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */
#ifndef _FILE_H
#define _FILE_H

typedef int (*read_copy_fn_t) (struct buffer_head * bh, char *buf, int offset, int size);
typedef int (*write_copy_fn_t) (ogfs_inode_t * ip, struct buffer_head * bh,
				const char *buf, int offset, int size, int new);

int ogfs_copy2mem(struct buffer_head * bh, char *buf, int offset, int size);
int ogfs_copy2user(struct buffer_head * bh, char *buf, int offset, int size);
int64 ogfs_readi(ogfs_inode_t * ip, char *buf, uint64 offset, uint64 size,
		read_copy_fn_t copy_fn);

int ogfs_copy_from_mem(ogfs_inode_t * ip, struct buffer_head * bh, const char *buf,
		      int offset, int size, int new);
int ogfs_copy_from_user(ogfs_inode_t * ip, struct buffer_head * bh, const char *buf,
		       int offset, int size, int new);
int64 ogfs_writei(ogfs_inode_t * ip, const char *buf, uint64 offset, uint64 size,
		 write_copy_fn_t copy_fn);

extern struct file_operations ogfs_file_fops;
extern struct file_operations ogfs_dir_fops;

static __inline__ int64
ogfs_internal_read(ogfs_inode_t * ip, char *buf, uint64 offset, uint64 size)
{
	return ogfs_readi(ip, buf, offset, size, ogfs_copy2mem);
}

static __inline__ int64
ogfs_internal_write(ogfs_inode_t * ip, const char *buf, uint64 offset,
		   uint64 size)
{
	return ogfs_writei(ip, buf, offset, size, ogfs_copy_from_mem);
}

#endif /* _FILE_H */
