/*
 *
 *    Copyright 1995-1999 Regents of the University of Minnesota
 *    Portion Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 */

/*
 * gmalloc accounting code
 */
#ifndef _GMALLOC_H
#define _GMALLOC_H

#ifdef DEBUG_MEM

#define GMALLOC_HASH_SIZE 4096
extern struct list_head gmalloc_head[GMALLOC_HASH_SIZE];

struct gmalloc_stats {
	unsigned int ct;
	unsigned int ct_max;
	unsigned int ct_total;
	unsigned int bytes;
	unsigned int bytes_max;
	uint64 bytes_total;
};
typedef struct gmalloc_stats gmalloc_stats_t;

extern gmalloc_stats_t gm_stats;
extern spinlock_t gmalloc_lock;

void *gmalloc_wrapper(unsigned int size, char *file, unsigned int line);
void gfree_wrapper(void *data, unsigned int size, char *file,
		   unsigned int line);
void gmalloc_check(void);

#define gmalloc(size) gmalloc_wrapper(size, __FILE__, __LINE__)
#define gfree(data, size) gfree_wrapper(data, size, __FILE__, __LINE__)

#else				/*  DEBUG_MEM  */

#define gmalloc_check()

#define gmalloc(size) osi_malloc(size)
#define gfree(data, size) osi_free(data, size)

#endif				/*  DEBUG_MEM  */

/*  Slab cache allocator prototypes  */

ogfs_glock_t *ogfs_alloc_glock(void);
void ogfs_free_glock(ogfs_glock_t * gl);

ogfs_inode_t *ogfs_alloc_inode(void);
void ogfs_free_inode(ogfs_inode_t * ip);

ogfs_bufdata_t *ogfs_alloc_bufdata(void);
void ogfs_free_bufdata(ogfs_bufdata_t *);

ogfs_rgrpd_t *ogfs_alloc_rgrpd(void);
void ogfs_free_rgrpd(ogfs_rgrpd_t * rgrp);

#endif /* _GMALLOC_H */
