/*
 *
 *    Copyright 1995-1999 Regents of the University of Minnesota
 *    Portions Copyright 1999-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */
#ifndef _INODE_H
#define _INODE_H

extern int ogfs_copyin_dinode(ogfs_inode_t * ip);

extern ogfs_inode_t *ogfs_get_istruct(ogfs_sbd_t * sdp, ogfs_glock_t * gl,
			     ogfs_inum_t * inum, int create);
static __inline__ void
ogfs_hold_istruct(ogfs_inode_t * ip)
{
	atomic_inc(&ip->i_count);
}

extern void ogfs_put_istruct(ogfs_inode_t * ip);
extern void ogfs_destroy_istruct(ogfs_inode_t * ip);

extern int ogfs_dealloc_inodes(ogfs_sbd_t * sdp);
extern int ogfs_limit_unlinked(ogfs_sbd_t * sdp);
extern int ogfs_change_nlink(ogfs_inode_t * ip, int diff);

extern int ogfs_lookupi(ogfs_inode_t * dip, struct qstr * name,
		int unlock, ogfs_inode_t ** ipp);
extern int ogfs_createi(ogfs_inode_t * dip, struct qstr * name,
		unsigned int type, int exclusive, int *new, ogfs_inode_t ** ipp);
extern int ogfs_unlinki(ogfs_inode_t * dip, struct qstr * dname, ogfs_inode_t * ip);
extern int ogfs_rmdiri(ogfs_inode_t * dip, struct qstr * dname, ogfs_inode_t * ip);

extern int ogfs_update_atime(ogfs_inode_t * ip);
#define OGFS_UPDATE_ATIME(ogfs_inode) (((ogfs_inode)->i_sbd->sd_noatime) ? 0 : ogfs_update_atime((ogfs_inode)))

/*  Symlink stuff  */

struct ogfs_symlink_ops {
	const char *name;
	const unsigned int len;
	unsigned int (*subst_fn) (char *str);
};

#define OGFS_FAST_SYMLINK_SIZE (256)

extern unsigned int ogfs_link_size(struct ogfs_symlink_ops *ops, char *s);
extern void ogfs_link_expand(struct ogfs_symlink_ops *ops, char *out, char *in);

/*  Inlines  */

static __inline__ int
ogfs_is_stuffed(ogfs_inode_t * ip)
{
	return !ip->i_di.di_height;
}

static __inline__ int
ogfs_is_jdata(ogfs_inode_t * ip)
{
	return ip->i_di.di_flags & OGFS_DIF_JDATA;
}

/*  These are architecture specific functions
    Either implement them or stub them out in your arch_* directory.  */

extern void ogfs_inode_attr_in(ogfs_inode_t * ip);
extern int ogfs_permission(ogfs_inode_t * ip, int mask);
extern void ogfs_drop_excess_inodes(ogfs_sbd_t * sdp);

#endif /* _INODE_H */
