/*
 *
 *    Copyright 2001 The OpenGFS Project
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */
#ifndef _KCOMPAT_H
#define _KCOMPAT_H

/*
 * This file contains roubtines to use newest kernel APIs even
 * on older kernels that lack them.
 */

#include <linux/version.h>
#include <linux/module.h>


/*
 * Newer kernel allow to specify a module license, older don't.
 */
#ifndef MODULE_LICENSE
#define MODULE_LICENSE(x)
#endif

/*
 * The possible GFP_ flags changed during the 2.4 kernel series.
 */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,6))
#define GFP_NOFS GFP_BUFFER
#endif

/*
 * In older kernels the drivers had to modify the gendisk chain
 * themselves.
 */
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,10))

#include <linux/genhd.h>

/*
 * gendisk helper routines from 2.4.10-ac1
 * (in fact they were introduced in 2.4.9-ac, but we can't test for that..)
 * This should work for 2.4.9 and 2.4.9-ac5
 * DANGER for 2.4.9-ac10 these functions should not be here and need locking.
 */
static __inline void
add_gendisk(struct gendisk *gp)
{
	/* write_lock(&gendisk_lock); */
	gp->next = gendisk_head;
	gendisk_head = gp;
	/* write_unlock(&gendisk_lock); */
}

static __inline void
del_gendisk(struct gendisk *gp)
{
	struct gendisk **gpp;

	/* write_lock(&gendisk_lock); */
	for (gpp = &gendisk_head; *gpp; gpp = &((*gpp)->next))
		if (*gpp == gp)
			break;
	if (*gpp)
		*gpp = (*gpp)->next;
	/* write_unlock(&gendisk_lock); */
}

static __inline int
walk_gendisk(int (*walk)(struct gendisk *, void *), void *data)
{
	struct gendisk *gp;
	int error = 0;

	/* read_lock(&gendisk_lock); */
	for (gp = gendisk_head; gp; gp = gp->next)
		if ((error = walk(gp, data)))
			break;
	/* read_unlock(&gendisk_lock); */

	return error;
}

#endif
#endif  /*  _KCOMPAT_H */
