/*
 *
 *    Copyright 1999-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */

#include "ogfs_debug.h"
#include "ogfs.h"
#include "dio.h"
#include "glock.h"
#include "locking.h"
#include "super.h"

#warning big fixme
int lm_mount(char *proto_name,
             char *table_name, char *host_data,
             lm_callback_t cb, lm_fsdata_t * fsdata,
             lm_lockstruct_t * lockstruct)
{
	return 0;
}

/**
 * ogfs_mount_lockproto - mount a locking protocol
 * @sdp: the filesystem
 * @args: mount arguements
 * @silent: if TRUE, don't complain if the FS isn't an OGFS fs
 *
 * Returns: 0 on success, -EXXX on failure
 */

int
ogfs_mount_lockproto(ogfs_sbd_t * sdp, int silent)
{
	ogfs_sb_t sb;
	struct buffer_head *bh;
	char proto[256], table[256], host[256];
	uint64 block;
	int error;

	ENTER(GFN_MOUNT_LOCKPROTO);
	/*  Copy in values from "args"  */
	strncpy(proto, sdp->sd_args.ar_lockproto, sizeof(proto));
	proto[sizeof(proto) - 1] = 0;
	strncpy(table, sdp->sd_args.ar_locktable, sizeof(table));
	table[sizeof(table) - 1] = 0;
	strncpy(host, sdp->sd_args.ar_hostdata, sizeof(host));
	host[sizeof(host) - 1] = 0;

	/*  Try to autodetect  */
	if (!proto[0] || !table[0]) {
		block = OGFS_SB_ADDR * OGFS_BASIC_BLOCK / sdp->sd_sb.sb_bsize;
		bh = ogfs_dread(
			sdp, block, NULL, DR_FORCE | DR_START | DR_WAIT);
		if (!bh) {
			EXIT(GFN_MOUNT_LOCKPROTO);
			return -ENOBUFS;
		}
		ogfs_sb_in(&sb, BH_DATA(bh));
		brelse(bh);
		error = ogfs_check_sb(sdp, &sb, silent);
		if (error) {
			EXIT(GFN_MOUNT_LOCKPROTO);
			return error;
		}
		if (!proto[0])
			strcpy(proto, sb.sb_lockproto);

		if (!table[0])
			strcpy(table, sb.sb_locktable);
	}
	DO_PROF(LM_MOUNT,
		error = lm_mount(proto, table, host,
				 ogfs_glock_cb, sdp, &sdp->sd_lockstruct);
	    );
	if (error) {
		con_printf("OGFS:  can't mount proto = %s, table = %s,"
			   " hostdata = %s\n", proto, table, host);
		EXIT(GFN_MOUNT_LOCKPROTO);
		return -1;
	}
	OGFS_ASSERT(sdp->sd_lockstruct.ls_lockspace,);
	OGFS_ASSERT(sdp->sd_lockstruct.ls_ops,);
	EXIT(GFN_MOUNT_LOCKPROTO);

	return 0;
}

/**
 * ogfs_unmount_lockproto - Unmount lock protocol
 * @sdp: The OGFS superblock
 *
 */

void
ogfs_unmount_lockproto(ogfs_sbd_t * sdp)
{
	ENTER(GFN_UNMOUNT_LOCKPROTO);
	DO_PROF(LM_UNMOUNT,
		sdp->sd_lockstruct.ls_ops->unmount(sdp->sd_lockstruct.
						   ls_lockspace);
	    );
	EXIT(GFN_UNMOUNT_LOCKPROTO);
}
