/*
 *
 *    Copyright 1999-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */
#ifndef _MOUNT_H
#define _MOUNT_H

/*  Allow args to be passed to OGFS when using an initial ram disk  */

extern char *ogfs_mount_args INIT(NULL);
extern unsigned int ogfs_mount_args_size INIT(0);
extern struct semaphore ogfs_mount_args_lock;

int ogfs_proc_write(struct file *file, const char *buffer,
		   unsigned long count, void *data);

/*  Values of ar_dir_alloc_method  */

#define OGFS_DAMETH_SINGLE (612324)
#define OGFS_DAMETH_RROBIN (612325)
#define OGFS_DAMETH_RANDOM (612326)

#define OGFS_DAMETH_DEFAULT OGFS_DAMETH_SINGLE
#define OGFS_ATIME_DEFAULT (60)

struct ogfs_args {
	char *ar_target;	/* The device the FS is mounted on */

	char ar_lockproto[256];	/* The name of the Lock Protocol */
	char ar_locktable[256];	/* The name of the Lock Table */
	char ar_hostdata[256];	/* The host specific data */

	unsigned int ar_dir_alloc_method;

	int ar_ignore_local_fs;	/* Ignore the local_fs field in the lm_lockops_t */
	int ar_localflocks;	/* let the VFS do flock|fcntl locks for us */
	int ar_localcaching;	/* Local-style caching (dangerous on mulithost) */

	unsigned int ar_buf_readahead;	/* Number of FS blocks to readahead */
	unsigned int ar_atime_quantum;	/* maximum difference allowed before atime is updated */

	unsigned int ar_upgrade;	/* Upgrade ondisk/multihost format */
};
typedef struct ogfs_args ogfs_args_t;

int ogfs_make_args(char *data, ogfs_args_t * args);

#endif /* _MOUNT_H*/
