/*
 *
 *    Copyright 1995-1999 Regents of the University of Minnesota
 *    Portions Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */
#ifndef _OGFS_H
#define _OGFS_H

/*  OS specific files  */

#include "osi.h"

/*  Extern Macro  */

#ifndef EXTERN
#define EXTERN extern
#define INIT(X)
#else
#undef EXTERN
#define EXTERN
#define INIT(X) =X
#endif

/*  Includes  */

#include "global.h"

/*  Needed for incore.h  */
#include "ogfs_ondisk.h"
#include "lm_interface.h"
#include "lvb.h"
#include "list.h"
#include "mount.h"

#include "incore.h"

#include "ogfs_ioctl.h"

#include "debug.h"

#include "gmalloc.h"

/*  Macros  */

#ifdef DEBUG_CRASHDUMP
#define OGFS_ICDB_PRINT() ogfs_icdb_print()
#define OGFS_PANIC_MESSAGE ("\n""   _   _           _   _                _ \n""  | | | | ___  ___| | | | __ ___      _| |\n""  | |_| |/ _ \\/ _ \\ |_| |/ _` \\ \\ /\\ / / |\n""  |  _  |  __/  __/  _  | (_| |\\ V  V /|_|\n""  |_| |_|\\___|\\___|_| |_|\\__,_| \\_/\\_/ (_)\n")
#else
#define OGFS_ICDB_PRINT()
#define OGFS_PANIC_MESSAGE ("OGFS: OGFS Panic\n")
#endif				/*  DEBUG_CRASHDUMP  */

#ifdef DEBUG_BUG
/*  Call osi_panic() after osi_bug() to keep the compiler from bitching  */
#define OGFS_PANIC(str) do { con_printf((str)); osi_bug(); osi_panic((str)); } while (0)
#else
#define OGFS_PANIC(str) osi_panic((str));
#endif				/*  DEBUG_BUG  */

#define OGFS_ASSERT(x, todo) \
do \
{ \
  if (!(x)) \
  { \
    con_printf("\nOGFS:  Assertion failed on line %d of file %s\n" \
               "OGFS:  assertion:  \"%s\"\n" \
               "OGFS:  time = %"FMT_64"u\n", \
	       __LINE__, __FILE__, #x, osi_gettimeofday()); \
    {todo} \
    con_printf("\n"); \
    OGFS_ICDB_PRINT(); \
    OGFS_PANIC(OGFS_PANIC_MESSAGE); \
  } \
} \
while (0)

#define NO_CREATE    (0)
#define CREATE       (1)

/*  Divide x by y.  Round up if there is a remainder.  */
#define DIV_RU(x, y) (((x) + (y) - 1) / (y))

#endif /* _OGFS_H */
