/*
 *
 *    Copyright 2000 Lior Amar <lior@cs.huji.ac.il>
 *    Portions Copyright 1999-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */
#ifndef _OGFS_DFSA_H
#define _OGFS_DFSA_H

/*
 * Some declartions needed for the MOSIX dfsa implementations.
 */

#ifdef CONFIG_MOSIX_DFSA

/* debugging flag */
#define DSDEB_DFSA_OGFS 0x4

/* maximun path length kept in the struct ogfs_ident, this length will be enough
 * for most of the pathnames, and for longer pathname sthere is  aspecial
 * treatment 
 */
#define MAX_PATH_LEN 64

/* in the yet unknow dentry we put the file name after the inode and parent
 * inode numbers in the following format /inode/parent_inode/file_name
 * inode numbers are 64bit so 8byte so 16nibles print the inode number with
 * %.16llx format
 * so total of 1 + 16 + 1 + 16 + 1 = 35
 */
#define FILENAME_START 35

/* values for the status field of ogfs_ident. Indicates how much of the PATH name
 * the dentry holds
 */
#define IDENT_FULL_NAME        1
#define IDENT_FILE_NAME        2
#define IDENT_NO_NAME          4

/* total size ib bytes = ino(8) + parent_ino(8) + status(1) + 
 *  first_index(1) + path(64) = 82 bytes.  
 */
struct ogfs_ident {
	uint64 ino;		/* inode number                     */
	uint64 parent_ino;	/* parent inode                     */
	unsigned char status;	/* how much of the name the dentry holds */
	unsigned char first_index;	/* index of the file name in the pathname */
	char path[MAX_PATH_LEN];	/* current path of the dentry so we can 
					   insert the dentry in the correct place  */
};

#endif /* CONFIG_MOSIX_DFSA */
#endif /* _OGFS_DFSA_H */
