/*
 *
 *    Copyright 2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */

#include "ogfs_debug.h"
#include "ogfs.h"

#include <asm/pgalloc.h>
#include <linux/blkdev.h>
#include <linux/mm.h>
#include <linux/fs.h>

#include "dio.h"

spinlock_t *inode_lock;

#if 0

static void __zap_mapping_list(struct vm_area_struct *vma)
{
	for (; vma; vma = vma->vm_next_share) {
		struct mm_struct *mm = vma->vm_mm;
		unsigned long start = vma->vm_start;
		unsigned long end = vma->vm_end;
		unsigned long len = end - start;

		flush_cache_range(mm, start, end);
		zap_page_range(mm, start, len);
		flush_tlb_range(mm, start, end);
	}
}

/**
 * zap_inode_mapping - Invalidate an whole mapping
 * @mapping: mapping to invalidate
 *
 * Invalidate all mappings for the given addressspace
 * to force pagefaults on next access.
 */
void zap_inode_mapping(struct address_space *mapping)
{
	spin_lock(&mapping->i_shared_lock);
	__zap_mapping_list(mapping->i_mmap_shared);
	spin_unlock(&mapping->i_shared_lock);
}

#endif

/**
 * ogfs_inval_pg - Invalidate all pages associated with a glock
 * @sdp: the filesystem
 * @gl: the glock
 *
 */

void
ogfs_inval_pg(ogfs_sbd_t * sdp, ogfs_glock_t * gl)
{
	struct inode *inode;

	ogfs_inode_t *ip = (ogfs_inode_t *) gl->gl_object;
	inode = ip->i_vino;
	invalidate_inode_pages(inode->i_mapping);
#if 0
	ogfs_inode_t *ip = (ogfs_inode_t *) gl->gl_object;
	struct inode *inode;
	spinlock_t	zaplock;
	struct address_space	*mapping	= inode->i_mapping;
	struct vm_area_struct	*vma		= inode->i_mapping->i_mmap_shared;

	ENTER(GFN_INVAL_PG);

	if (ip) {
		inode = ip->i_vino;
		if (inode) {
			//zap_inode_mapping(inode->i_mapping);
			spin_lock(zaplock);
			for (; vma; vma = vma->vm_next) {
				struct mm_struct *mm = vma->vm_mm;
				unsigned long start = vma->vm_start;
				unsigned long end = vma->vm_end;
				unsigned long len = end - start;

				flush_cache_range(vma, start, end);
				zap_page_range(vma, vma->vm_start, (vma->vm_end - vma->vm_start));
				flush_tlb_range(vma, start, end);
			}

			spin_unlock(zaplock);

			if (inode->i_mapping)
				truncate_inode_pages(inode->i_mapping, 0);
		}
	}

	EXIT(GFN_INVAL_PG);
#endif
}

/**
 * ogfs_sync_pg - sync the pages associated with a glock
 * @sdp: the filesystem
 * @gl: the glock
 * @wait:  if TRUE, wait for sync
 *
 * We probably also want to do a filemap_fdatasync() here
 * when we do shared-writeable mappings.
 *
 */

void
ogfs_sync_pg(ogfs_sbd_t * sdp, ogfs_glock_t * gl, int wait)
{
	//ogfs_inode_t *ip = (ogfs_inode_t *) gl->gl_object;
	//unsigned long last = -1;
	//struct writeback_control *wbc = NULL;
	//struct inode *inode = ip->i_vino;

	spin_lock(&inode_lock);
	//sync_sb_inodes(inode, wbc);
	//sync_inodes_sb(sdp->sd_vsb, wait);
#warning fixme probably not right
	blk_run_queues();
	spin_unlock(&inode_lock);


#if 0

	ENTER(GFN_SYNC_PG);

	if (ip) {
		struct inode *inode = ip->i_vino;

		/*
		 * We should probably start worrying about the inode going
		 * away here.
		 */
		if (inode) {
			/*
			 * This code doesn't necessarily handle pages dirtied
			 * by a shared, writeable mapping.  We need something
			 * like filemap_fdatasync() and filemap_fdatawait().
			 *
			 * Even with that, there are still big problems on
			 * glock release because there is a window after this
			 * sync stuff finished, but before zap_inode_mapping()
			 * is completed.
			 */
			if (generic_buffer_fdatasync(inode, 0, last))
				OGFS_ASSERT(0,
					   con_printf
					   ("Panicking because of write error\n"););
		}
	}

	EXIT(GFN_SYNC_PG);
#endif
}
