/*
 *
 *    Copyright 1999-2001 Sistina Software, Inc.
 *    Portions Copyright 2003 Brian Jackson
 *    Portions Copyright 2003 The OpenGFS Project
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 *    This file was maintained by:
 *      David C. Teigland <teigland@sistina.com>
 *
 */
#ifndef _MEMEXP_H
#define _MEMEXP_H

#include "osi.h"

/*  Extern Macro  */

#ifndef EXTERN
#define EXTERN extern
#define INIT(X)
#else
#undef EXTERN
#define EXTERN
#define INIT(X) =X
#endif

/*  Static Macro  */

#ifndef DEBUG_SYMBOLS
#define STATIC static
#else
#define STATIC
#endif

#include <linux/net.h>
//#include <linux/locks.h>
#include <linux/fs.h>

#include "global.h"
#include "cluster_config.h"
#include "lm_interface.h"

#define PROTO_NAME "memexp"

#define ME_ACT_NOP              (0)
#define ME_ACT_U2E		(1)
#define ME_ACT_U2D		(2)
#define ME_ACT_U2S		(3)
#define ME_ACT_E2D		(4)
#define ME_ACT_E2S		(5)
#define ME_ACT_E2U		(6)
#define ME_ACT_D2E		(7)
#define ME_ACT_D2S		(8)
#define ME_ACT_D2U		(9)
#define ME_ACT_S2E		(10)
#define ME_ACT_S2D		(11)
#define ME_ACT_S2U		(12)
#define ME_ACT_X2U		(13)

#define ME_ST_UNLOCKED          (0)
#define ME_ST_EXCLUSIVE         (1)
#define ME_ST_DEFERRED		(2)
#define ME_ST_SHARED            (3)

#define NO_CONV                 (0)
#define YES_CONV                (1)

#define CID_NOT_USED            (0)
#define CID_USED                (1)
#define CID_STOMITH             (2)
#define CID_STOMITH_DONE        (3)
#define CID_EXPIRED_MARKING     (4)
#define CID_EXPIRED             (5)
#define CID_RESETING            (6)

#define OWNER_NONE              (65535)

#define MOUNT_STATE_NULL	(0)
#define MOUNT_STATE_INIT	(1)
#define MOUNT_STATE_RESET	(2)
#define MOUNT_STATE_OK		(3)

/*  Tunable config values  */

#define MAX_CLIENTS             (32)	/* multiple of 8 */
#define DROPLOCK_THRESHOLD      (230)	/* scale between 1 and 255, 230 = 90% */
#define DBUFFER_LENGTH          (2048)	/* per dump in bytes */
#define BIDSIZE			(9)	/* 72 bits div 8 */
#define HEARTBEAT_INTERVAL      (2)	/* seconds between heartbeats */

/*  This is what goes into the buffer for a lock of type CIDBUF  */

struct cidbuf_s {
	uint64 counter;		/* increment this to prove you're alive */
	uint16 cleaner;		/* put your cid here to claim recovery rights */
	unsigned char inuse;	/* state of buffer */
	unsigned char dead;	/* local use only - TRUE if counter hasn't changed */
};
typedef struct cidbuf_s cidbuf_t;

struct memexp;
typedef struct memexp memexp_t;

/* Private "memexp" lock structure */

struct me_lock_s {
	lm_lockname_t lockname;
	unsigned int cancel;
	lm_lvb_t *lvb;
	memexp_t *mep;
};
typedef struct me_lock_s me_lock_t;

/* Results of an LSM lock operation */

struct lsm_status_s {
	uint8 success:1, state:2, conv:1, hvconv:1, extra:3;
	uint8 full;
	uint16 live_holders;
	uint16 expired_holders;
	uint8 cid_list[MAX_CLIENTS / 8];
};
typedef struct lsm_status_s lsm_status_t;

struct ll_memexp_ops;

/* Passed among OGFS, LHS lock module, RHS lock state machine, low level
   transport-specific memexp code, and cluster info code */

struct memexp {
	char table_name[256];	/* lock table name */
	char lockdev_name[256];	/* lock device name */

	lm_callback_t cb;	/* file system callback function */
	lm_fsdata_t *fsdata;	/* private file system data */

	struct file *cfile;	/* cidev - Cluster Information Device */

	unsigned short cb_port;	/* callback port */
	unsigned int jid;	/* local journal id */
	uint16 cid;		/* local client id */
	uint32 node_timeout;	/* seconds for an inactive node to timeout */

	void *cluster_data;	/* cluster-info private data */
	void *lsm_data;		/* mid level lsm private data */
	void *client_data;	/* client state private data */
	void *ll_memexp_data;	/* low level dmep private data */

	struct ll_memexp_ops *ll_memexp_ops;	/* low level lock source functions */

	struct socket *cb_send_sock;	/* inter-machine callbacks */
	struct socket *cb_recv_sock;
	struct semaphore cb_send_lock;

	int cb_recvd_stop;
	struct semaphore cb_recvd_sem;

	int node_monitor_stop;
	struct task_struct *node_monitor_ts;
	struct semaphore node_monitor_sem;

	int lock_refresh_stop;
	struct task_struct *lock_refresh_ts;
	struct semaphore lock_refresh_sem;

	unsigned long cb_meter;	/* used for delay between drop lock callbacks */
	unsigned long heartbeat_time;	/* detect delayed heartbeats */

	struct memexp *next;
};

#ifdef MEP_ASSERT
#undef MEP_ASSERT
#endif
#define MEP_ASSERT(x, act)						\
do {									\
	if (x)								\
		break;							\
	printk("\nmemexp:  Assertion failed on line %d of file %s\n"	\
			"memexp:  assertion:  \"%s\"\n",		\
			__LINE__, __FILE__, #x);			\
	{act}								\
	panic("\nmemexp:  The End...\n");				\
} while (0)

#endif /* _MEMEXP_H */
