/*
 *
 *    Copyright 1999-2001 Sistina Software, Inc.
 *    Portions Copyright 2003 Brian Jackson
 *    Portions Copyright 2003 The OpenGFS Project
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 */
#ifndef _OSI_LINUX24_H
#define _OSI_LINUX24_H


#include <linux/module.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <asm/uaccess.h>
#include <linux/spinlock.h>
#include <asm/atomic.h>
#include <linux/config.h>
//#include <linux/locks.h>
#include <linux/version.h>

//#include "kcompat.h"


/*
 * Memory allocation abstraction
 */
/*
static __inline void *
osi_malloc(unsigned int size)
{
	if (size > 2*PAGE_SIZE)
		return __vmalloc(size, GFP_NOFS|__GFP_HIGHMEM, PAGE_KERNEL);
	return kmalloc(size, GFP_NOFS);
}

static __inline void
osi_free(void *data, unsigned int size)
{
	if (size > 2*PAGE_SIZE)
		vfree(data);
	else
		kfree(data);
}
*/
/*  panic abstractions  */

#ifdef HAVE_HARD_PANIC
# define _osi_panic(x) hard_panic(x)
#else
# define _osi_panic(x) panic(x)
#endif
#ifdef DEBUG_LOOP_PANIC
#define osi_panic(x) \
do { \
	while (1) { \
		current->state = TASK_INTERRUPTIBLE; \
		schedule_timeout(HZ/10); \
	} \
	_osi_panic(x); \
} while (0)
#else
#define osi_panic(x) _osi_panic(x)
#endif

#define osi_bug() BUG()

/*  Process abstraction  */

#define osi_pid() ((long)(current->pid))
#define get_transaction() ((ogfs_trans_t *)current->journal_info)
#define set_transaction(x) (current->journal_info = (void *)(x))

#define osi_wake_up_process(x) wake_up_process(x)
typedef struct task_struct osi_task_t;

/*  Time abstraction  */

#define osi_current_time() (CURRENT_TIME)

static __inline__ uint64_t
osi_gettimeofday(void)
{
	struct timeval tv;
	do_gettimeofday(&tv);
	return ((uint64_t) tv.tv_sec * 1000000 + tv.tv_usec);
}

typedef unsigned long osi_clock_ticks_t;
#define osi_time_stamp(x) (*(x) = jiffies)

static __inline__ int
osi_check_timeout(osi_clock_ticks_t * t, int hz)
{
	return ((jiffies - *(t)) >= ((hz) * HZ));
}

#define osi_ct_to_secs(x) CT_TO_SECS(x)

/*  Schedule abstraction - Macro that makes a process temporarily
    give up control of the processor and lets other processes have
    a change to run.
    Sleep abstraction - Sleep for x number of sections.  */

#define osi_schedule() \
do { \
	current->policy |= SCHED_YIELD; \
	schedule(); \
} while(0)
#define osi_sleep(x) \
do { \
	current->state = TASK_INTERRUPTIBLE; \
	schedule_timeout((x) * HZ); \
} while (0)
#define osi_sleep_unint(x) \
do { \
	current->state = TASK_UNINTERRUPTIBLE; \
	schedule_timeout((x) * HZ); \
} while (0)


/* signals abstraction */

#define osi_pending_signals() signal_pending(current)

/*  Other Stuff  */

typedef wait_queue_head_t osi_wait_chan_t;

#define osi_wait_vars(x, y)     DECLARE_WAITQUEUE(x, current)
#define osi_init_wait_vars(x)   init_waitqueue_entry(&x, current);
#define osi_init_wait_chan(x)   init_waitqueue_head(&(x));
#define osi_asleep(x, y)        current->state = TASK_UNINTERRUPTIBLE; \
                                add_wait_queue(&(y), &(x))

#define osi_await(x)            schedule()

#define osi_unsleep(x, y)       remove_wait_queue(&(y), &(x)); \
	current->state = TASK_RUNNING
#define osi_wakeup(x) \
do { \
	if (waitqueue_active(&(x))) \
		wake_up(&(x)); \
} while(0)

#endif /* _OSI_LINUX24_H */
