/*
 *
 *    Copyright 1999 Regents of the University of Minnesota
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 *    This file was maintained by:
 *      Michael C Tilstra <conrad@sistina.com>
 *
 */

#include "config.h"

#ifdef TIMESTATS

/*
 * ok, build a struct that will keep track of min, max, avg, stddivation.
 * a function to start, function to stop, and one to print.
 */

#include <linux/module.h>

#include "osi.h"
#include "global.h"

#include "timed.h"
#ifndef MIN
#define MIN(a,b) ((a<b)?a:b)
#endif
#ifndef MAX
#define MAX(a,b) ((a>b)?a:b)
#endif

static __inline u64
time_get(void)
{
        struct timeval tv;

        do_gettimeofday(&tv);
        return ((u64)tv.tv_sec*1000000 + tv.tv_usec);
}


void
__time_start(timestats_t * t)
{
	t->time = time_get();
}

void
__time_stop(timestats_t * t)
{
	u64 dur;
	dur = (time_get()) - t->time;

	t->count++;
	t->max = MAX(dur, t->max);
	t->min = MIN(dur, t->min);
	t->std += (dur * dur);
	t->total += dur;
}

/******************************************************************************
 *
 * So this does max, min, mean, squared std. (why waste cpu cycles taking
 * the sqrt? if you really want that, do it afterwards quick).
 * I'm not bothering with doing sqrt or any of these below. Damn near all
 * of the results I pull up the sstd is like ~0.1 and since there's no
 * floats in the kernel, why waste the effort. (besides, you can do the
 * detail math somewhere else later, I give you all of the numbers)
 * So then for ppl that may be interested:
 * 60% confidence
 *   mean  (0.842 * sqrt(sstd)) / sqrt(count)
 * 90% confidence
 *   mean  (1.645 * sqrt(sstd)) / sqrt(count)
 * 99% confidence
 *   mean  (2.576 * sqrt(sstd)) / sqrt(count)
 */
void
__time_print(timestats_t * t, char *s)
{
	u64 devi = 0;
	u64 part1;		/* These parts may have to be 64bit to avoid overflow. */
	u64 part2;
	u64 part3;
	u64 mean = 0;

	/* not sure this is working right....
	 * It isn't working.
	 * Nevermind it works fine. I don't have floating point, so 0.## is lost*/
	part1 = (u64) t->count * (u64) t->std;
	part2 = (u64) t->total * (u64) t->total;
	part3 = (u64) t->count * ((u64) t->count - (u64) 1);
	if (part3 != 0)
		devi = (part1 - part2) / part3;
	if (t->count != 0)
		mean = t->total / t->count;

	printk("TS: %s: Min: %" FMT_64 "u  Max: %" FMT_64 "u  Mean: %"
		   FMT_64 "u\n", s, t->min, t->max, mean);
	printk("TS: %s: Called %lu times, with total time %" FMT_64 "u\n",
		   s, t->count, t->total);
	printk("TS: %s: Squared Std Deviation: %" FMT_64 "u\n", s, devi);
	printk("TS: %s:  (%" FMT_64 "u - %" FMT_64 "u) / %" FMT_64 "u\n", s,
		   part1, part2, part3);
	printk("TS: %s: 99%%conf = (2.576 * sqrt(%" FMT_64
		   "u))/sqrt(%lu)\n", s, devi, t->count);

}

#endif /* TIMESTATS */
