/*
 *
 *    Copyright 1999 Regents of the University of Minnesota
 *    Copyright 2000-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 *    This file was maintained by:
 *      Michael C Tilstra <conrad@sistina.com>
 *
 */
#ifndef _TIMED_H
#define _TIMED_H

/*
 * Resolution is about microseconds.
 * Assuming do_gettimeofday is returning accurate values.
 */

#ifdef TIMESTATS
typedef struct {
	u64 time;		/* start time */
	u64 min;
	u64 max;
	u64 std;
	u64 total;		/* total of all calls. */
	unsigned long count;	/* Number of times time_stop was called. */
} timestats_t;

/*
 * Need to find a way on initing this that is less platform dependant. If
 * we move to a non-x86, the size of a long and longlong will change...
 */
#define TIMESTAT_INIT (timestats_t){0ULL,~0ULL,0ULL,0ULL,0ULL,0UL}
#define Timestats_Init(t) static timestats_t (t)=TIMESTAT_INIT
#define Timestats_reset(t) t=TIMESTAT_INIT

void __time_start(timestats_t * t);
void __time_stop(timestats_t * t);
void __time_print(timestats_t * t, char *s);
#define time_start(t) __time_start(&(t))
#define time_stop(t)  __time_stop(&(t))
#define time_print(t) __time_print(&(t),#t)

#else /* TIMESTATS */

#define Timestats_Init(t)
#define time_start(t)
#define time_stop(t)
#define time_print(t)
#define Timestats_reset(t)

#endif /* TIMESTATS */
#endif /* _TIMED_H */
