/*
 *
 *    Copyright 1999-2001 Sistina Software, Inc.
 *
 *    This is free software released under the GNU General Public License.
 *    There is no warranty for this software.  See the file COPYING for
 *    details.
 *
 *    See the file AUTHORS for a list of contributors.
 *
 *    This file was maintained by:
 *      David C. Teigland <teigland@sistina.com>
 *
 */

#include "config.h"
#include <linux/module.h>
#include <linux/string.h>
#include <linux/kernel.h>
#include <asm/byteorder.h>

#include "global.h"

/**
 * atoi
 *
 * @c:
 *
 */

int
atoi(char *c)
{
	int x = 0;

	while ('0' <= *c && *c <= '9') {
		x = x * 10 + (*c - '0');
		c++;
	}

	return (x);
}

/**
 * inet_aton
 *
 * @ascii:
 * @ip:
 *
 */

int
inet_aton(char *ascii, uint32 * ip)
{
	uint32 value;
	int x;

	*ip = 0;

	for (x = 0; x < 4; x++) {
		value = atoi(ascii);
		if (value > 255)
			return (-1);

		*ip = (*ip << 8) | value;

		if (x != 3) {
			ascii = strstr(ascii, ".");
			if (!ascii)
				return (-1);
			ascii++;
		}
	}

	return (0);
}

/**
 * inet_ntoa
 *
 * @ascii:
 * @ip:
 *
 */

void
inet_ntoa(uint32 ip, char *buf)
{
	int i;
	char *p;

	p = buf;

	for (i = 3; i >= 0; i--) {
		p += sprintf(p, "%d", (ip >> (8 * i)) & 0xFF);
		if (i > 0)
			*(p++) = '.';
	}

}

/**
 * dump_buffer - debug stuff
 *
 * @buf: < Buffer to print
 * @len: < How much there is
 *
 * Simple routine to print out the contents of a buffer to the screen.
 * Uses the farily common hex format.
 */

void
dump_buffer(void *buf, int len)
{
	int i;
	u8 *c = buf;

	for (i = 0; i < len; i++) {
		if (i % 8 == 0)
			printk("\n%4u: ", i);
		printk("%2x ", c[i]);
	}
	printk("\n");
}
