#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include "ring.h"

void poldhu_init_ring(struct poldhu_ring_buffer *ring)
{
	memset(ring, 0, sizeof(struct poldhu_ring_buffer));
	ring->lock = SPIN_LOCK_UNLOCKED;
}

int poldhu_add_skb(struct poldhu_ring_buffer * ring, struct sk_buff *skb)
{
	unsigned long flags;
	int retval = 0;

	spin_lock_irqsave(&ring->lock, flags);

	if (skb) {
		/* If the ring is full, don't add any more skb's */
		if (ring->full)
			retval = -ENOSPC;
		else
		{
			ring->ring[ring->tail] = skb;
			ring->tail = (ring->tail + 1) % POLDHU_RING_SIZE;
			if (ring->tail == ring->head)
				ring->full = 1;
		}
	} else 
		retval = -ENOENT;

	spin_unlock_irqrestore(&ring->lock, flags);

	return retval;
}
int poldhu_ring_empty(struct poldhu_ring_buffer *ring)
{
	return (ring->ring[ring->head] == NULL);
}

struct sk_buff* poldhu_get_skb(struct poldhu_ring_buffer *ring)
{
	return ring->ring[ring->head];
}

int poldhu_remove_skb(struct poldhu_ring_buffer *ring)
{
	unsigned long flags;
	int retval = 0;

	spin_lock_irqsave(&ring->lock, flags);

	if (poldhu_ring_empty(ring))
		retval = -ENOENT;
	else
	{
		dev_kfree_skb_any(ring->ring[ring->head]);
		ring->ring[ring->head] = NULL;
		ring->head = (ring->head + 1) % POLDHU_RING_SIZE;
		ring->full = 0;
	}

	spin_unlock_irqrestore(&ring->lock, flags);
	
	return retval;
}

