/*                                                        -*- linux-c -*-
 *   _  _ ____ __ _ ___ ____ ____ __ _ _ _ _ | 
 * .  \/  |--| | \|  |  |--< [__] | \| | _X_ | s e c u r e  s y s t e m s
 *
 * .vt|ar5k - PCI/CardBus 802.11a WaveLAN driver for Atheros AR5k chipsets
 *
 * Copyright (c) 2002, .vantronix | secure systems
 *                     and Reyk Floeter <reyk@vantronix.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/wireless.h>
#include <linux/init.h>
#include <linux/proc_fs.h>
#include <linux/pci.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/list.h>
#include <asm/io.h>
#include <asm/uaccess.h>

/* internal headers */
#include <vt_wlan.h>
#include <vt_types.h>
#include <vt_ar5kregs.h>

#ifndef VANTRONIX_AR5K_HEADER
#define VANTRONIX_AR5K_HEADER

/****** DEFINES ******/

#ifndef WAVEHAN
#define WAVEHAN 23
#endif

#ifndef VANTRONIX
#define VANTRONIX WAVEHAN+5
#endif

#ifndef IW_MODE_MONITOR
#define IW_MODE_MONITOR 6
#endif

#define PCIVENDOR_ATHEROS 0x168c
#define PCIDEVICE_ATHEROS 0x0107
#define CB_DEVICE_ATHEROS 0x0007
#define PCI_SIZE 0x1000 

#define SUPPORTED_CHANNELS 5

#define NUMDESCRIPTORS 4

#define	MAX_KEY_LEN_BITS 128
#define	MAX_KEY_LEN_BYTES (MAX_KEY_LEN_BITS / 8)
#define MAX_SHARED_KEYS	4
#define MAX_REG_COUNT 256
#define MAX_RX_DESC_BUF 128
#define MAX_RX_DATA_BUF 256
#define RX_BUF_LEN 1600
#define AR5K_DESC_ALIGN 4 
#define AR5K_REG_SIZE 32

#define DEFAULT_SWRETRY_QDEPTH 100
#define DEFAULT_SWRETRY_MAXRETRIES 1

/** INTERRUPT MASKS **/
#define INIT_INTERRUPT_MASK \
			(AR5K_IMR_DPERR | AR5K_IMR_SSERR | AR5K_IMR_MCABT | \
			 AR5K_IMR_TXERR | AR5K_IMR_TXOK | AR5K_IMR_RXORN | \
			 AR5K_IMR_RXEOL | AR5K_IMR_RXERR | AR5K_IMR_RXOK | AR5K_IMR_SWI)
#define FATAL_INTERRUPT_MASK (AR5K_IMR_DPERR|AR5K_IMR_SSERR|AR5K_IMR_MCABT|AR5K_IMR_RXORN)
#define RECEIVE_INTERRUPT_MASK (AR5K_IMR_RXERR | AR5K_IMR_RXOK)
#define TRANSMIT_INTERRUPT_MASK (AR5K_IMR_TXDESC)

/** RX FILTER **/
#define INIT_RX_FILTER (AR5K_RX_UCAST | AR5K_RX_MCAST | AR5K_RX_BCAST)

/** communication between host<->card **/

#define VT_WLAN_OUT8(v,a) writeb((v), dev->mem_start + (a))
#define VT_WLAN_IN8(a) (u8) readb(dev->mem_start + (a))
#define VT_WLAN_OUT16(v,a) writew((v), dev->mem_start + (a))
#define VT_WLAN_IN16(a) (u16) readw(dev->mem_start + (a))
/* #define VT_WLAN_DEBUG_OUT32(v,a) \
 printk("%s (%s): 0x%lx-0x%lx, addr: 0x%lx (0x%x), val: 0x%x\n", \
 dev_info, dev->name, dev->mem_start, dev->mem_end, dev->mem_start + (a), (a), (v)) */
#define VT_WLAN_DEBUG_OUT32(v,a) \
 printk("%s: write offset: 0x%x, val: 0x%x\n", dev_info, (a), (v))
#if VT_DEBUG > 2
#define VT_WLAN_OUT32(v,a) VT_WLAN_DEBUG_OUT32((v),(a)); writel((v), dev->mem_start + (a))
#else
#define VT_WLAN_OUT32(v,a) writel((v), dev->mem_start + (a))
#endif
#define VT_WLAN_IN32(a) (u32) readl(dev->mem_start + (a))

/****** structures/typedefs ******/

/** generic **/

extern int turbo_channel;
extern int channel;
extern char reg_domain[AR5K_REG_SIZE];

typedef struct pci_link {
	void *priv;
	struct pci_link *next;
} pci_link_t;

/** station structures **/

/* NIC registers */
typedef struct wlan_register_val {
	u32 offset;
	u32 value;
} wlan_register_val_t;

typedef struct wlan_init_registers {
	wlan_register_val_t values[MAX_REG_COUNT];
	u32 count;
} wlan_init_registers_t;

/* keys */
typedef struct wlan_keys {
	u32 keyIV;						/* IV generation control */
	u16 keyFlags;
	u16 keyLength;					/* Length of key in bits */
	u8 keyVal[MAX_KEY_LEN_BYTES];	/* Array for key */
} wlan_keys_t;

/* configuration struct for station */
typedef struct sta_config {
	wlan_mac_addr_t mac_perm_addr; /* Permanent MAC Address of device */
	wlan_mac_addr_t mac_addr; /* MAC Address of device */
	wlan_bssid_t bssid;
	wlan_ssid_t ssid; /* desired ssid */
	wlan_bsstype_t bsstype;
	int turbo_enable; 
	u16 channel;
	u16 hw_channel; /* channel currently set in hardware, may be different */
	u16 adhoc_channel; /* adhoc channel for starting IBSS */
	u32 limit_tx_retries;
	int swretry_enabled; 
	int qos_enabled; 
	u32 aifs; 
	u32 cwmin;
	u16 tp_scale;
	u16 reg_domain; 
} sta_config_t;


/** rx/tx **/

typedef struct ar5k_rx_descriptor {
	u32 reserved1;
	u32 buf_len:12, 
	    reserved2:1, 
            inter_req:1, 
            reserved3:18;		    
} ar5k_rx_t;

typedef struct ar5k_tx_descriptor {
	u32 frame_len:12, 
            hdr_len:6, 
            xmit_rate:4, 
            rts_cts_enable:1, 
            reserved1:1, 
            clear_dest_mask:1, 
            ant_mode_xmit:1, 
            frm_type:3, 
            inter_req:1, 
            encrypt_key_valid:1, 
            reserved2:1;
	u32 buf_len:12,
	    more:1,
  	    encrypt_key_idx:6,
	    rts_duration:13;
} ar5k_tx_t;

typedef struct ar5k_rx_status {
	u32 data_len:12,
            more:1,
            reserved1:1,
            rcv_antenna:1,
            rcv_rate:4,
            rcv_sig_strength:8,
            reserved2:5;
	u32 done:1,
            frame_rcv_ok:1,
            crc_error:1,
            fifo_overrun:1,
            decrypt_crc_error:1,
            phy_error:3,
            key_idx_valid:1,
            key_idx:6,
            rcv_timestamp:13,
	    key_cache_miss:1,
            reserved3:3;
} ar5k_rx_status_t;

typedef struct ar5k_tx_status {
	u32 frm_xmit_ok:1,
  	    excessive_retries:1,
            fifo_underrun:1,
	    filtered:1,
	    sort_retry_count:4,
	    long_retry_count:4,
  	    reserved1:4,
	    send_timestamp:16;
	u32 done:1,
 	    seq_num:12,
	    ack_sig_strength:8,
            reserved2:11;
} ar5k_tx_status_t;

typedef struct ar5k_buffer {
	u8 buffer[RX_BUF_LEN];
} ar5k_buf_t;

/* hardware descriptor structure */
typedef struct ar5k_descriptor {
	/* word 0,1 */
	u32 link_ptr; /* address of next descriptor */
	u32 buffer_ptr;	/* address of this buffer */

	/* word 2,3 */
	union {
		ar5k_rx_t rx;
		ar5k_tx_t tx;
	} u;

	/* word 4,5 */
	union {
		ar5k_rx_status_t rx_compl_stat;
		ar5k_tx_status_t tx_compl_stat;
	} s;

	/* software */
	struct ar5k_descriptor *next;
	ar5k_buf_t *data;
} ar5k_desc_t;

typedef struct ark5_desc_queue {
	struct list_head head;
	rwlock_t lock;
} ar5k_desc_queue_t;

typedef struct vt_local {
	int irq;
	unsigned long io_port;
	u32 int_mask; /* interrupt mask */
	unsigned long int_count;

	u16 mac_rev, phy_rev, a2_rev; /* chip/phy/a2 revision */
 
	int nic_active, nic_ready, led_mode;
	int opmode, monitor, prism_header;
	u32 rx_filter;
	u16 dev_type;

	/* rx ring */
#ifdef USE_SOFTIRQ
	VT_TASKLET rx_tasklet;
#endif

	ar5k_desc_t *rx_desc, *rx_first;
	ar5k_buf_t *rx_buf;
	dma_addr_t rx_desc_dma, rx_buf_dma;
	int rx_desc_size, rx_buf_size, rx_count, rx_drop;

	/* tx ring */
	ar5k_desc_t *tx_desc, *tx_first;
	ar5k_buf_t *tx_buf;
	dma_addr_t tx_desc_dma, tx_buf_dma;
	int tx_desc_size, tx_buf_size, tx_active;

	wlan_init_registers_t init_regs;

	u32 gpio0_bit; /* gpio register value */

	spinlock_t lock;

	pci_link_t *link;
	sta_config_t *conf;
	struct pci_dev *pdev;
	struct net_device *dev;
	struct net_device_stats stats;
	struct proc_dir_entry *proc;
#ifdef WIRELESS_EXT
	struct iw_statistics *wext_stats;
#endif
} vt_local_t;

/****** extern variables ******/

extern char *dev_info;
extern char vt_ar5k_mac_addr[6];
extern char vt_ar5k_bssid[6];
extern const wlan_register_val_t vt_ar5k_register_init[];

/****** functions ******/

/** main **/
int vt_ar5k_nic_init(vt_local_t *local);
int vt_ar5k_nic_exit(vt_local_t *local);
int vt_ar5k_nic_resume(vt_local_t *local);
int vt_ar5k_nic_suspend(vt_local_t *local);
int vt_ar5k_nic_reset(vt_local_t *local, u32 val);
int vt_ar5k_resume(struct pci_dev *pdev);
int vt_ar5k_suspend(struct pci_dev *pdev, u32 state);

int vt_ar5k_probe(struct pci_dev *pdev, const struct pci_device_id *id);
void vt_ar5k_remove(struct pci_dev *pdev);
int vt_ar5k_config(vt_local_t *local);
int vt_ar5k_config_registers(vt_local_t *local);
int vt_ar5k_config_register(vt_local_t *local);
int vt_ar5k_set_channel(vt_local_t *local, u32 channel_ieee);
int vt_ar5k_setup_rx_queue(vt_local_t *local);
int vt_ar5k_setup_tx_queue(vt_local_t *local);
void vt_ar5k_free_tx_queue(vt_local_t *local);
void vt_ar5k_free_rx_queue(vt_local_t *local);

/* net_device */
irqreturn_t vt_ar5k_interrupt(const int irq, void *dev_instance, struct pt_regs *regs);
void vt_ar5k_rx(vt_local_t *local);
int vt_ar5k_monitor_rx(vt_local_t *local, ar5k_desc_t *desc, int len);
int vt_ar5k_80211_rx(vt_local_t *local, ar5k_desc_t *desc, int len);
int vt_ar5k_start_xmit(struct sk_buff *skb, struct net_device *dev);
int vt_ar5k_open(struct net_device *dev);
int vt_ar5k_stop(struct net_device *dev);
int vt_ar5k_nic_open(struct net_device *dev, u32 rx_filter);
void vt_ar5k_nic_stop(struct net_device *dev);

/* misc */
struct net_device_stats *vt_ar5k_get_stats(struct net_device *dev);
u32 vt_ar5k_bitswap(u32 val, int bit_count);
int vt_ar5k_eeprom_read(vt_local_t *local, u32 offset, u16 *data);
int vt_ar5k_eeprom_read_mac(vt_local_t *local, wlan_mac_addr_t *omac);

/** gpio **/
int vt_ar5k_gpio_out(vt_local_t *local, u32 gpio);
int vt_ar5k_gpio_in(vt_local_t *local, u32 gpio);
int vt_ar5k_gpio_set(vt_local_t *local, u32 gpio, u32 val);
u32 vt_ar5k_gpio_get(vt_local_t *local, u32 gpio);
void vt_ar5k_gpio_int(vt_local_t *local, u32 level); /* set gpio0 int */

/** proc **/
void vt_ar5k_init_proc(vt_local_t *local);
void vt_ar5k_remove_proc(vt_local_t *local);
void vt_ar5k_cleanup_proc(void);
struct iw_statistics *vt_ar5k_wireless_stats(struct net_device *dev);

/** ioctl **/
int vt_ar5k_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd);

/** regulation **/
int vt_ar5k_reg_init(void);

#endif /* VANTRONIX_AR5K_HEADER */
