/*                                                        -*- linux-c -*-
 *   _  _ ____ __ _ ___ ____ ____ __ _ _ _ _ | 
 * .  \/  |--| | \|  |  |--< [__] | \| | _X_ | s e c u r e  s y s t e m s
 *
 * .vt|ar5k - PCI/CardBus 802.11a WaveLAN driver for Atheros AR5k chipsets
 *
 * Copyright (c) 2002, .vantronix | secure systems
 *                     and Reyk Floeter <reyk@vantronix.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef VANTRONIX_ATHEROS_REGS_HEADER
#define VANTRONIX_ATHEROS_REGS_HEADER

/* DMA Control and Interrupt Registers */
#define AR5K_TXDP0 0x0000 
#define AR5K_TXDP1 0x0004 

#define AR5K_CR 0x0008 
#define AR5K_CR_TXE0 0x00000001 
#define AR5K_CR_TXE1 0x00000002 
#define AR5K_CR_RXE 0x00000004 
#define AR5K_CR_TXD0 0x00000008 
#define AR5K_CR_TXD1 0x00000010 
#define AR5K_CR_RXD 0x00000020 
#define AR5K_CR_SWI 0x00000040 

#define AR5K_RXDP 0x000C 

#define AR5K_CFG 0x0014 
#define AR5K_CFG_SWTD 0x00000001 
#define AR5K_CFG_SWTB 0x00000002 
#define AR5K_CFG_SWRD 0x00000004 
#define AR5K_CFG_SWRB 0x00000008 
#define AR5K_CFG_SWRG 0x00000010 
#define AR5K_CFG_PHOK 0x00000100 
#define AR5K_CFG_EEBS 0x00000200 
#define AR5K_CFG_TXCNT_M 0x00007800 
#define AR5K_CFG_TXCNT_S 11 
#define AR5K_CFG_TXFSTAT 0x00008000 
#define AR5K_CFG_TXFSTRT 0x00010000 

#define AR5K_ISR 0x001c 
#define AR5K_ISR_RXOK 0x00000001 
#define AR5K_ISR_RXDESC 0x00000002 
#define AR5K_ISR_RXERR 0x00000004 
#define AR5K_ISR_RXNOPKT 0x00000008 
#define AR5K_ISR_RXEOL 0x00000010 
#define AR5K_ISR_RXORN 0x00000020 
#define AR5K_ISR_TXOK 0x00000040 
#define AR5K_ISR_TXDESC 0x00000080 
#define AR5K_ISR_TXERR 0x00000100 
#define AR5K_ISR_TXNOPKT 0x00000200 
#define AR5K_ISR_TXEOL 0x00000400 
#define AR5K_ISR_TXURN 0x00000800 
#define AR5K_ISR_MIB 0x00001000 
#define AR5K_ISR_SWI 0x00002000 
#define AR5K_ISR_RXPHY 0x00004000 
#define AR5K_ISR_RXKCM 0x00008000 
#define AR5K_ISR_SWBA 0x00010000 
#define AR5K_ISR_BRSSI 0x00020000 
#define AR5K_ISR_BMISS 0x00040000 
#define AR5K_ISR_F2ERR 0x00080000 
#define AR5K_ISR_MCABT 0x00100000 
#define AR5K_ISR_SSERR 0x00200000 
#define AR5K_ISR_DPERR 0x00400000 
#define AR5K_ISR_GPIO 0x01000000 
#define AR5K_ISR_RESV0 0xFE000000 

#define AR5K_IMR 0x0020 
#define AR5K_IMR_RXOK 0x00000001 
#define AR5K_IMR_RXDESC 0x00000002 
#define AR5K_IMR_RXERR 0x00000004 
#define AR5K_IMR_RXNOPKT 0x00000008 
#define AR5K_IMR_RXEOL 0x00000010 
#define AR5K_IMR_RXORN 0x00000020 
#define AR5K_IMR_TXOK 0x00000040 
#define AR5K_IMR_TXDESC 0x00000080 
#define AR5K_IMR_TXERR 0x00000100 
#define AR5K_IMR_TXNOPKT 0x00000200 
#define AR5K_IMR_TXEOL 0x00000400 
#define AR5K_IMR_TXURN 0x00000800 
#define AR5K_IMR_MIB 0x00001000 
#define AR5K_IMR_SWI 0x00002000 
#define AR5K_IMR_RXPHY 0x00004000 
#define AR5K_IMR_RXKCM 0x00008000 
#define AR5K_IMR_SWBA 0x00010000 
#define AR5K_IMR_BRSSI 0x00020000 
#define AR5K_IMR_BMISS 0x00040000 
#define AR5K_IMR_F2ERR 0x00080000 
#define AR5K_IMR_MCABT 0x00100000 
#define AR5K_IMR_SSERR 0x00200000 
#define AR5K_IMR_DPERR 0x00400000 
#define AR5K_IMR_GPIO 0x01000000 
#define AR5K_IMR_RESV0 0xFE000000 
#define AR5K_IMR_X 0x0000FFFF 

#define AR5K_IER 0x0024 
#define AR5K_IER_ENABLE 0x00000001 
#define AR5K_IER_DISABLE 0x00000000 

#define AR5K_BCR 0x0028 
#define AR5K_BCR_APMODE 0x00000000 
#define AR5K_BCR_STAMODE 0x00000001 
#define AR5K_BCR_BDMAE 0x00000002 
#define AR5K_BCR_TQ1PV 0x00000004 
#define AR5K_BCR_TQ1V 0x00000008 
#define AR5K_BCR_BCGET 0x00000010 

#define AR5K_BSR 0x002c 
#define AR5K_BSR_BDLYSW 0x00000001 
#define AR5K_BSR_BDLYDMA 0x00000002 
#define AR5K_BSR_TXQ1F 0x00000004 
#define AR5K_BSR_ATIMDLY 0x00000008 
#define AR5K_BSR_SNPBCMD 0x00000010 
#define AR5K_BSR_SNPBDMAE 0x00000020 
#define AR5K_BSR_SNPTQ1FV 0x00000040 
#define AR5K_BSR_SNPTQ1V 0x00000080 
#define AR5K_BSR_SNP_VALID 0x00000100 
#define AR5K_BSR_SWBACNT_M 0x00FF0000 
#define AR5K_BSR_SWBACNT_S 16 

#define AR5K_TXCFG 0x0030 
#define AR5K_TXCFG_TXFSTP 0x00000008 
#define AR5K_TXCFG_TXFULL 0x00000070 
#define AR5K_TXCFG_CONT_EN 0x00000080 
#define AR5K_RXCFG 0x0034 
#define AR5K_RXCFG_ZLFDMA 0x00000010 
#define AR5K_DMASIZE_4B 0x00000000 
#define AR5K_DMASIZE_8B 0x00000001 
#define AR5K_DMASIZE_16B 0x00000002 
#define AR5K_DMASIZE_32B 0x00000003 
#define AR5K_DMASIZE_64B 0x00000004 
#define AR5K_DMASIZE_128B 0x00000005 
#define AR5K_DMASIZE_256B 0x00000006 
#define AR5K_DMASIZE_512B 0x00000007 

#define AR5K_MIBC 0x0040 
#define AR5K_MIBC_COW 0x00000001 
#define AR5K_MIBC_FMC 0x00000002 
#define AR5K_MIBC_CMC 0x00000004 
#define AR5K_MIBC_MCS 0x00000008 

#define AR5K_TOPS 0x0044 
#define AR5K_TOPS_MASK 0x0000FFFF 

#define AR5K_RXNPTO 0x0048 
#define AR5K_RXNPTO_MASK 0x000003FF 

#define AR5K_TXNPTO 0x004C 
#define AR5K_TXNPTO_MASK 0x000003FF 

#define AR5K_RPGTO 0x0050 
#define AR5K_RPGTO_MASK 0x000003FF 

#define AR5K_RPCNT 0x0054 
#define AR5K_RPCNT_MASK 0x0000001F 
 
#define AR5K_F2MISC 0x0058 
#define AR5K_MISC_LED_DELAY_M 0x001C0000 
#define AR5K_MISC_LED_DELAY_S 18 
#define AR5K_MISC_LED_BLINK_M 0x00E00000 
#define AR5K_MISC_LED_BLINK_S 21 

/* DMA & PCI Registers in PCI space (usable during sleep) */
#define AR5K_RC 0x4000 
#define AR5K_RC_PCU 0x00000001 
#define AR5K_RC_DMA 0x00000002 
#define AR5K_RC_F2 0x00000004 
#define AR5K_RC_D2 0x00000008 
#define AR5K_RC_PCI 0x00000010 

#define AR5K_SCR 0x4004 
#define AR5K_SCR_SLDUR_MASK 0x0000ffff 
#define AR5K_SCR_SLE_MASK 0x00030000 
#define AR5K_SCR_SLE_FWAKE 0x00000000 
#define AR5K_SCR_SLE_FSLEEP 0x00010000 
#define AR5K_SCR_SLE_NORMAL 0x00020000 
#define AR5K_SCR_SLE_UNITS 0x00000008 

#define AR5K_INTPEND 0x4008 
#define AR5K_INTPEND_TRUE 0x00000001 

#define AR5K_SFR 0x400C 
#define AR5K_SFR_SLEEP 0x00000001 

#define AR5K_PCICFG 0x4010 
#define AR5K_PCICFG_EEAE 0x00000001 
#define AR5K_PCICFG_CLKRUNEN 0x00000004 
#define AR5K_PCICFG_LED_PEND 0x00000020 
#define AR5K_PCICFG_LED_ACT 0x00000040 
#define AR5K_PCICFG_SL_INTEN 0x00000800 
#define AR5K_PCICFG_BCTL		 0x00001000 
#define AR5K_PCICFG_SPWR_DN 0x00010000 

#define AR5K_NUM_GPIO 6		

#define AR5K_GPIOCR 0x4014 
#define AR5K_GPIOCR_CR_SHIFT 2 
#define AR5K_GPIOCR_0_CR_N 0x00000000 
#define AR5K_GPIOCR_0_CR_0 0x00000001 
#define AR5K_GPIOCR_0_CR_1 0x00000002 
#define AR5K_GPIOCR_0_CR_A 0x00000003 
#define AR5K_GPIOCR_1_CR_N 0x00000000 
#define AR5K_GPIOCR_1_CR_0 0x00000004 
#define AR5K_GPIOCR_1_CR_1 0x00000008 
#define AR5K_GPIOCR_1_CR_A 0x0000000C 
#define AR5K_GPIOCR_2_CR_N 0x00000000 
#define AR5K_GPIOCR_2_CR_0 0x00000010 
#define AR5K_GPIOCR_2_CR_1 0x00000020 
#define AR5K_GPIOCR_2_CR_A 0x00000030 
#define AR5K_GPIOCR_3_CR_N 0x00000000 
#define AR5K_GPIOCR_3_CR_0 0x00000040 
#define AR5K_GPIOCR_3_CR_1 0x00000080 
#define AR5K_GPIOCR_3_CR_A 0x000000C0 
#define AR5K_GPIOCR_4_CR_N 0x00000000 
#define AR5K_GPIOCR_4_CR_0 0x00000100 
#define AR5K_GPIOCR_4_CR_1 0x00000200 
#define AR5K_GPIOCR_4_CR_A 0x00000300 
#define AR5K_GPIOCR_5_CR_N 0x00000000 
#define AR5K_GPIOCR_5_CR_0 0x00000400 
#define AR5K_GPIOCR_5_CR_1 0x00000800 
#define AR5K_GPIOCR_5_CR_A 0x00000C00 
#define AR5K_GPIOCR_INT_SEL0 0x00000000 
#define AR5K_GPIOCR_INT_SEL1 0x00001000 
#define AR5K_GPIOCR_INT_SEL2 0x00002000 
#define AR5K_GPIOCR_INT_SEL3 0x00003000 
#define AR5K_GPIOCR_INT_SEL4 0x00004000 
#define AR5K_GPIOCR_INT_SEL5 0x00005000 
#define AR5K_GPIOCR_INT_EN 0x00008000 
#define AR5K_GPIOCR_INT_SELL 0x00000000 
#define AR5K_GPIOCR_INT_SELH 0x00010000 

#define AR5K_GPIODO 0x4018 
#define AR5K_GPIODI 0x401C 
#define AR5K_GPIOD_MASK 0x0000002F 

#define AR5K_SREV 0x4020 
#define AR5K_SREV_ID_M 0x000000FF 
#define AR5K_SREV_FPGA 1
#define AR5K_SREV_D2PLUS 2
#define AR5K_SREV_D2PLUS_MS 3 
#define AR5K_SREV_CRETE 4
#define AR5K_SREV_CRETE_MS 5 
#define AR5K_SREV_CRETE_MS23 7 
#define AR5K_SREV_CRETE_23 8 

/* EEPROM Registers in the MAC */
#define AR5K_EEPROM_BASE 0x6000 
#define AR5K_EEPROM_RDATA	0x6800 
#define AR5K_EEPROM_STATUS 0x6c00 
#define AR5K_EEPROM_STAT_RDERR 0x0001 
#define AR5K_EEPROM_STAT_RDDONE 0x0002 
#define AR5K_EEPROM_STAT_WRERR 0x0004 
#define AR5K_EEPROM_STAT_WRDONE 0x0008 

/* MAC PCU Registers */
#define AR5K_STA_ID0 0x8000 
#define AR5K_STA_ID1 0x8004 
#define AR5K_STA_ID1_SADH_MASK 0x0000FFFF 
#define AR5K_STA_ID1_STA_AP 0x00010000 
#define AR5K_STA_ID1_AD_HOC 0x00020000 
#define AR5K_STA_ID1_PWR_SAV 0x00040000 
#define AR5K_STA_ID1_KSRCHDIS 0x00080000 
#define AR5K_STA_ID1_PSPOLLDIS 0x00100000 
#define AR5K_STA_ID1_PCF 0x00200000 
#define AR5K_STA_ID1_DESC_ANT 0x00400000 
#define AR5K_STA_ID1_DEFAULT_ANT 0x00800000 
#define AR5K_STA_ID1_ACKCTS_6MB 0x01000000 

#define AR5K_BSS_ID0 0x8008 
#define AR5K_BSS_ID1 0x800C 
#define AR5K_BSS_ID1_U16_M 0x0000FFFF 
#define AR5K_BSS_ID1_AID_M 0xFFFF0000 
#define AR5K_BSS_ID1_AID_S 16 

#define AR5K_SLOT_TIME 0x8010 
#define AR5K_SLOT_TIME_MASK 0x000007FF 

#define AR5K_TIME_OUT 0x8014 
#define AR5K_TIME_OUT_ACK_M 0x00001FFF 
#define AR5K_TIME_OUT_CTS_M 0x1FFF0000 
#define AR5K_TIME_OUT_CTS_S 16 

#define AR5K_RSSI_THR 0x8018 
#define AR5K_RSSI_THR_MASK 0x000000FF 
#define AR5K_RSSI_THR_BM_THR_M 0x00000700 
#define AR5K_RSSI_THR_BM_THR_S 8 

#define AR5K_RETRY_LMT 0x801c 
#define AR5K_RETRY_LMT_SH_M 0x0000000F 
#define AR5K_RETRY_LMT_LG_M 0x000000F0 
#define AR5K_RETRY_LMT_LG_S 4 
#define AR5K_RETRY_LMT_SSH_M 0x00003F00 
#define AR5K_RETRY_LMT_SSH_S 8 
#define AR5K_RETRY_LMT_SLG_M 0x000FC000 
#define AR5K_RETRY_LMT_SLG_S 14 
#define AR5K_RETRY_LMT_CW_M 0x3FF00000 
#define AR5K_RETRY_LMT_CW_S 20 

#define AR5K_USEC 0x8020 
#define AR5K_USEC_M 0x0000007F 
#define AR5K_USEC_32_M 0x00003F80 
#define AR5K_USEC_32_S 7 
#define AR5K_USEC_TX_LAT_M 0x000FC000 
#define AR5K_USEC_TX_LAT_S 14 
#define AR5K_USEC_RX_LAT_M 0x03F00000 
#define AR5K_USEC_RX_LAT_S 20 

#define AR5K_BEACON 0x8024 
#define AR5K_BEACON_PERIOD_MASK 0x0000FFFF 
#define AR5K_BEACON_TIM_MASK 0x007F0000 
#define AR5K_BEACON_TIM_S 16 
#define AR5K_BEACON_EN 0x00800000 
#define AR5K_BEACON_RESET_TSF 0x01000000 

#define AR5K_CFP_PERIOD 0x8028 
#define AR5K_TIMER0 0x802c 
#define AR5K_TIMER1 0x8030 
#define AR5K_TIMER2 0x8034 
#define AR5K_TIMER3 0x8038 

#define AR5K_IFS0 0x8040 
#define AR5K_IFS0_SIFS_M 0x000007FF 
#define AR5K_IFS0_DIFS_M 0x007FF800 
#define AR5K_IFS0_DIFS_S 11 

#define AR5K_IFS1 0x8044 
#define AR5K_IFS1_PIFS_M 0x00000FFF 
#define AR5K_IFS1_EIFS_M 0x03FFF000 
#define AR5K_IFS1_EIFS_S 12 
#define AR5K_IFS1_CS_EN 0x04000000 

#define AR5K_CFP_DUR 0x8048 

#define AR5K_RX_FILTER 0x804C 
#define AR5K_RX_FILTER_ALL 0x00000000 
#define AR5K_RX_UCAST 0x00000001 
#define AR5K_RX_MCAST 0x00000002 
#define AR5K_RX_BCAST 0x00000004 
#define AR5K_RX_CONTROL 0x00000008 
#define AR5K_RX_BEACON 0x00000010 
#define AR5K_RX_PROM 0x00000020 

#define AR5K_MCAST_FIL0 0x8050 
#define AR5K_MCAST_FIL1 0x8054 
#define AR5K_TX_MASK0 0x8058 
#define AR5K_TX_MASK1 0x805c 
#define AR5K_CLR_TMASK 0x8060 
#define AR5K_TRIG_LEV 0x8064 

#define AR5K_DIAG_SW 0x8068 
#define AR5K_DIAG_CACHE_ACK 0x00000001 
#define AR5K_DIAG_ACK_DIS 0x00000002 
#define AR5K_DIAG_CTS_DIS 0x00000004 
#define AR5K_DIAG_ENCRYPT_DIS 0x00000008 
#define AR5K_DIAG_DECRYPT_DIS 0x00000010 
#define AR5K_DIAG_TX_DIS 0x00000020 
#define AR5K_DIAG_RX_DIS 0x00000040 
#define AR5K_DIAG_LOOP_EN 0x00000080 
#define AR5K_DIAG_CORR_FCS 0x00000100 
#define AR5K_DIAG_CHAN_INFO 0x00000200 
#define AR5K_DIAG_EN_SCRAMSD 0x00000400 
#define AR5K_DIAG_SCRAM_SEED_M 0x0003F800 
#define AR5K_DIAG_SCRAM_SEED_S 11 
#define AR5K_DIAG_DIS_SEQ_INC 0x00040000 
#define AR5K_DIAG_FRAME_NV0 0x00080000 

#define AR5K_TSF_L32 0x806c 
#define AR5K_TSF_U32 0x8070 
#define AR5K_LAST_TSTP 0x8080 
#define AR5K_RETRY_CNT 0x8084 
#define AR5K_RETRY_CNT_SSH_M 0x0000003F 
#define AR5K_RETRY_CNT_SLG_M 0x00000FC0 
#define AR5K_RETRY_CNT_SLG_S 6 

#define AR5K_BACKOFF 0x8088 
#define AR5K_BACKOFF_CW_M 0x000003FF 
#define AR5K_BACKOFF_CNT_M 0x03FF0000 
#define AR5K_BACKOFF_CNT_S 16 

#define AR5K_NAV 0x808c 
#define AR5K_RTS_OK 0x8090 
#define AR5K_RTS_FAIL 0x8094 
#define AR5K_ACK_FAIL 0x8098 
#define AR5K_FCS_FAIL 0x809C 
#define AR5K_BEACON_CNT 0x80A0 

/* EEPROM defines for Version 1 */
#define AR5K_EEPROM_REG_DOMAIN_OFFSET 0xBF 
#define AR5K_EEPROM_PROTECT_OFFSET 0x3F 
#define AR5K_EEPROM_ATHEROS_OFFSET 0xC0 
#define AR5K_EEPROM_ATHEROS_ENTRIES 64 
#define AR5K_EEPROM_REG_DOMAINS 4 
#define AR5K_EEPROM_CHANNELS_SUPPORTED 5 
#define AR5K_EEPROM_TP_SETTINGS_OFFSET 0x09 
#define AR5K_EEPROM_TP_SETTINGS_SIZE 11 
#define AR5K_EEPROM_TP_SCALING_ENTRIES 11 

/* EEPROM defines for Version 0 */
#define AR5K_EEPROM_ENTRIES_FOR_TXPOWER 32 
#define AR5K_EEPROM_TXPOWER_OFFSET 0xE0 

/* Protect Bits RP is read protect, WP is write protect */
#define EEPROM_PROTECT_RP_0_31 0x0001
#define EEPROM_PROTECT_WP_0_31 0x0002 
#define EEPROM_PROTECT_RP_32_63 0x0004
#define EEPROM_PROTECT_WP_32_63 0x0008
#define EEPROM_PROTECT_RP_64_127 0x0010
#define EEPROM_PROTECT_WP_64_127 0x0020
#define EEPROM_PROTECT_RP_128_191 0x0040
#define EEPROM_PROTECT_WP_128_191 0x0080
#define EEPROM_PROTECT_RP_192_207 0x0100
#define EEPROM_PROTECT_WP_192_207 0x0200
#define EEPROM_PROTECT_RP_208_223 0x0400
#define EEPROM_PROTECT_WP_208_223 0x0800
#define EEPROM_PROTECT_RP_224_239 0x1000
#define EEPROM_PROTECT_WP_224_239 0x2000
#define EEPROM_PROTECT_RP_240_255 0x4000
#define EEPROM_PROTECT_WP_240_255 0x8000

/* D2 register values */
#define FIRPWR_SH_AMT 18
#define FIRSTEP_SH_AMT 12
#define RELPWR_SH_AMT 6
#define RELSTEP_SH_AMT 0
#define QUICK_DROP_SH_AMT 22
#define COARSE_HIGH_SH_AMT 15
#define COARSE_LOW_SH_AMT 7
#define COARSEPWR_CONST_SH_AMT 0
#define COARSE_HIGH_MASK 0x3f8000
#define COARSE_HIGH_SIZE 7
#define COARSE_LOW_MASK	0x7f80
#define COARSE_LOW_SIZE	8
#define COARSEPWR_CONST_MASK 0x7f
#define COARSEPWR_CONST_SIZE 7
#define FIRPWR_MASK 0x3fc0000
#define FIRPWR_SIZE 8

/* PHY registers */
#define PHY_BASE 0x9800 

#define PHY_FRAME_CONTROL 0x9804 
#define PHY_FC_TURBO_MODE 0x00000001 
#define PHY_FC_TURBO_SHORT 0x00000002 
#define PHY_FC_TIMING_ERR 0x01000000 
#define PHY_FC_PARITY_ERR 0x02000000 
#define PHY_FC_ILLRATE_ERR 0x04000000 
#define PHY_FC_ILLLEN_ERR 0x08000000 
#define PHY_FC_SERVICE_ERR 0x20000000 
#define PHY_FC_TX_UNDER_ERR 0x40000000 

#define PHY_CHIP_ID 0x9818 

#define PHY_ACTIVE 0x981C 
#define PHY_ACTIVE_EN 0x00000001 
#define PHY_ACTIVE_DIS 0x00000000 

#define PHY_AGC_CONTROL 0x9860 
#define PHY_AGC_CONTROL_CAL 0x00000001 
#define PHY_AGC_CONTROL_NF 0x00000002 

/* Device ID defines for AR5210 devices */
#define DEV_E2_PCI 0x0101 
#define DEV_E5_PCI 0x0102
#define DEV_E7_PCI 0x0103
#define DEV_E7_PCI_PA 0x0104
#define DEV_E7_PC_PA 0x0004
#define DEV_E9_PCI_PA 0x0105
#define DEV_E9_PC_PA 0x0005
#define DEV_E9_PC_ANT 0x0006
#define DEV_AR5210_PCI 0x0107
#define DEV_AR5210_PC 0x0007
#define DEV_AR5210_AP 0x0207
#define DEV_AR5001 0x0010
#define DEV_LEGACY 0x1107

#endif /* VANTRONIX_ATHEROS_HEADER */
