/*                                                        -*- linux-c -*-
 *   _  _ ____ __ _ ___ ____ ____ __ _ _ _ _ | 
 * .  \/  |--| | \|  |  |--< [__] | \| | _X_ | s e c u r e  s y s t e m s
 *
 * .vt|ar5k - PCI/CardBus 802.11a WaveLAN driver for Atheros AR5k chipsets
 *
 * Copyright (c) 2002, .vantronix | secure systems
 *                     and Reyk Floeter <reyk@vantronix.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef VANTRONIX_WLAN_HEADER
#define VANTRONIX_WLAN_HEADER

/****** DEFINES ******/

/* 802.11 header */
#define WLAN_FRAME_TYPE_MGT 0
#define WLAN_FRAME_TYPE_CTRL 1
#define WLAN_FRAME_TYPE_DATA 2
#define WLAN_FRAME_TYPE_RESERVED 3

/* List of Element IDs */
#define WLAN_ELE_SSID 0	
#define WLAN_ELE_SUPPORTED_RATES 1 
#define WLAN_ELE_FH_PARAM_SET 2	
#define	WLAN_ELE_DS_PARAM_SET 3	/* DS Parameter set */
#define	WLAN_ELE_CF_PARAM_SET 4	/* CF Parameter set */
#define WLAN_ELE_TIM 5 
#define WLAN_ELE_IBSS_PARAM_SET 6 
#define WLAN_ELE_CHLG_TEXT 16 
/* 17-31 reserved for challenge txt extension */

/* Max sizes of variable size information elements */
#define WLAN_ELE_INFO_SIZE 255 
#define WLAN_ELE_SSID_SIZE 32 
#define WLAN_ELE_RATE_SIZE 8 
#define WLAN_ELE_BITMAP_SIZE 251 
#define WLAN_ELE_CHLG_TEXT_SIZE 253 
#define WLAN_ELE_CF_PARAM_SIZE 6 
#define WLAN_ELE_IBSS_PARAM_SIZE 2 

#define	WLAN_MAC_ADDR_SIZE 6

#define POWER_UP_TIME 200

/****** structures/typedefs ******/

typedef union wlan_mac_addr {
	u8 octets[WLAN_MAC_ADDR_SIZE];
	struct {
		u32 word;
		u16 half;
	} /*__ATTRIB_PACK*/ st;
} wlan_mac_addr_t;

typedef wlan_mac_addr_t wlan_bssid_t;

/* note:
 * bit fields are ordered for little endian.
 * please use the corresponding swap() function for big endian. 
 */

typedef struct wlan_cap_info {
	u16 ess:1,
	    ibss:1, 
	    cf_pollable:1,
	    cf_poll_req:1,
	    privacy:1,
	    reserved:11;
} wlan_cap_info_t;

typedef struct wlan_ssid {
	u8 element_id;
	u8 length;
	u8 ssid[WLAN_ELE_SSID_SIZE];
} /*__ATTRIB_PACK*/ wlan_ssid_t;

typedef struct wlan_rate_set {
	u8 element_id;
	u8 length; 
	u8 rates[WLAN_ELE_RATE_SIZE]; 
} /*__ATTRIB_PACK*/ wlan_rate_set_t;

/** 802.11 header **/
typedef struct wlan_frame_control {
        u16 version:2,
	    type:2,
	    subtype:4,
	    to_ds:1,
	    from_ds:1,
	    more_frag:1,
	    retry:1,
	    pwr_mgt:1,
	    more_data:1,
	    wep:1,
	    order:1;
        u16 duration:15,
	    clear_bit:1;
} wlan_frame_control_t;

typedef struct wlan_sequence {
	u16 frag:4,
	    num:12;
} wlan_seq_t;

typedef struct wlan_frame_header {
	wlan_frame_control_t control;
	u8 address1[WLAN_MAC_ADDR_SIZE];
	u8 address2[WLAN_MAC_ADDR_SIZE];
	u8 address3[WLAN_MAC_ADDR_SIZE];
	wlan_seq_t seq;
} wlan_frame_hdr_t;

/* ARPHRD_IEEE80211_PRISM (from linux-wlan-ng) */
typedef struct linux_wlan_ng_val {
	u32 did;
	u16 status, len;
	u32 data;
} linux_wlan_ng_val_t;

typedef struct linux_wlan_ng_prism_hdr {
	u32 msgcode, msglen;
	char devname[16];
	linux_wlan_ng_val_t hosttime, mactime, channel, rssi, sq, 
		signal,	noise, rate, istx, frmlen;
} linux_wlan_ng_prism_hdr_t;

/************ MLME/PLME ************/

typedef enum {
	INFRASTRUCTURE_BSS = 1, /* MIB spec specifies this */
	INDEPENDENT_BSS,
	ANY_BSS,
	MONITOR
} wlan_bsstype_t;

#endif /* VANTRONIX_WLAN_HEADER */
