/*                                                        -*- linux-c -*-
 *   _  _ ____ __ _ ___ ____ ____ __ _ _ _ _ | 
 * .  \/  |--| | \|  |  |--< [__] | \| | _X_ | s e c u r e  s y s t e m s
 *
 * .vt|ar5k - PCI/CardBus 802.11a WirelessLAN driver for Atheros AR5k chipsets
 *
 * Copyright (c) 2002, .vantronix | secure systems
 *                     and Reyk Floeter <reyk@vantronix.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <vt_ar5kconfig.h>
#include <vt_ar5k.h>

/*
 * GPIO control routines.   Only output is currently supported.
 */
int vt_ar5k_gpio_out(vt_local_t *local, u32 gpio) 
{
	struct net_device *dev = (struct net_device *)local->dev;
	u32 reg;

	if(gpio > AR5K_NUM_GPIO) {
		printk("%s (%s): gpio_out failed (GPIO:%x > AR5K_NUM_GPIO:%x)", 
		       dev_info, dev->name, gpio, AR5K_NUM_GPIO);
		return (1);
	}

	reg = VT_WLAN_IN32(AR5K_GPIOCR);
	reg &= ~(AR5K_GPIOCR_0_CR_A << (gpio * AR5K_GPIOCR_CR_SHIFT));
	reg |= AR5K_GPIOCR_0_CR_1 << (gpio * AR5K_GPIOCR_CR_SHIFT);
	
	VT_WLAN_OUT32(reg, AR5K_GPIOCR);
	
	return (0);
}

int vt_ar5k_gpio_in(vt_local_t *local, u32 gpio)
{
	struct net_device *dev = (struct net_device *)local->dev;
	u32 reg;

	if(gpio > AR5K_NUM_GPIO) {
		printk("%s (%s): gpio_in failed (GPIO:%x > AR5K_NUM_GPIO:%x)", 
		       dev_info, dev->name, gpio, AR5K_NUM_GPIO);
		return (1);
	}

	reg = VT_WLAN_IN32(AR5K_GPIOCR);
	reg &= ~(AR5K_GPIOCR_0_CR_A << (gpio * AR5K_GPIOCR_CR_SHIFT));
	reg |= AR5K_GPIOCR_0_CR_N << (gpio * AR5K_GPIOCR_CR_SHIFT);

	VT_WLAN_OUT32(reg, AR5K_GPIOCR);

	return (0);
}

int vt_ar5k_gpio_set(vt_local_t *local, u32 gpio, u32 val) 
{
	struct net_device *dev = (struct net_device *)local->dev;
	u32 reg;

	if(gpio > AR5K_NUM_GPIO) {
		printk("%s (%s): gpio_set failed (GPIO:%x > AR5K_NUM_GPIO:%x)", 
		       dev_info, dev->name, gpio, AR5K_NUM_GPIO);
		return (1);
	}

	reg = VT_WLAN_IN32(AR5K_GPIODO);
	reg &= ~(1 << gpio);
	reg |= (val&1) << gpio;

	VT_WLAN_OUT32(reg, AR5K_GPIODO);

	return (0);
}

u32 vt_ar5k_gpio_get(vt_local_t *local, u32 gpio) 
{
	struct net_device *dev = (struct net_device *)local->dev;
	u32 reg;
	
	if(gpio > AR5K_NUM_GPIO) {
		printk("%s (%s): gpio_get failed (GPIO:%x > AR5K_NUM_GPIO:%x)", 
		       dev_info, dev->name, gpio, AR5K_NUM_GPIO);
		return (0);
	}
	
	reg = VT_WLAN_IN32(AR5K_GPIODI);
	reg = ((reg & AR5K_GPIOD_MASK) >> gpio) & 0x1;
	
	return (reg);
}

void vt_ar5k_gpio_int(vt_local_t *local, u32 level) 
{
	struct net_device *dev = (struct net_device *)local->dev;
	u32	reg;
	
	reg = VT_WLAN_IN32(AR5K_GPIOCR);

	/* clear the bits that we will modify */
	reg &= ~(AR5K_GPIOCR_INT_SEL0 | AR5K_GPIOCR_INT_EN | AR5K_GPIOCR_0_CR_A | AR5K_GPIOCR_INT_SELH);
	reg |= AR5K_GPIOCR_INT_SEL0 | AR5K_GPIOCR_INT_EN | AR5K_GPIOCR_0_CR_N;
	
	if(level) {
		reg |= AR5K_GPIOCR_INT_SELH;
	}

	/* don't need to change anything for low level interrupt. */
	VT_WLAN_OUT32(reg, AR5K_GPIOCR);

	/* change the interrupt mask */
	VT_WLAN_OUT32(VT_WLAN_IN32(AR5K_IMR) | AR5K_IMR_GPIO, AR5K_IMR);
	local->int_mask = VT_WLAN_IN32(AR5K_IMR);
}

