/*                                                        -*- linux-c -*-
 *   _  _ ____ __ _ ___ ____ ____ __ _ _ _ _ | 
 * .  \/  |--| | \|  |  |--< [__] | \| | _X_ | s e c u r e  s y s t e m s
 *
 * .vt|ar5k - PCI/CardBus 802.11a WirelessLAN driver for Atheros AR5k chipsets
 *
 * Copyright (c) 2002, .vantronix | secure systems
 *                     and Reyk Floeter <reyk@vantronix.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <vt_ar5kconfig.h>
#include <vt_ar5k.h>

static __inline void vt_ar5k_ioctl_monitor(struct net_device *dev, int val)
{
	vt_local_t *local = (vt_local_t *) dev->priv;

	VENTER();

	if(val) {
		VPRINT("enable monitor mode\n");
		local->monitor = 1;
		local->dev_type = dev->type; /* store type */
		
		if(!local->prism_header)
			dev->type = ARPHRD_IEEE80211;
		else 
			dev->type = ARPHRD_IEEE80211_PRISM;
	
		/* reset card to monitor mode */
		if(local->nic_active) {
			vt_ar5k_nic_stop(dev);
			vt_ar5k_nic_open(dev, AR5K_RX_PROM);
		}
	} else {
		VPRINT("disable monitor mode\n");
		local->monitor = 0;
		dev->type = local->dev_type; /* store type */
		
		/* reset card to normal mode */
		if(local->nic_active) {
			vt_ar5k_nic_stop(dev);
			vt_ar5k_nic_open(dev, local->rx_filter);
		}
	}

	VEXIT();
}

int vt_ar5k_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd) 
{
	vt_local_t *local = (vt_local_t *) dev->priv;
	struct iwreq *wrq = (struct iwreq *) ifr;
	unsigned long flags;
	int val;
	int ret = 0;

	spin_lock_irqsave(&local->lock, flags);
	
	switch (cmd) {
	case SIOCGIWNAME:
		if(local->conf->turbo_enable) {
			strcpy(wrq->u.name, "IEEE 802.11a+");
		} else {
			strcpy(wrq->u.name, "IEEE 802.11a");
		}
		
		break;

	case SIOCSIWFREQ:
		if (!capable(CAP_NET_ADMIN)) {
			ret = -EPERM;
			break;
		}

		VDEBUG("set channel %d\n", wrq->u.freq.m);
		
		if(wrq->u.freq.e != 0) {
			ret = -EINVAL;
			break;
		}

		ret = vt_ar5k_set_channel(local, (u32)wrq->u.freq.m);
		break;

	case SIOCGIWFREQ:
		wrq->u.freq.e = 0;
		wrq->u.freq.m = local->conf->channel;
		break;

	case SIOCSIWMODE:
		if (!capable(CAP_NET_ADMIN)) {
			ret = -EPERM;
			break;
		}

		switch(wrq->u.mode) {
		case IW_MODE_MONITOR:
			vt_ar5k_ioctl_monitor(dev, 1); /* enable monitor */
			break;
		case IW_MODE_INFRA:
			if(local->monitor)
				vt_ar5k_ioctl_monitor(dev, 0); /* disable monitor */
			break;
		default:
			VDEBUG("not implemented\n");
			ret = -EINVAL;
		}
		break;
			
	case SIOCGIWMODE:
		if(local->monitor) {
			wrq->u.mode = IW_MODE_MONITOR;
		} else {
			switch(local->conf->bsstype) {
			case ANY_BSS:
				wrq->u.mode = IW_MODE_INFRA;
				break;
			case INFRASTRUCTURE_BSS:
				wrq->u.mode = IW_MODE_ADHOC;
				break;
			case INDEPENDENT_BSS:
				wrq->u.mode = IW_MODE_MASTER;
				break;
			default:
				wrq->u.mode = -EINVAL;
			}
		}
		break;
		
	case SIOCGIWPRIV:
		if (wrq->u.data.pointer) {
			struct iw_priv_args privtab[] = {
				{ SIOCDEVPRIVATE + 0x0,
				  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1,
				  0, "led_mode" },
				{ SIOCDEVPRIVATE + 0x1,
				  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1,
				  0, "turbo_enable" },
				{ SIOCDEVPRIVATE + 0x2,
				  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1,
				  0, "swi" },
				{ SIOCDEVPRIVATE + 0x3,
				  IW_PRIV_TYPE_INT | IW_PRIV_SIZE_FIXED | 1,
				  0, "prism" },
			};
			
			ret = verify_area(VERIFY_WRITE, wrq->u.data.pointer, sizeof(privtab));
			if (ret)
				break;
			
			wrq->u.data.length = sizeof(privtab) / sizeof(privtab[0]);
			if (copy_to_user(wrq->u.data.pointer, privtab, sizeof(privtab)))
				ret = -EFAULT;
		}
		break;
		
	case SIOCDEVPRIVATE + 0x0: /* led_mode */
		if (!capable(CAP_NET_ADMIN)) {
			ret = -EPERM;
			break;
		}
		
		val = *((int*)wrq->u.name);
		
		/* store LED mode */
		local->led_mode = val ? 1 : -1;
		
		if (val > 0) {
			VT_WLAN_OUT32(AR5K_PCICFG_LED_ACT, AR5K_PCICFG); 
		} else {
			VT_WLAN_OUT32(AR5K_PCICFG_LED_PEND, AR5K_PCICFG); 
		}
		break;

	case SIOCDEVPRIVATE + 0x1: /* turbo_enable */
		if (!capable(CAP_NET_ADMIN)) {
			ret = -EPERM;
			break;
		}

		val = *((int*)wrq->u.name);

		/* store turbo mode */
		local->conf->turbo_enable = val ? 1 : 0;
  
		if (val) {
			VT_WLAN_OUT32(PHY_FC_TURBO_MODE, PHY_FRAME_CONTROL);
		} else {
			VT_WLAN_OUT32(VT_WLAN_IN32(PHY_FRAME_CONTROL) & ~(PHY_FC_TURBO_MODE), PHY_FRAME_CONTROL);
		}

		ret = vt_ar5k_set_channel(local, val ? turbo_channel : channel);
		break;

	case SIOCDEVPRIVATE + 0x2: /* fire software interrupt - for testing */
		VT_WLAN_OUT32((VT_WLAN_IN32(AR5K_CR) | AR5K_CR_SWI), AR5K_CR); 
		break;

	case SIOCDEVPRIVATE + 0x3: /* enable prism header support */
		if (!capable(CAP_NET_ADMIN)) {
			ret = -EPERM;
			break;
		}

		val = *((int*)wrq->u.name);

		/* use prism header? */
		local->prism_header = val ? 1 : 0;

		/* reset monitor to new mode */
		if(local->monitor) { 
			vt_ar5k_ioctl_monitor(dev, 0);
			vt_ar5k_ioctl_monitor(dev, 1);
		}				

		break;
	default:
#ifdef VT_DEBUG
		VPRINT("ioctl 0x%04X not supported!\n", cmd);
#endif
		ret = -EOPNOTSUPP;
		break;
	}

	spin_unlock_irqrestore(&local->lock, flags);
  
	return(ret);
}
