/*                                                        -*- linux-c -*-
 *   _  _ ____ __ _ ___ ____ ____ __ _ _ _ _ | 
 * .  \/  |--| | \|  |  |--< [__] | \| | _X_ | s e c u r e  s y s t e m s
 *
 * .vt|ar5k - PCI/CardBus 802.11a WirelessLAN driver for Atheros AR5k chipsets
 *
 * Copyright (c) 2002, .vantronix | secure systems
 *                     and Reyk Floeter <reyk@vantronix.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <vt_ar5kconfig.h>
#include <vt_ar5k.h>
#include <vt_ar5kinitregs.h>

#define DESCRIPTION "PCI/CardBus 802.11a WirelessLAN driver for Atheros AR5k chipsets" 
#define AUTHOR "Reyk Floeter <reyk@vantronix.net>, (C) 2002 by .vantronix | secure systems and Reyk Floeter"
#define MODNAME "AR5000"

char *version = DESCRIPTION "\n" AUTHOR;
char *dev_info = "vt_ar5k";
pci_link_t *dev_list = NULL; 

/* atheros default- alike */
char vt_ar5k_mac_addr[6] = {0x00, 0x03, 0x7f, 0x00, 0x00, 0x23}; 
//char vt_ar5k_bssid[6] = {0x44, 0x44, 0x44, 0x44, 0x44, 0x44}; // 00:20:E0:3D:00:28
char vt_ar5k_bssid[6] = {0x00, 0x20, 0xe0, 0x3d, 0x00, 0x28}; // 00:20:E0:3D:00:28

/** generic driver options **/
struct pci_device_id pci_id_tbl[]  __devinitdata = {
	{PCIVENDOR_ATHEROS, CB_DEVICE_ATHEROS, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 
	 /* Driver data, we just put the name here */
	 (unsigned long)"Atheros 802.11a CardBus Wireless LAN AR5000"},
	{PCIVENDOR_ATHEROS, PCIDEVICE_ATHEROS, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 
	 (unsigned long)"Atheros 802.11a PCI Wireless LAN AR5000"},
	{0, 0, 0, 0, 0, 0, 0}
};

struct pci_driver vt_ar5k_pci_drv_id = {
	.name     = "ar5k", 
	.id_table = pci_id_tbl,
	.probe    = vt_ar5k_probe,
	.remove   = vt_ar5k_remove,
#ifdef CONFIG_PM
	.suspend  = vt_ar5k_suspend,
	.resume   = vt_ar5k_resume,
#endif /* CONFIG_PM */
};

/** module options **/
MODULE_DESCRIPTION(DESCRIPTION);
MODULE_AUTHOR(AUTHOR);
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif
MODULE_DEVICE_TABLE(pci, pci_id_tbl);

int mtu = 1600 - sizeof(wlan_frame_hdr_t) - 4;
MODULE_PARM(mtu, "i");
MODULE_PARM_DESC(mtu, "maximum transfer unit");

char essid[WLAN_ELE_SSID_SIZE + 1] = "vantronix";
MODULE_PARM(essid, "c" __MODULE_STRING(WLAN_ELE_SSID_SIZE));
MODULE_PARM_DESC(essid, "set ESSID (default: vantronix)");

char reg_domain[AR5K_REG_SIZE];
MODULE_PARM(reg_domain, "c" __MODULE_STRING(AR5K_REG_SIZE));
MODULE_PARM_DESC(reg_domain, "please specify your regulation domain (no default)");

int channel = 64;
MODULE_PARM(channel, "i");
MODULE_PARM_DESC(channel, "select default 802.11a channel (64)");

int turbo_channel = 58;
MODULE_PARM(turbo_channel, "i");
MODULE_PARM_DESC(turbo_channel, "select default turbo mode channel (58)");

int turbo_enable = 0;
MODULE_PARM(turbo_enable, "i");
MODULE_PARM_DESC(turbo_enable, "enable turbo mode (> 0)");

int led_mode = 0;
MODULE_PARM(led_mode, "i");
MODULE_PARM_DESC(led_mode, "LED mode (> 0 = activity status, < 1 = pending status, default 0)");

int opmode = 0;
MODULE_PARM(opmode, "i");
MODULE_PARM_DESC(opmode, "initial operation mode (0:station [default], 1:ad-hoc, 2:ap)");

int vt_ar5k_probe(struct pci_dev *pdev, const struct pci_device_id *id) 
{
	unsigned long phymem;
	unsigned long mem = 0;
	pci_link_t *link = NULL;
	vt_local_t *local = NULL;
	struct net_device *dev = NULL;
	sta_config_t *conf = NULL;
	struct iw_statistics *wext_stats = NULL;
	static int cards_found /* = 0 */;

	VENTER();
	
	if (pci_enable_device(pdev))
		return(-EIO);

	if(!(phymem = pci_resource_start(pdev, 0)))
		goto err_out;

	/* enable bus master */
	pci_set_master(pdev);

	if(pci_request_regions(pdev, MODNAME)) {
		VINIT("cannot reserve PCI memory region\n");
		goto err_out;
	}

	if((mem = 
	    (unsigned long) ioremap(phymem, pci_resource_len(pdev, 0))) == 0) {
		VINIT("cannot remap PCI memory region\n");
		goto err_out_free_res;
	}

	/* enable DMA */
	if (pci_set_dma_mask(pdev, 0xffffffff)) {
                VINIT("no suitable DMA available.\n");
                goto fail;
	}

	link = kmalloc(sizeof(pci_link_t), GFP_KERNEL);  
	local = kmalloc(sizeof(vt_local_t), GFP_KERNEL);
	dev = kmalloc(sizeof(struct net_device), GFP_KERNEL);
	conf = kmalloc(sizeof(sta_config_t), GFP_KERNEL);
	wext_stats = kmalloc(sizeof(struct iw_statistics), GFP_KERNEL);

	if (wext_stats != NULL) {
		memset(wext_stats, 0, sizeof(struct iw_statistics));
	} else {
		VINIT("out of memory\n");
		goto fail;
	}
	
	if ((local != NULL) && (dev != NULL) && (link != NULL) 
	    && (conf != NULL)) {
		memset(link, 0, sizeof(pci_link_t));
		memset(local, 0, sizeof(vt_local_t));
		memset(dev, 0, sizeof(struct net_device));
		memset(conf, 0, sizeof(sta_config_t));
	} else {
		VINIT("out of memory\n");
		goto fail;
	}

	cards_found++;
	
	local->dev = dev;
	link->priv = local;
	dev->priv = local;
	local->link = link;
	link->next = dev_list;
	dev_list = link;
	local->conf = conf;
	local->wext_stats = wext_stats;

	spin_lock_init(&local->lock);
	
	ether_setup(dev);
	
	local->io_port = phymem;
	dev->irq = local->irq = pdev->irq;

	dev->mem_start = mem;
	dev->mem_end = mem + pci_resource_len(pdev, 0);

	memcpy(dev->name, "wlan%d", 7);

	VINIT("found [mem_start:0x%lx, mem_end:0x%lx, len: %ld, irq:%d]\n", 
	      dev->mem_start, dev->mem_end, 
	      dev->mem_end - dev->mem_start, dev->irq);
	
	netif_stop_queue(dev);
	
	SET_MODULE_OWNER(dev);
	
	dev->mtu = mtu;
	dev->priv = local;
	dev->open = vt_ar5k_open;
	dev->stop = vt_ar5k_stop;
	dev->do_ioctl = vt_ar5k_ioctl;
	dev->get_stats = vt_ar5k_get_stats;
	dev->hard_start_xmit = vt_ar5k_start_xmit;
	dev->get_wireless_stats = vt_ar5k_wireless_stats;
	
	if (vt_ar5k_config(local)) {
		VPRINT("configuration failed\n");
		goto fail;
	}
	
	pci_set_drvdata(pdev, dev);
	
	if (register_netdev(dev)) {
		VPRINT("register_netdev() failed!\n");
		goto fail;
	}
	
	if(vt_ar5k_nic_init(local)) {
		VPRINT("nic initialization failed!\n");
		goto fail;
	}

	if(vt_ar5k_config_registers(local)) {
		VPRINT("register configuration failed!\n");
		goto fail;
	}

	vt_ar5k_init_proc(local);

	local->nic_ready = 1;

	VPRINT("card initialized successfully\n");
	
	VEXIT();
	
	return(0);
	
 fail:
	if (mem != 0) iounmap((void *) mem);
	if (link != NULL) kfree(link);
	if (local != NULL) kfree(local);
	if (dev != NULL) kfree(dev);
	if (conf != NULL) kfree(conf);
	if (wext_stats != NULL)	kfree(wext_stats);

 err_out_free_res:
	release_mem_region(phymem, pci_resource_len(pdev, 0));
	/* pci_release_regions(pdev); */

 err_out:	
	pci_disable_device(pdev);
	
	VPRINT("card initialization failed!\n");

	return(-ENODEV);
}

void __devexit vt_ar5k_remove(struct pci_dev *pdev) 
{
	struct net_device *dev = (struct net_device*) pci_get_drvdata(pdev);
	vt_local_t *local = (vt_local_t*) dev->priv;
	pci_link_t *link = (pci_link_t*)local->link;
	
	VENTER();
	
	VPRINT("resetting device\n");
	vt_ar5k_nic_exit(local);
	
	VPRINT("unregistering net device\n");
	vt_ar5k_remove_proc(local);

	iounmap((void*)(dev->mem_start));
	unregister_netdev(dev);
	kfree(link);
	kfree(local->conf);
	kfree(local->wext_stats);
	kfree(local);
	kfree(dev);
	
	pci_release_regions(pdev);
	pci_disable_device(pdev);
	pci_set_drvdata(pdev,NULL);
	
	VEXIT();
}

int vt_ar5k_nic_init(vt_local_t *local) 
{
	struct net_device *dev = (struct net_device*) local->dev;
	int i;
        int ret;
	
	VENTER();

	local->int_count = 0;

#ifdef USE_GPIO
	/* enable gpio[2] and set it to value 1 */
	if(vt_ar5k_gpio_out(local, 2) || 		
	   vt_ar5k_gpio_set(local, 2, 1))
		return(1);
#endif	

	/** step 1: reset and wakeup procedure... **/
	if (vt_ar5k_nic_resume(local)) {
		VPRINT("nic wakeup failed\n");
		return(1);
	}	

	/* get MAC revision */
	local->mac_rev = (VT_WLAN_IN16(AR5K_SREV) & AR5K_SREV_ID_M);
	
	/* enable turbo mode? this is an ar5k enhancement to 802.11a */
	if(local->conf->turbo_enable) {
		VT_WLAN_OUT32(PHY_FC_TURBO_MODE, PHY_FRAME_CONTROL);
	} else {
		VT_WLAN_OUT32(0, PHY_FRAME_CONTROL);
	}

	ret = vt_ar5k_nic_reset(local, AR5K_RC_PCU | AR5K_RC_DMA | AR5K_RC_D2 | AR5K_RC_F2);
	if (ret) 
		return(ret);
	udelay(2300);
	
	/* wakeup again */
	if (vt_ar5k_nic_resume(local))
		return(1);
	
	ret = vt_ar5k_nic_reset(local, 0);
	if (ret) 
		return(ret);

	/* some gpio stuff... (???) */
#ifdef USE_GPIO
	if(vt_ar5k_gpio_out(local, 2) ||
	   vt_ar5k_gpio_set(local, 2, 1)) 
		return(1);
	udelay(5000); /* sleep for 5ms */
	VT_WLAN_OUT32(0x14, (PHY_BASE+(0x34<<2))); /* clear a2's reg5 */
	vt_ar5k_gpio_set(local, 2, 0); /* gpio[2] = 0 */
	vt_ar5k_gpio_set(local, 2, 1); /* gpio[2] = 1 (burnin) */
	vt_ar5k_gpio_set(local, 2, 0); /* gpio[2] = 0 */
#endif	

	/* get PHY revision */
	local->phy_rev = VT_WLAN_IN16(PHY_CHIP_ID);

	/* messy business reading */
	VT_WLAN_OUT32(0x00001c16, (PHY_BASE+(0x34<<2)));
	for (i=0; i<4; i++) {
		VT_WLAN_OUT32(0x00010000, (PHY_BASE+(0x20<<2)));
	}
	
	local->a2_rev = 
		(u16)(vt_ar5k_bitswap(((VT_WLAN_IN16(PHY_BASE + (256<<2)) >> 28) 
				       & 0xf), 4) + 1);
	
	/* gpio.... AGAIN */
	if ((local->gpio0_bit = vt_ar5k_gpio_get(local, 0)) == 0) {
		/* already closed, force no rf */
		vt_ar5k_gpio_int(local, 1);
	} else {
		vt_ar5k_gpio_int(local, 0);
	}

	VPRINT("starting card in %s mode\n", 
	       turbo_enable ? "turbo" : "standard");

	printk("%s (%s): MAC revision 0x%04x\n"
	       "%s (%s): PHY revision 0x%04x\n"
	       "%s (%s): A2  revision 0x%04x\n", 
	       dev_info, dev->name, local->mac_rev,
	       dev_info, dev->name, local->phy_rev,
	       dev_info, dev->name, local->a2_rev);
	
	VEXIT();
	
	return(0);
}

int vt_ar5k_nic_exit(vt_local_t *local) 
{
	VENTER();
	
	if (vt_ar5k_nic_suspend(local))
		return(1);

	VEXIT();
	
	return(0);
}

int vt_ar5k_nic_suspend(vt_local_t *local)
{
	struct net_device *dev = (struct net_device*) local->dev;
	unsigned long flags;
	u32 i;
	
	VENTER();

	spin_lock_irqsave(&local->lock, flags);

	local->nic_ready = 0;

	VT_WLAN_OUT32(AR5K_SCR_SLE_FSLEEP, AR5K_SCR);
	
	for (i = 0; i < POWER_UP_TIME / 200; i++) {
		if ((VT_WLAN_IN16(AR5K_PCICFG) & AR5K_PCICFG_SPWR_DN) == 0) {
			spin_unlock_irqrestore(&local->lock, flags);
			return(0);
		}
		udelay (200);
		VT_WLAN_OUT32(AR5K_SCR_SLE_FSLEEP, AR5K_SCR);
	}
	
	spin_unlock_irqrestore(&local->lock, flags);

	VEXIT();
	
	return(1);
}

int vt_ar5k_nic_resume(vt_local_t *local)
{
	struct net_device *dev = (struct net_device *)local->dev;
	unsigned long flags;
	u32 i;

	VENTER();
	
	spin_lock_irqsave(&local->lock, flags);

	local->nic_ready = 1;

	VT_WLAN_OUT32(AR5K_SCR_SLE_FWAKE, AR5K_SCR);
	
	for (i = 0; i < 2000 / 200; i++) {
		if ((VT_WLAN_IN16(AR5K_PCICFG) & AR5K_PCICFG_SPWR_DN) == 0) {
			spin_unlock_irqrestore(&local->lock, flags);
			return(0);		}
		udelay (200);
		VT_WLAN_OUT32(AR5K_SCR_SLE_FWAKE, AR5K_SCR);
	}
	
	spin_unlock_irqrestore(&local->lock, flags);

	VEXIT();
	
	return(1);
}

#ifdef CONFIG_PM
int vt_ar5k_resume(struct pci_dev *pdev)
{
	struct net_device *dev = pci_get_drvdata (pdev);
	vt_local_t *local = (vt_local_t *)dev->priv;

	VENTER();

#if 0	
	if (!netif_running(dev)) {
		VPRINT("resume failed - %s is running\n", dev->name);
		return(0);
	}

	netif_device_attach(dev);
#endif

	if(vt_ar5k_nic_resume(local)) {
		VPRINT("resume failed\n");
		return(1);
	}
	
	VEXIT();
	
	return(0);
}

int vt_ar5k_suspend(struct pci_dev *pdev, u32 state)
{
	struct net_device *dev = pci_get_drvdata (pdev);
	vt_local_t *local = (vt_local_t *)dev->priv;

#if 0
	if (!netif_running(dev)) {
		VPRINT("suspend failed - %s is running\n", dev->name);
		return(0);
	}

	netif_device_detach(dev);
#endif

	if(vt_ar5k_nic_suspend(local)) {
		VPRINT("suspend failed\n");
		return(1);
	}

	return(0);
}
#endif /* CONFIG_PM */

int vt_ar5k_nic_reset(vt_local_t *local, u32 val) 
{
	struct net_device *dev = (struct net_device*) local->dev;
	u32 mask = val ? val : ~0, read;
	int i, result = 1;

	VENTER();

	/* read added before writing, 'cause kernel hanged when reinserting module */
	read = VT_WLAN_IN32(AR5K_RC);
	VT_WLAN_OUT32(val, AR5K_RC);
	
	/* need to wait at least 128 clocks when reseting PCI before read */
	udelay(15);
	
	val  &= AR5K_RC_PCU | AR5K_RC_DMA | AR5K_RC_D2 | AR5K_RC_F2;
	mask &= AR5K_RC_PCU | AR5K_RC_DMA | AR5K_RC_D2 | AR5K_RC_F2;
	
	for (i = 0; i < 10000; i++) {
		read = VT_WLAN_IN16(AR5K_RC);
		if ((read & mask) != val) 
			udelay(50);
		else {
			result = 0;
			break;
		}
	}

	if (result)	
		VPRINT("nic reset failed %x != %x failed\n", val,
		       (VT_WLAN_IN16(AR5K_RC) & mask));
	
	VEXIT();
	return(result);
}

#ifdef USE_SOFTIRQ
static void vt_ar5k_rx_tasklet(unsigned long data)
{
	vt_local_t *local = (vt_local_t*)data;
	vt_ar5k_rx(local);
}
#endif

int vt_ar5k_config(vt_local_t *local) 
{
        int ret = 0;

	VENTER();

	ret = vt_ar5k_config_register(local);
	
	/** local data configuration **/
	local->led_mode = led_mode;
	local->rx_filter = INIT_RX_FILTER;
	local->opmode = opmode; /* sta / ap */
	local->monitor = 0;
	local->prism_header = 0;
	local->rx_drop = 0; /* dirty frame */
	local->tx_active = 0;

	/** station data configuration **/
	local->conf->turbo_enable = turbo_enable;
	local->conf->channel = turbo_enable ? turbo_channel : channel;
	local->conf->qos_enabled = 0; /* TODO */
	local->conf->cwmin = 0; /* TODO */
	local->conf->aifs = 0; /* TODO */
	local->conf->bsstype = ANY_BSS;

	/** setup software interrupts **/
#ifdef USE_SOFTIRQ
	VT_TASKLET_INIT(&local->rx_tasklet,
			vt_ar5k_rx_tasklet, (unsigned long) local);
#endif

	VEXIT();

	return(ret);
}

int vt_ar5k_config_registers(vt_local_t *local) 
{	
	struct net_device *dev = (struct net_device*)local->dev;
	int ret = 0;
	u32 ul_address_low, ul_address_high;
	u32 i, data;
	int is_ap = 0; 
	u32 channel_ieee;
	wlan_mac_addr_t mac;

	VENTER();

	/** read mac address from eeprom or use default address on failure **/
	if(vt_ar5k_eeprom_read_mac(local, &mac) == 0)
		memcpy(vt_ar5k_mac_addr, mac.octets, WLAN_MAC_ADDR_SIZE);
	
	memcpy(local->conf->mac_perm_addr.octets, vt_ar5k_mac_addr, WLAN_MAC_ADDR_SIZE);
	memcpy(local->conf->mac_addr.octets, vt_ar5k_mac_addr, WLAN_MAC_ADDR_SIZE);
	memcpy(local->conf->bssid.octets, vt_ar5k_bssid, WLAN_MAC_ADDR_SIZE);
	memcpy(&dev->dev_addr, local->conf->mac_addr.octets, WLAN_MAC_ADDR_SIZE);
	
	VPRINT("hardware address " MACSTR "\n", MAC2STR(dev->dev_addr));

	/** set mac address, set mode **/
	memcpy(&ul_address_low, &(local->conf->mac_addr.octets[0]), 4);
	memcpy(&ul_address_high, &(local->conf->mac_addr.octets[4]), 2);

	ul_address_high = 0x0000ffff & cpu_to_le32(ul_address_high);

	VT_WLAN_OUT32(cpu_to_le32(ul_address_low), AR5K_STA_ID0);
	VT_WLAN_OUT32(ul_address_high, AR5K_STA_ID1);

#if 0
	is_ap = memcmp(local->conf->mac_addr, local->conf->bssid);
#endif
	
	if (is_ap) {
		VT_WLAN_OUT32(AR5K_STA_ID1_STA_AP | ul_address_high | 
			      AR5K_STA_ID1_PSPOLLDIS | AR5K_STA_ID1_DESC_ANT,
			      AR5K_STA_ID1);
	} else if (local->conf->bsstype == INDEPENDENT_BSS) {
		VT_WLAN_OUT32(AR5K_STA_ID1_AD_HOC | ul_address_high | 
			      AR5K_STA_ID1_PSPOLLDIS | AR5K_STA_ID1_DESC_ANT, 
			      AR5K_STA_ID1);
	} else { 
		VT_WLAN_OUT32(ul_address_high | 
			      AR5K_STA_ID1_PSPOLLDIS | AR5K_STA_ID1_PWR_SAV,
			      AR5K_STA_ID1);
	} 

	/** setup bssid **/
	memcpy(&ul_address_low, &(local->conf->bssid.octets[0]), 4);
	memcpy(&ul_address_high, &(local->conf->bssid.octets[4]), 2);

	ul_address_high = 0x0000ffff & cpu_to_le32(ul_address_high);

	VT_WLAN_OUT32(cpu_to_le32(ul_address_low), AR5K_BSS_ID0);
	VT_WLAN_OUT32(ul_address_high, AR5K_BSS_ID1);

	/** reset tx/rx descriptor pointers **/
	VT_WLAN_OUT32(0, AR5K_TXDP0);
	VT_WLAN_OUT32(0, AR5K_TXDP1);
	VT_WLAN_OUT32(0, AR5K_RXDP);

	/** reset interrupt register **/
	(void)VT_WLAN_IN32(AR5K_ISR); /* reset */
	VT_WLAN_OUT32(0, AR5K_IMR);
	VT_WLAN_OUT32(AR5K_IER_DISABLE, AR5K_IER);
	local->nic_ready = 0;

	/** set dma size - default is 128byte **/
	VT_WLAN_OUT32(AR5K_DMASIZE_256B, AR5K_TXCFG);
 	VT_WLAN_OUT32(AR5K_DMASIZE_256B | AR5K_RXCFG_ZLFDMA, AR5K_RXCFG); // XXX kaleda dadejau zero len dma
	VT_WLAN_OUT32(0, AR5K_CFG); /* big endian */
	
	/** set timeouts **/
	VT_WLAN_OUT32(INIT_TOPS, AR5K_TOPS);
	VT_WLAN_OUT32(INIT_RXNOPKT, AR5K_RXNPTO);
	VT_WLAN_OUT32(INIT_RPGTO, AR5K_RPGTO);
	VT_WLAN_OUT32(INIT_TXNOPKT, AR5K_TXNPTO);

	/** disable forced sleep **/
	VT_WLAN_OUT32(0, AR5K_SFR);

	/** set led status **/
	if (is_ap) {
		data = AR5K_PCICFG_LED_ACT;
	} else {
		data = AR5K_PCICFG_CLKRUNEN | AR5K_PCICFG_LED_PEND;
	}

	VT_WLAN_OUT32(data | AR5K_PCICFG_BCTL, AR5K_PCICFG);

	/** reset mac configuration and status register **/
	VT_WLAN_OUT32(cpu_to_le32(INIT_CONFIG_STATUS), AR5K_CFG);

	/** reset mib counters **/
	VT_WLAN_OUT32(0, AR5K_MIBC);
	
	/** set 802.11e tunable cwmin **/
	if (!local->conf->qos_enabled) {
		local->conf->cwmin = INIT_CWMIN;
	}

	data = 1;
	while (data < local->conf->cwmin)
		data = (data << 1) | 1;

	/** set retry limits **/
	data = (data << AR5K_RETRY_LMT_CW_S)
		| (INIT_SLG_RETRY << AR5K_RETRY_LMT_SLG_S)
		| (INIT_SSH_RETRY << AR5K_RETRY_LMT_SSH_S);
	
	if (local->conf->swretry_enabled) {
		data |= ((local->conf->limit_tx_retries / 2) << AR5K_RETRY_LMT_LG_S)
			| (local->conf->limit_tx_retries / 2);
	} else {
		data |= (INIT_LG_RETRY << AR5K_RETRY_LMT_LG_S) | INIT_SH_RETRY;
	}
	VT_WLAN_OUT32(data, AR5K_RETRY_LMT);

	/** set initial filters **/
	VT_WLAN_OUT32(AR5K_RX_FILTER_ALL, AR5K_RX_FILTER);
	VT_WLAN_OUT32(0, AR5K_MCAST_FIL0);
	VT_WLAN_OUT32(0x00000002, AR5K_MCAST_FIL1);
	VT_WLAN_OUT32(0, AR5K_TX_MASK0);
	VT_WLAN_OUT32(0, AR5K_TX_MASK1);
	VT_WLAN_OUT32(1, AR5K_CLR_TMASK);

	/** set transmit fifo threshold **/
	VT_WLAN_OUT32(INIT_TX_FIFO_THRESHOLD, AR5K_TRIG_LEV);

	/** reset pcu control register **/
	VT_WLAN_OUT32(0, AR5K_DIAG_SW);

	/** reset and enable beacon registers and timers **/
	if (is_ap) {
		VT_WLAN_OUT32(INIT_BCON_CNTRL_REG | AR5K_BCR_APMODE, AR5K_BCR);
	} else if (local->conf->bsstype == INDEPENDENT_BSS) {
		VT_WLAN_OUT32(INIT_BCON_CNTRL_REG | AR5K_BCR_STAMODE, AR5K_BCR);
	} else {
		VT_WLAN_OUT32(INIT_BCON_CNTRL_REG, AR5K_BCR);
	}

	(void)VT_WLAN_IN32(AR5K_BSR); /* reset */
	VT_WLAN_OUT32(INIT_RSSI_THR, AR5K_RSSI_THR);
	VT_WLAN_OUT32(INIT_NEXT_BEACON_TIME, AR5K_TIMER0);
	VT_WLAN_OUT32(INIT_TSF_CLOCK_LOW, AR5K_TSF_L32);
	VT_WLAN_OUT32(INIT_DMA_BEACON_ALERT, AR5K_TIMER1);
	VT_WLAN_OUT32(INIT_SW_BCON_ALERT, AR5K_TIMER2);
	VT_WLAN_OUT32(0, AR5K_CFP_DUR);
	VT_WLAN_OUT32(INIT_CFP_INTERVAL, AR5K_CFP_PERIOD);
	VT_WLAN_OUT32(1, AR5K_TIMER3);
	VT_WLAN_OUT32(INIT_BEACON_CONTROL, AR5K_BEACON); /* enable beacons */

	if (!local->conf->qos_enabled) 
		local->conf->aifs = 2;

	/** set turbo/base mode parameters **/
	if(local->conf->turbo_enable) {
		/** turbo mode **/
		VT_WLAN_OUT32(INIT_SLOT_TIME_TURBO, AR5K_SLOT_TIME);
		VT_WLAN_OUT32(INIT_ACK_CTS_TIMEOUT_TURBO, AR5K_TIME_OUT);
		VT_WLAN_OUT32(INIT_TRANSMIT_LATENCY_TURBO, AR5K_USEC);
		data = ((INIT_SIFS_TURBO + local->conf->aifs * INIT_SLOT_TIME_TURBO) << AR5K_IFS0_DIFS_S)
			| INIT_SIFS_TURBO;
		VT_WLAN_OUT32(data, AR5K_IFS0);
		VT_WLAN_OUT32(INIT_PROTO_TIME_CNTRL_TURBO, AR5K_IFS1);
		VT_WLAN_OUT32((VT_WLAN_IN32(PHY_BASE+(17<<2)) & ~0x7F) |
			      0x38, PHY_BASE+(17<<2));
		VT_WLAN_OUT32(PHY_FC_SERVICE_ERR | PHY_FC_TX_UNDER_ERR |
			      PHY_FC_ILLLEN_ERR | PHY_FC_ILLRATE_ERR |
			      PHY_FC_PARITY_ERR | PHY_FC_TIMING_ERR |
			      0x2020 | PHY_FC_TURBO_MODE | PHY_FC_TURBO_SHORT,
			      PHY_FRAME_CONTROL);
	} else {
		/** base mode **/
		VT_WLAN_OUT32(INIT_SLOT_TIME, AR5K_SLOT_TIME);
		VT_WLAN_OUT32(INIT_ACK_CTS_TIMEOUT, AR5K_TIME_OUT);
		VT_WLAN_OUT32(INIT_TRANSMIT_LATENCY, AR5K_USEC);
		data = ((INIT_SIFS + local->conf->aifs * INIT_SLOT_TIME) << AR5K_IFS0_DIFS_S)
			| INIT_SIFS;
		VT_WLAN_OUT32(data, AR5K_IFS0);
		VT_WLAN_OUT32(INIT_PROTO_TIME_CNTRL, AR5K_IFS1);
		VT_WLAN_OUT32((VT_WLAN_IN32(PHY_BASE+(17<<2)) & ~0x7F) |
			      0x1C, PHY_BASE+(17<<2));
		VT_WLAN_OUT32(PHY_FC_SERVICE_ERR | PHY_FC_TX_UNDER_ERR | 
			      PHY_FC_ILLLEN_ERR | PHY_FC_ILLRATE_ERR |
			      PHY_FC_PARITY_ERR | PHY_FC_TIMING_ERR | 0x1020,
			      PHY_FRAME_CONTROL);
	}

	/** write initial phyreg values **/
	for (i = 0; i < local->init_regs.count; i++) 	{
		VT_WLAN_OUT32(local->init_regs.values[i].value,
			      local->init_regs.values[i].offset);
	}

	channel_ieee = local->conf->channel;

	/* XXX TODO: set IEEE channel numbers by reg domain */

	/* XXX TODO: set power and ref board antenna values if they were not set by the INI */

	vt_ar5k_set_channel(local, channel_ieee);

	VPRINT("status 0x%08lx\n", 
	       (unsigned long)VT_WLAN_IN32(AR5K_CFG));

	VEXIT();

	return(ret);
}

int vt_ar5k_set_channel(vt_local_t *local, u32 channel_ieee)
{
	struct net_device *dev = (struct net_device*)local->dev;
	u32 data, data_orig;
	int i;

	VENTER();

	VT_WLAN_OUT32(PHY_ACTIVE_DIS, PHY_ACTIVE);
	udelay(1000);

	/* FCC and Japan ,-) */
	if(local->conf->turbo_enable) {
		switch(channel_ieee) {
		case 3:
		case 42:
		case 50:
		case 58:
			break;
		default:
			VDEBUG("unsupported channel in turbo mode\n");
			return(-EOPNOTSUPP);
		}
	} else {
		switch(channel_ieee) {
		case 4:
		case 8:
		case 34:
		case 36:
		case 38:
		case 40:
		case 42:
		case 44:
		case 46:
		case 48:
		case 52:
		case 56:
		case 60:
		case 64:
			break;
		default:
			VDEBUG("unsupported channel\n");
			return(-EOPNOTSUPP);
		}
	}

	/** set the channel */
	data_orig = vt_ar5k_bitswap((channel_ieee - 24)/2, 5);
	data = (data_orig << 1) | (1 << 6) | 0x1;
	VT_WLAN_OUT32(data, PHY_BASE+(0x27<<2));
	VT_WLAN_OUT32(0, PHY_BASE+(0x30<<2));

	/** wait, activate d2, wait **/
	udelay(1000);
	VT_WLAN_OUT32(PHY_ACTIVE_EN, PHY_ACTIVE);
	udelay(1000);

	/** calibrate it and poll the bit going to 0 for completion **/
	VT_WLAN_OUT32(VT_WLAN_IN32(PHY_AGC_CONTROL) | PHY_AGC_CONTROL_CAL, 
		      PHY_AGC_CONTROL);

	for (i = 0; i < 10000; i++) {
		if ((VT_WLAN_IN32(PHY_AGC_CONTROL) 
		     & PHY_AGC_CONTROL_CAL) == 0)
			break;
		udelay(50);
	}

	/** set the noise floor **/
	VT_WLAN_OUT32(VT_WLAN_IN32(PHY_AGC_CONTROL) | PHY_AGC_CONTROL_NF,
		      PHY_AGC_CONTROL);

	/** save the current channel **/
	local->conf->hw_channel = local->conf->channel = channel_ieee;

	VEXIT();

	return(0);
}

int vt_ar5k_config_register(vt_local_t *local) 
{
	const wlan_register_val_t *init_regs;

	VENTER();
	
	init_regs = vt_ar5k_register_init;
	local->init_regs.count = 0;
	
	VINIT("setting up initial configuration register\n");

	while (init_regs[local->init_regs.count].offset != 0xffffffff) {
		if (local->init_regs.count > MAX_REG_COUNT) {
			VINIT("init register overflow!\n");
			return(1);
		}
		local->init_regs.values[local->init_regs.count].offset =
			init_regs[local->init_regs.count].offset;
		local->init_regs.values[local->init_regs.count].value =
			init_regs[local->init_regs.count].value;
		local->init_regs.count++;
	}
	
	VEXIT();

	return(0);

}

u32 vt_ar5k_bitswap(u32 val, int bit_count) 
{
	u32 retval;
	u32 bit;
	int i;
	
	VENTER();

	retval = 0;
	
	for (i = 0; i < bit_count; i++) {
		bit = (val >> i) & 1;
		retval = (retval << 1) | bit;
	}
	
	VEXIT();

	return retval;
}

int vt_ar5k_eeprom_read(vt_local_t *local, u32 offset, u16 *data)
{
	struct net_device *dev = (struct net_device*) local->dev;
	int timeout = 10000;   
	u32 status;
	u32 pcicfg;

	VENTER();

	/** enable eeprom access */
	pcicfg = VT_WLAN_IN32(AR5K_PCICFG);
	VT_WLAN_OUT32(pcicfg | AR5K_PCICFG_EEAE, AR5K_PCICFG);

	/* prime read pump */
	(void)VT_WLAN_IN32(AR5K_EEPROM_BASE + (4 * offset));
	
	while (timeout > 0) {
		udelay(1);
		status = VT_WLAN_IN32(AR5K_EEPROM_STATUS);
		if (status & AR5K_EEPROM_STAT_RDDONE) {
			if (status & AR5K_EEPROM_STAT_RDERR) {
				VPRINT("eeprom read access failed!\n");
				return(1);
			}
			status = VT_WLAN_IN32(AR5K_EEPROM_RDATA);
			*data = (u16)(status & 0xffff);
			return(0);
		}
		timeout--;
	}
	
	VPRINT("eeprom read timeout!\n");

	VEXIT();

	return(1);
}

int vt_ar5k_eeprom_read_mac(vt_local_t *local, wlan_mac_addr_t *omac)
{
	struct net_device *dev = (struct net_device*) local->dev;
	u32 total = 0, offset;
	u32 pcicfg;
	u16 data = 0;
	int octet = 0;
	wlan_mac_addr_t mac;

	VENTER();

	/** called from an early stage... be careful **/
	if(dev == NULL) {
		VPRINT("NULL dev pointer.\n");
		return(1);
	}

	if(omac == NULL) {
		VPRINT("NULL mac pointer.\n");
		return(1);
	}

	if (VT_WLAN_IN32(AR5K_CFG) & AR5K_CFG_EEBS) {
		VPRINT("eeprom is busy.\n");
		memset(&mac, 0, sizeof(wlan_mac_addr_t));
		return(1);
	}

	/** enable eeprom access **/
	pcicfg = VT_WLAN_IN32(AR5K_PCICFG);
	VT_WLAN_OUT32(pcicfg | AR5K_PCICFG_EEAE, AR5K_PCICFG);

	for(offset = 0x1f; offset >= 0x1d; offset--) {
		if (vt_ar5k_eeprom_read(local, offset, &data)) {
			VPRINT("eeprom read error\n");
			return(1);
		}

		total += data;
		mac.octets[octet + 1] = data & 0xff;
		mac.octets[octet] = data >> 8;
		octet += 2;
	}

	/** reset pci configuration register **/
	VT_WLAN_OUT32(pcicfg, AR5K_PCICFG);

	if ((!total) || total == (3*0xffff)) {
		VPRINT("total==%d\n", total);
		return(1); 
	}

	memcpy(omac->octets, mac.octets, WLAN_MAC_ADDR_SIZE);

	VEXIT();

	return(0);
}

void vt_ar5k_free_rx_queue(vt_local_t *local)
{
	struct net_device *dev = (struct net_device*)local->dev;
	u32 tmp_cr;

	VENTER();
	
	/* disable RX descriptors */
	tmp_cr = VT_WLAN_IN32(AR5K_CR) & (~AR5K_CR_RXE);
	VT_WLAN_OUT32(tmp_cr, AR5K_CR);

	/* write null pointer to RXDP register (just to make full clearing) */
	VT_WLAN_OUT32(0, AR5K_RXDP);
	
	if(local->rx_desc_size)
		pci_free_consistent(local->pdev, local->rx_desc_size, local->rx_desc, local->rx_desc_dma);
	if(local->rx_buf_size)
		pci_free_consistent(local->pdev, local->rx_buf_size, local->rx_buf, local->rx_buf_dma);

	VEXIT();
}

int vt_ar5k_setup_rx_queue(vt_local_t *local)
{
	struct net_device *dev = (struct net_device*)local->dev;
	ar5k_desc_t *ptr = NULL;
	ar5k_buf_t *buf;
	dma_addr_t desc_addr, buf_addr;
	int i = 0;

	VENTER();

	local->rx_count = MAX_RX_DESC_BUF;

	/* rx buffer */
	local->rx_buf_size = local->rx_count * sizeof(ar5k_buf_t);
	local->rx_buf = pci_alloc_consistent(local->pdev, 
					     local->rx_buf_size, 
					     &local->rx_buf_dma);
	
	if(!local->rx_buf_dma) {
		local->rx_buf_size = 0;
		VDEBUG ("local->rx_buf_dma == NULL\n");
		return -ENOMEM;
	}

	memset(local->rx_buf, 0, local->rx_buf_size);
	
	/* rx descriptor list */
	local->rx_desc_size = ((local->rx_count * sizeof(ar5k_desc_t)) 
			       + AR5K_DESC_ALIGN - 1) & ~(AR5K_DESC_ALIGN - 1);

	local->rx_desc = pci_alloc_consistent(local->pdev, 
					      local->rx_desc_size, 
					      &local->rx_desc_dma);
	
	if(!local->rx_desc_dma) {
		pci_free_consistent(local->pdev, local->rx_buf_size, local->rx_buf, local->rx_buf_dma);
		local->rx_buf_size = 0;
		local->rx_desc_size = 0;
		VDEBUG ("local->rx_desc_dma == NULL\n");
		return(-ENOMEM);
	}

	memset(local->rx_desc, 0, local->rx_desc_size);

	/* init buffer list */
	ptr = local->rx_first = local->rx_desc;
	buf = local->rx_buf;
	desc_addr = local->rx_desc_dma;
	buf_addr = local->rx_buf_dma;

	for(i = 0; i < local->rx_count; i++) {
		ptr->s.rx_compl_stat.done = 0;
		ptr->u.rx.buf_len = sizeof(ar5k_buf_t);
		ptr->u.rx.inter_req = 1;

		/* set receive buffer pointer */
		ptr->data = buf;
		ptr->buffer_ptr = buf_addr;

		desc_addr = desc_addr + sizeof(ar5k_desc_t);
		buf_addr = buf_addr + sizeof(ar5k_buf_t);

		if(i < (local->rx_count - 1)) {
			ptr->next = ptr + 1;
			ptr->link_ptr = desc_addr;
		} else {
			ptr->next = local->rx_first;
			ptr->link_ptr = local->rx_desc_dma; /* rx ring */
		}
		buf++;
		ptr++;
	}

	VT_WLAN_OUT32((u32)local->rx_desc_dma, AR5K_RXDP); /* set rx descriptor */
	VDEBUG ("set dma rx descriptor 0x%08x\n", (u32)local->rx_desc_dma);

	VEXIT();

	return(0);
}

#if VT_DEBUG > 2
static void vt_ar5k_dump(char *info, u8 *data, u32 datalen) 
{
	int i;

	VENTER();

	printk(KERN_DEBUG "%s data [%d]: 0x", info, datalen);
	for(i = 0; i < datalen; i++) {
		printk("%02x", data[i]);
	}
	printk("\n");

	VEXIT();
}
#endif

irqreturn_t vt_ar5k_interrupt(const int irq, void *dev_instance, struct pt_regs *regs) 
{
	struct net_device *dev = (struct net_device*)dev_instance;
	vt_local_t *local = (vt_local_t *)dev->priv;

	spin_lock(&local->lock);

	if(VT_WLAN_IN16(AR5K_INTPEND) & AR5K_INTPEND_TRUE) {
		u32 int_reg;

		VENTER();
		
		VT_WLAN_OUT32(AR5K_IER_DISABLE, AR5K_IER); /* disable interrupt */
		int_reg = VT_WLAN_IN16(AR5K_ISR);
		VDEBUG ("real irq: 0x%08x\n", int_reg);
	
		if(int_reg == AR5K_IMR_X) {
			VDEBUG("[%ld] interrupt... card removed (mask 0x%08x)\n", 
			       local->int_count, local->int_mask);
			VT_WLAN_OUT32(local->int_mask, AR5K_IMR); /* reset interrupt mask */
			VT_WLAN_OUT32(AR5K_IER_ENABLE, AR5K_IER); /* enable interrupt */
			spin_unlock(&local->lock);
			return IRQ_NONE;
		}
	
		if(int_reg & AR5K_ISR_DPERR) {
			VDEBUG("[%ld] interrupt... AR5K_ISR_DPERR\n", 
			       local->int_count);
		}
	
		if(int_reg & AR5K_ISR_SSERR) {
			VDEBUG("[%ld] interrupt... AR5K_ISR_SSERR\n", 
			       local->int_count);
		}
	
		if(int_reg & AR5K_ISR_MCABT) {
			VDEBUG("[%ld] interrupt... AR5K_ISR_MCABT\n", 
			       local->int_count);
		}
		if(int_reg & AR5K_ISR_RXNOPKT) {
			VDEBUG("[%ld] interrupt... AR5K_ISR_RXNOPKT\n", 
			       local->int_count);
		}
	
		if(int_reg & AR5K_ISR_TXERR) {
#if USE_TX
			ar5k_desc_t *desc = local->tx_desc;
	
			VPRINT("[%ld] interrupt... AR5K_ISR_TXERR\n", 
			       local->int_count);
	
			if(desc) {
				VPRINT("[%ld] tx (done=%u,ok=%u,underrun=%u,filtered=%u,timestamp=%u,seq=%u)\n", 
				       local->int_count,
				       desc->s.tx_compl_stat.done,
				       desc->s.tx_compl_stat.frm_xmit_ok,
				       desc->s.tx_compl_stat.fifo_underrun,
				       desc->s.tx_compl_stat.filtered,
				       desc->s.tx_compl_stat.send_timestamp,
				       desc->s.tx_compl_stat.seq_num);
			} else {
				VPRINT("tx desc == NULL!\n");
			}

			VT_WLAN_OUT32((VT_WLAN_IN32(AR5K_CR) | AR5K_CR_TXD0), AR5K_CR); /* disable transmit bit */
	
			local->tx_active = 0;
			local->stats.tx_errors++;
#endif
		}
	
		if(int_reg & AR5K_IMR_TXOK) {
			VPRINT("[%ld] interrupt... AR5K_IMR_TXOK\n", 
			       local->int_count);

			VT_WLAN_OUT32((VT_WLAN_IN32(AR5K_CR) | AR5K_CR_TXD0), AR5K_CR); /* disable transmit bit */
	
			local->tx_active = 0;
	
			local->stats.tx_packets++;
		}
	
		if(int_reg & AR5K_IMR_RXORN) {
			VDEBUG("[%ld] interrupt... AR5K_IMR_RXORN\n", 
			       local->int_count);

			local->stats.rx_over_errors++;
		}
	
		if(int_reg & AR5K_IMR_RXEOL) {
			VDEBUG("[%ld] interrupt... AR5K_IMR_RXEOL\n", 
			       local->int_count);
	
			local->stats.rx_packets++;
			local->stats.rx_dropped++;
		}
	
		if(int_reg & AR5K_IMR_RXERR) {
			VDEBUG("[%ld] interrupt... AR5K_IMR_RXERR\n", 
			       local->int_count);
	
			local->stats.rx_errors++;
		}
	
		if(int_reg & AR5K_IMR_RXOK) {
			VDEBUG("[%ld] interrupt... AR5K_IMR_RXOK\n", 
			       local->int_count);

			VT_WLAN_OUT32((VT_WLAN_IN32(AR5K_DIAG_SW) | AR5K_DIAG_RX_DIS), AR5K_DIAG_SW);
#ifdef USE_SOFTIRQ
			VT_TASKLET_SCHEDULE(&local->rx_tasklet);
#else
			vt_ar5k_rx(local);
#endif
		}
	
		if(int_reg & AR5K_IMR_SWI) {
			VDEBUG("[%ld] interrupt... AR5K_IMR_SWI\n", 
			       local->int_count);
		}
	
#ifdef USE_GPIO
		if(int_reg & AR5K_IMR_GPIO) {
			VDEBUG("[%ld] interrupt... AR5K_IMR_GPIO\n", 
			       local->int_count);
	
			vt_ar5k_gpio_get(local, 0) ? 
				vt_ar5k_gpio_int(local, 0) :
				vt_ar5k_gpio_int(local, 1);
		}
#endif
	
		local->int_count++;
	
		VT_WLAN_OUT32(local->int_mask, AR5K_IMR); /* reset interrupt mask */
		VT_WLAN_OUT32(AR5K_IER_ENABLE, AR5K_IER); /* enable interrupt */

		VEXIT();
	}

	spin_unlock(&local->lock);
	return IRQ_HANDLED;
}

int vt_ar5k_monitor_rx(vt_local_t *local, ar5k_desc_t *desc, int len)
{
	struct net_device *dev = (struct net_device*) local->dev;
	struct sk_buff *skb = NULL;
	u8 *buf, *data;
	u32 csum; /* crc32 */
	int plen = len + sizeof(linux_wlan_ng_prism_hdr_t) - 4;

	VENTER();

	/* substract checksum */
	len -= 4; 

	data = desc->data->buffer;

	if(local->prism_header) {
		VDEBUG ("local->prism_header == TRUE\n");
		linux_wlan_ng_prism_hdr_t phdr;

		skb = dev_alloc_skb(plen + 2); 
		skb_reserve(skb, 2);

		if(skb == NULL)
			goto drop;

		/* partially ripped from linux hostap driver ,)
		 * -> http://hostap.epitest.fi */
		memset(&phdr, 0, sizeof(linux_wlan_ng_prism_hdr_t));
#define LWNG_DID_BASE (4 | (1 << 6)) /* section 4, group 1 */
		phdr.msgcode = LWNG_DID_BASE;
		phdr.msglen = sizeof(linux_wlan_ng_prism_hdr_t);
		memcpy(phdr.devname, dev->name, sizeof(phdr.devname));
#define LWNG_SETVAL(f,i,s,l,d) \
phdr.f.did = LWNG_DID_BASE | (i << 12); phdr.f.status = s; phdr.f.len = l; \
phdr.f.data = d
		LWNG_SETVAL(hosttime, 1, 0, 4, jiffies);
		LWNG_SETVAL(mactime, 2, 0, 4, le32_to_cpu(desc->s.rx_compl_stat.rcv_timestamp));
		LWNG_SETVAL(channel, 3, 0, 4, le32_to_cpu(local->conf->hw_channel));
		LWNG_SETVAL(rssi, 4, 1 /* no value */, 4, 0);
		LWNG_SETVAL(sq, 5, 1 /* no value */, 4, 0);
		LWNG_SETVAL(signal, 6, 0, 4, le32_to_cpu(desc->s.rx_compl_stat.rcv_sig_strength));
		LWNG_SETVAL(noise, 7, 1 /* no value */, 4, 0);
		LWNG_SETVAL(rate, 8, 0, 4, le32_to_cpu(desc->s.rx_compl_stat.rcv_rate));
		LWNG_SETVAL(istx, 9, 0, 4, 0);
		LWNG_SETVAL(frmlen, 10, 0, 4, len);
#undef LWNG_SETVAL
#undef LWNG_DID_BASE

		/* send socket buffer */
		buf = skb_put(skb, plen);
		memset(buf, 0, plen);
		memcpy(buf, &phdr, sizeof(linux_wlan_ng_prism_hdr_t)); /* prepend prism header */
		memcpy(buf + sizeof(linux_wlan_ng_prism_hdr_t), data, len);
		memcpy(&csum, buf + plen, 4); /* crc32 */
	} else {
		VDEBUG ("local->prism_header == FALSE\n");
		skb = dev_alloc_skb(len + 2); 
		skb_reserve(skb, 2); 

		if(skb == NULL)
			goto drop;

		/* send socket buffer */
		buf = skb_put(skb, len);
		memset(buf, 0, len);
		memcpy(buf, data, len);
		memcpy(&csum, buf + len, 4); /* crc32 */
	}	

	skb->dev = dev;
	skb->mac.raw = buf; /* address 1 */				
	skb->pkt_type = PACKET_OTHERHOST;
	skb->protocol = __constant_htons(ETH_P_802_2);
	skb->csum = csum;
	
	netif_rx(skb);
	
	/* update byte count */
	local->stats.rx_bytes += skb->len;

	VEXIT();

	return(0);

 drop:
	local->stats.rx_errors++; /* frame not ok */
	local->stats.rx_dropped++; 

	return(-EINVAL); /* skip frame */
}

int vt_ar5k_80211_rx(vt_local_t *local, ar5k_desc_t *desc, int len)
{
	struct net_device *dev = (struct net_device*) local->dev;
	struct sk_buff *skb = NULL;
	u8 *buf, *data;
	u32 csum; /* crc32 */

	VENTER();

	/* substract checksum */
	len =- 4;

	data = desc->data->buffer;

	skb = dev_alloc_skb(len); 
	
	if(skb == NULL) {
		local->stats.rx_errors++; /* frame not ok */
		local->stats.rx_dropped++; 
		return(-EINVAL); /* skip frame */
	}
	
	/* send socket buffer */
	buf = skb_put(skb, len);
	memset(buf, 0, len);
	memcpy(buf, data, len);
	skb->dev = dev;
	skb->mac.raw = buf; /* address 1 */				
	skb->pkt_type = PACKET_OTHERHOST;
	skb->protocol = __constant_htons(ETH_P_802_2);
	memcpy(&csum, buf + len, 4); /* crc32 */
	skb->csum = csum;

	netif_rx(skb);
	
	/* update byte count */
	local->stats.rx_bytes += skb->len;

	VEXIT();

	return(0);
}

void vt_ar5k_rx(vt_local_t *local)
{
	struct net_device *dev = (struct net_device*) local->dev;
	ar5k_desc_t *desc;
	int i = 0;

	VENTER();

	desc = local->rx_desc;
	for(i = 0; i < local->rx_count; i++) {
		int len = desc->s.rx_compl_stat.data_len;
		
#if VT_DEBUG > 2
		VDEBUG("[%8lu][%4d] rx (done=%u,len=%u,rate=%u,sig=%u,antenna=%u,more=%u,timestamp=%u)\n", 
		       local->int_count,
		       i, 
		       desc->s.rx_compl_stat.done,
		       len,
		       desc->s.rx_compl_stat.rcv_rate,
		       desc->s.rx_compl_stat.rcv_sig_strength,
		       desc->s.rx_compl_stat.rcv_antenna,
		       desc->s.rx_compl_stat.more,
		       desc->s.rx_compl_stat.rcv_timestamp);
#endif

		/* fragmentation is not yet supported */
		if(desc->s.rx_compl_stat.more) {
			VDEBUG ("Fragmentation not supported !\n");
			local->rx_drop = 1;
			memset(&desc->s, 0, sizeof(ar5k_rx_status_t));
			desc++;
			continue;
		}

		/* drop last packet of a fragmented frame */
		if(local->rx_drop) {
			VDEBUG ("Dropping packet !\n");
			local->stats.rx_dropped++; 
			local->rx_drop = 0;
			memset(&desc->s, 0, sizeof(ar5k_rx_status_t));
			desc++;
			continue;
		}

		if(desc->s.rx_compl_stat.done) {
			if ((len <= RX_BUF_LEN) &&
			    (len > sizeof(wlan_frame_hdr_t))) {
				   
				VDEBUG("[%8lu][%4d] rx (done=%u,len=%u,rate=%u,sig=%u,antenna=%u,more=%u,timestamp=%u)\n", 
				       local->int_count,
				       i, 
				       desc->s.rx_compl_stat.done,
				       len,
				       desc->s.rx_compl_stat.rcv_rate,
				       desc->s.rx_compl_stat.rcv_sig_strength,
				       desc->s.rx_compl_stat.rcv_antenna,
				       desc->s.rx_compl_stat.more,
				       desc->s.rx_compl_stat.rcv_timestamp);
	
				   
				if(desc->s.rx_compl_stat.frame_rcv_ok) {
					wlan_frame_hdr_t *hdr;
					
					hdr = (wlan_frame_hdr_t*)desc->data->buffer;
					
	
#if VT_DEBUG > 1
					VDEBUG("rx hdr (order=%u,wep=%u,retry=%u,from_ds=%u,to_ds=%u,subtype=%u,type=%u,version=%u,addr1=" 
					       MACSTR ",addr2=" MACSTR ",addr3=" MACSTR ",seq=%u)\n",
					       hdr->control.order,
					       hdr->control.wep,
					       hdr->control.retry,
					       hdr->control.from_ds,
					       hdr->control.to_ds,
					       hdr->control.subtype,
					       hdr->control.type,
					       hdr->control.version,
					       MAC2STR(hdr->address1),
					       MAC2STR(hdr->address2),
					       MAC2STR(hdr->address3),
					       hdr->seq.num);
#endif
	
					if(local->monitor) {
						/* raw monitor rx */
						vt_ar5k_monitor_rx(local, desc, len);
					} else {
						/* normal receive */
						vt_ar5k_80211_rx(local, desc, len);
					}
					local->stats.rx_packets++;
					
				} else {
					local->stats.rx_dropped++; /* frame not ok */
				}
			}
			
			VDEBUG ("buf_len = %d, data_len = %d\n", desc->u.rx.buf_len, desc->s.rx_compl_stat.data_len);
			memset(&desc->s, 0, sizeof(ar5k_rx_status_t));
			desc->u.rx.buf_len = sizeof(ar5k_buf_t);
			desc->u.rx.inter_req = 1;
		}
		desc++;
	}
	
	VT_WLAN_OUT32((VT_WLAN_IN32(AR5K_DIAG_SW) & (~AR5K_DIAG_RX_DIS)), AR5K_DIAG_SW);

	VEXIT();
}

#ifdef USE_TX
void vt_ar5k_free_tx_queue(vt_local_t *local)
{
	VENTER();

	if(local->tx_desc_size)
		pci_free_consistent(local->pdev, local->tx_desc_size, local->tx_desc, local->tx_desc_dma);
	if(local->tx_buf_size)
		pci_free_consistent(local->pdev, local->tx_buf_size, local->tx_buf, local->tx_buf_dma);

	VEXIT();
}

int vt_ar5k_setup_tx_queue(vt_local_t *local)
{
	struct net_device *dev = (struct net_device*) local->dev;

	VENTER();

	/* tx0 buffer */
	local->tx_buf_size = mtu + sizeof(wlan_frame_hdr_t) + 4;
	local->tx_buf = pci_alloc_consistent(local->pdev, 
					     local->tx_buf_size, 
					     &local->tx_buf_dma);

	if(!local->tx_buf_dma) {
		local->tx_buf_size = 0;
		return(-ENOMEM);
	}

	memset(local->tx_buf, 0, local->tx_buf_size);

	/* tx descriptor */
	local->tx_desc_size = sizeof(ar5k_desc_t);
	local->tx_desc = pci_alloc_consistent(local->pdev, 
					      local->tx_desc_size, 
					      &local->tx_desc_dma);

	if(!local->tx_desc_dma) {
		pci_free_consistent(local->pdev, local->tx_buf_size, local->tx_buf, local->tx_buf_dma);
		local->tx_buf_size = 0;
		local->tx_desc_size = 0;
		return(-ENOMEM);
	}

	memset(local->tx_desc, 0, local->tx_desc_size);

	local->tx_desc->link_ptr = 0; /* local->tx_desc_dma; */
	local->tx_desc->buffer_ptr = local->tx_buf_dma;
	local->tx_desc->data = local->tx_buf;

	/* set tx descriptor */
	VT_WLAN_OUT32((u32)local->tx_desc_dma, AR5K_TXDP0);

	VEXIT();

	return(0);
}
#endif

int vt_ar5k_start_xmit(struct sk_buff *skb, struct net_device *dev) 
{
#ifdef USE_TX
	vt_local_t *local = (vt_local_t *)dev->priv;
	unsigned long flags;
	wlan_frame_hdr_t *frame = NULL;
	ar5k_desc_t *desc = NULL;
	u8 *buf;
#endif	
	int ret = 0;

	VENTER();

#ifdef USE_TX
	spin_lock_irqsave(&local->lock, flags);

	if(skb->len < ETH_HLEN || skb->len > mtu || local->tx_active) {
		ret = -EINVAL;
		goto out;
	}

	/* clear tx buffer */
	memset(local->tx_buf, 0, local->tx_buf_size);

	desc = local->tx_desc;

	/* init atheros descriptor */
	desc->u.tx.frame_len = ((skb->len + sizeof(wlan_frame_hdr_t)) + AR5K_DESC_ALIGN - 1) & ~(AR5K_DESC_ALIGN - 1);
	desc->u.tx.hdr_len = (sizeof(wlan_frame_hdr_t) + AR5K_DESC_ALIGN - 1) & ~(AR5K_DESC_ALIGN - 1);
	desc->u.tx.xmit_rate = 6;
	desc->u.tx.ant_mode_xmit = 1;
	desc->u.tx.clear_dest_mask = 1;
	desc->u.tx.frm_type = 3;
	desc->u.tx.inter_req = 1;
	desc->u.tx.buf_len = local->tx_buf_size;
	desc->u.tx.more = 0;

	/* init 802.11 header */
	frame = (wlan_frame_hdr_t*)desc->data;
	frame->control.to_ds = htonl(1);
	frame->control.type = htonl(WLAN_FRAME_TYPE_DATA);

	memcpy(&frame->address1, local->conf->mac_addr.octets, WLAN_MAC_ADDR_SIZE);
	memcpy(&frame->address2, local->conf->bssid.octets, WLAN_MAC_ADDR_SIZE);
	memcpy(&frame->address3, local->conf->bssid.octets, WLAN_MAC_ADDR_SIZE);

	/* copy payload */
	buf = (u8*)desc->data + sizeof(wlan_frame_hdr_t);
	memcpy(buf, skb->data, skb->len);

	local->tx_active = 1;

	VPRINT("[%ld] tx (frm_len=%u,hdr_len=%u,xmit_rate=%u,ant=%u,clr=%u,type=%u,intr=%u,buf_len=%u,more=%u,buf=%lu)\n",
	       local->int_count,
	       desc->u.tx.frame_len,
	       desc->u.tx.hdr_len,
	       desc->u.tx.xmit_rate,
	       desc->u.tx.ant_mode_xmit,
	       desc->u.tx.clear_dest_mask,
	       desc->u.tx.frm_type,
	       desc->u.tx.inter_req,
	       desc->u.tx.buf_len,
	       desc->u.tx.more,
	       (unsigned long)desc->buffer_ptr);

	/* enable transmit bit */
	VT_WLAN_OUT32((VT_WLAN_IN32(AR5K_CR) | AR5K_CR_TXE0), AR5K_CR);

 out:
	spin_unlock_irqrestore(&local->lock, flags);

#endif

	VEXIT();

	return(ret);
}

int vt_ar5k_nic_open(struct net_device *dev, u32 rx_filter) 
{
	int ret;
	vt_local_t *local = (vt_local_t *)dev->priv;
	u32 tmp_cr;

	VENTER();

	if (request_irq(dev->irq, vt_ar5k_interrupt, SA_SHIRQ, dev->name,
			dev)) {
		VPRINT("request_irq failed\n");
		return(-EINVAL);
	}

	/* init rx ring */
	if((ret = vt_ar5k_setup_rx_queue(local)) != 0)
		return(ret);

#ifdef USE_TX
	/* setup tx ring */
	if((ret = vt_ar5k_setup_tx_queue(local)) != 0)
		return(ret);
#endif
	
	/* get and store interrupt mask */
	VT_WLAN_OUT32(INIT_INTERRUPT_MASK, AR5K_IMR);
	local->int_mask = VT_WLAN_IN16(AR5K_IMR);

	/* enable interrupts */
	VT_WLAN_OUT32(AR5K_IER_ENABLE, AR5K_IER);

	/* enable rx */
/*	tmp_cr = VT_WLAN_IN32(AR5K_CR) & ~AR5K_CR_RXD | AR5K_CR_RXE; */
	tmp_cr = AR5K_CR_RXE;
	VT_WLAN_OUT32(tmp_cr, AR5K_CR); /* enable receive bit */
	VDEBUG ("Writing command register = 0x%08x\n", tmp_cr);
	VT_WLAN_OUT32(VT_WLAN_IN32(AR5K_DIAG_SW) & ~(AR5K_DIAG_RX_DIS), AR5K_DIAG_SW); 
	VT_WLAN_OUT32(rx_filter, AR5K_RX_FILTER); /* set receive filter */

#ifdef USE_TX
	/* enable tx */
	VT_WLAN_OUT32(VT_WLAN_IN32(AR5K_DIAG_SW) & ~(AR5K_DIAG_TX_DIS), AR5K_DIAG_SW); 
#endif

	netif_device_attach(dev);
	netif_start_queue(dev);

	local->nic_active = 1;

	VEXIT();

	return(ret);
}

void vt_ar5k_nic_stop(struct net_device *dev)
{
	vt_local_t *local = (vt_local_t *)dev->priv;

	VENTER();

	if(netif_running(dev)) {
		netif_stop_queue(dev);
		netif_device_detach(dev);
	}

	/* disable interrupts */
	VT_WLAN_OUT32(AR5K_IER_DISABLE, AR5K_IER);
	
	/* reset interrupt mask */
	VT_WLAN_OUT32(0, AR5K_IMR);
	local->int_mask = VT_WLAN_IN16(AR5K_IMR);
	
	/* disable rx */
	VT_WLAN_OUT32((VT_WLAN_IN32(AR5K_CR) | AR5K_CR_RXD), AR5K_CR); /* disable receive bit */
	VT_WLAN_OUT32(AR5K_DIAG_RX_DIS, AR5K_DIAG_SW); 
	VT_WLAN_OUT32(AR5K_RX_FILTER_ALL, AR5K_RX_FILTER);

	/* free rx ring */
	vt_ar5k_free_rx_queue(local);

#ifdef USE_TX
	/* disable tx */
	VT_WLAN_OUT32(AR5K_DIAG_TX_DIS, AR5K_DIAG_SW); 

	/* free tx ring */
	vt_ar5k_free_tx_queue(local);
#endif

	free_irq(dev->irq, dev);

	local->nic_active = 0;

	VEXIT();
}

int vt_ar5k_open(struct net_device *dev) 
{
	vt_local_t *local = (vt_local_t *)dev->priv;
	unsigned long flags;
	int ret = 0;
	u32 rx_filter;

	VENTER();

	/* check if card is ready or has been removed */
	if(!local->nic_ready || local == NULL)
		return(-ENODEV);

	VPRINT("net_device opened...\n");

	spin_lock_irqsave(&local->lock, flags);

	/* monitor mode? */
	rx_filter = local->monitor ? AR5K_RX_PROM : local->rx_filter;

	ret = vt_ar5k_nic_open(dev, rx_filter);

	spin_unlock_irqrestore(&local->lock, flags);

	MOD_INC_USE_COUNT;

	VEXIT();

	return(ret);
}

int vt_ar5k_stop(struct net_device *dev) 
{
	vt_local_t *local = (vt_local_t *)dev->priv;
	unsigned long flags;

	VENTER();

	/* check if card is ready or has been removed */
	if(!local->nic_ready || local == NULL)
		return(-ENODEV);

	VPRINT("net_device stopped...\n");

	spin_lock_irqsave(&local->lock, flags);

	vt_ar5k_nic_stop(dev);

	spin_unlock_irqrestore(&local->lock, flags);

	MOD_DEC_USE_COUNT;

	VEXIT();

	return(0);
}

struct net_device_stats *vt_ar5k_get_stats(struct net_device *dev) 
{
	vt_local_t *local = (vt_local_t*)dev->priv;
	VENTER();

	if(local == NULL) {
		VPRINT("priv data should not be NULL!\n");
		return(NULL);
	}

	VEXIT();
	return &local->stats;
} 

/** MODULE INIT **/
int __init vt_ar5k_init(void) 
{
	VENTER();
	
	VINIT("%s\n", version);
	
	if(vt_ar5k_reg_init() != 0)
		return(-EINVAL);
	
	if (pci_register_driver(&vt_ar5k_pci_drv_id) <= 0) {
		VINIT("no devices found, driver not installed.\n");
		pci_unregister_driver(&vt_ar5k_pci_drv_id);
		return -ENODEV;
	}
	
	VEXIT();
	
	return(0);
}

/** MODULE EXIT **/
void __exit vt_ar5k_exit(void) 
{
	VENTER();
	
	VINIT("unregistering pci device\n");
	pci_unregister_driver(&vt_ar5k_pci_drv_id);
	
	VINIT("driver unloaded\n");
	
	VEXIT();
}

module_init(vt_ar5k_init)
module_exit(vt_ar5k_exit)
