/*                                                        -*- linux-c -*-
 *   _  _ ____ __ _ ___ ____ ____ __ _ _ _ _ | 
 * .  \/  |--| | \|  |  |--< [__] | \| | _X_ | s e c u r e  s y s t e m s
 *
 * .vt|ar5k - PCI/CardBus 802.11a WirelessLAN driver for Atheros AR5k chipsets
 *
 * Copyright (c) 2002, .vantronix | secure systems
 *                     and Reyk Floeter <reyk@vantronix.net> 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <vt_ar5kconfig.h>
#include <vt_ar5k.h>

static struct proc_dir_entry *vt_ar5k_proc = NULL;

static int vt_ar5k_proc_info_read(char *page, char **start, off_t off,
				  int count, int *eof, void *data) 
{
	char *p = page;
	vt_local_t *local = (vt_local_t*)data;
	struct net_device *dev = (struct net_device*)local->dev;
	u16 ee_data;

	if (off != 0) {
		*eof = 1;
		return 0;
	}

	p += sprintf(p, "dot11manufacturerName=\"Atheros Communications, Inc.\"\n");
	p += sprintf(p, "dot11manufacturerProductName=\"AR52k 802.11a Wireless LAN Controller\"\n");
	p += sprintf(p, "dot11manufacturerProductVersion=\"MAC=0x%04x, PHY=0x%04x, A2=0x%04x\"\n",
		     local->mac_rev, local->phy_rev, local->a2_rev);
	p += sprintf(p, "ar5kNICIOPort=0x%lx\nar5kNICInterrupt=%d\n", local->io_port, local->irq);
	p += sprintf(p, "ar5kNICStatus=\"%s\"\n", local->nic_active ? "active" : "not active");
	p += sprintf(p, "ar5kNetDeviceName=\"%s\"\n", dev->name);
	p += sprintf(p, "ar5kNetDeviceState=%ld\n", dev->state);
	p += sprintf(p, "ar5kLEDMode=\"%s\"\n", local->led_mode ? "activity" : "pending");
	p += sprintf(p, "ar5kNICInterruptCount=%lx\n", local->int_count);
	p += sprintf(p, "dot11MACAddress=" MACSTR "\n", MAC2STR(dev->dev_addr));
	p += sprintf(p, "dot11TurboModeEnabled=%d\n", local->conf->turbo_enable);
	p += sprintf(p, "dot11CurrentChannel=%d\n", local->conf->channel);
	if (vt_ar5k_eeprom_read(local, AR5K_EEPROM_REG_DOMAIN_OFFSET, &ee_data) < 1) 
		p += sprintf(p, "dot11RegulatoryDomain=%x\n", ee_data);

	return (p - page);
}

void vt_ar5k_init_proc(vt_local_t *local) 
{
	local->proc = NULL;
  
	if (proc_net != NULL) {
		if (vt_ar5k_proc == NULL)
			vt_ar5k_proc = proc_mkdir(dev_info, proc_net);
		if (vt_ar5k_proc != NULL)
			local->proc = proc_mkdir(local->dev->name,
						 vt_ar5k_proc);
	}
  
	if (local->proc == NULL) {
		VINIT("proc init failed\n");
		return;
	}

	create_proc_read_entry("info", 0, local->proc,
			       vt_ar5k_proc_info_read, local);
}

void vt_ar5k_remove_proc(vt_local_t *local) 
{
	if (local->proc != NULL) {
		remove_proc_entry("info", local->proc);
		if (local->dev != NULL && local->dev->name != NULL &&
		    vt_ar5k_proc != NULL)
			remove_proc_entry(local->dev->name, vt_ar5k_proc);
	}
}


void vt_ar5k_cleanup_proc(void) 
{
	if (vt_ar5k_proc != NULL) {
		vt_ar5k_proc = NULL;
		remove_proc_entry(dev_info, proc_net);
	}
}

struct iw_statistics *vt_ar5k_wireless_stats(struct net_device *dev) 
{
	vt_local_t *local = (vt_local_t*) dev->priv;

	VENTER();

	if(local == NULL) {
		VPRINT("priv data should not be NULL!\n");
		return(NULL);
	}

	local->wext_stats->status = 0L;
	local->wext_stats->qual.qual = 0L;
	local->wext_stats->qual.level = 0L;
	local->wext_stats->qual.noise = 0L;

	local->wext_stats->qual.updated = 7;
	local->wext_stats->discard.nwid = 0L; 
	local->wext_stats->discard.code = 0L;
	local->wext_stats->discard.misc = 0L; 
  
	VEXIT();

	return(local->wext_stats);
}
