/*
 * machine_kexec.c - handle transition of Linux booting another kernel
 * Copyright (C) 2002-2003 Eric Biederman  <ebiederm@xmission.com>
 *
 * GAMECUBE/PPC32 port Copyright (C) 2004 Albert Herranz
 *
 * This source code is licensed under the GNU General Public License,
 * Version 2.  See the file COPYING for more details.
 */

#include <linux/mm.h>
#include <linux/kexec.h>
#include <linux/delay.h>
#include <asm/pgtable.h>
#include <asm/pgalloc.h>
#include <asm/mmu_context.h>
#include <asm/io.h>
#include <asm/hw_irq.h>
#include <asm/cacheflush.h>

#ifdef CONFIG_GAMECUBE
u32 kexec_gc_saved_state[KEXEC_GC_SAVED_STATE_SIZE/sizeof(u32)] = { 
	0xDEADC0DE
};
#endif /* CONFIG_GAMECUBE */

typedef void (*relocate_new_kernel_t)(
	unsigned long indirection_page, unsigned long reboot_code_buffer,
	unsigned long start_address);

const extern unsigned char relocate_new_kernel[];
const extern unsigned int relocate_new_kernel_size;
extern void use_mm(struct mm_struct *mm);

/*
 * Do not allocate memory (or fail in any way) in machine_kexec().
 * We are past the point of no return, committed to rebooting now. 
 */
void machine_kexec(struct kimage *image)
{
	unsigned long indirection_page;
	unsigned long reboot_code_buffer;
	relocate_new_kernel_t rnk;

	/* switch to an mm where the reboot_code_buffer is identity mapped */
	use_mm(&init_mm);

	/* Interrupts aren't acceptable while we reboot */
	local_irq_disable();

	reboot_code_buffer = page_to_pfn(image->reboot_code_pages) <<PAGE_SHIFT;
	indirection_page = image->head & PAGE_MASK;

	/* copy it out */
	memcpy((void *)reboot_code_buffer, 
		relocate_new_kernel, relocate_new_kernel_size);

#ifdef CONFIG_GAMECUBE
	memcpy((void *)reboot_code_buffer + relocate_new_kernel_size,
		kexec_gc_saved_state, KEXEC_GC_SAVED_STATE_SIZE);
#endif /* CONFIG_GAMECUBE */

	flush_icache_range(reboot_code_buffer, reboot_code_buffer + KEXEC_REBOOT_CODE_SIZE);

	printk(KERN_INFO "Bye!\n");
	/* now call it */
	rnk = (relocate_new_kernel_t) reboot_code_buffer;
	(*rnk)(indirection_page, reboot_code_buffer, image->start);
}
