/*
 * Network interface GDB stub
 *
 * Written by San Mehat (nettwerk@biodome.org)
 * Based upon 'gdbserial' by David Grothe (dave@gcom.com)
 * and Scott Foehner (sfoehner@engr.sgi.com)
 *
 * Twiddled for 2.6 by Robert Walsh <rjwalsh@durables.org>
 * and wangdi <wangdi@clusterfs.com>.
 *
 * Refactored for netpoll API by Matt Mackall <mpm@selenic.com>
 *
 */

#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/config.h>
#include <linux/string.h>
#include <linux/netpoll.h>

#include <asm/system.h>
#include <asm/kgdb.h>
#include <asm/io.h>
#include <asm/bitops.h>
#include <asm/system.h>
#include <asm/irq.h>
#include <asm/atomic.h>

#define IN_BUF_SIZE 512 /* power of 2, please */
#define OUT_BUF_SIZE 256

static char in_buf[IN_BUF_SIZE], out_buf[OUT_BUF_SIZE];
static int in_head, in_tail, out_count;
static atomic_t in_count;
int kgdboe = 0; /* Default to tty mode */

extern void set_debug_traps(void);
extern void breakpoint(void);
static void rx_hook(struct netpoll *np, int port, char *msg, int len);

static struct netpoll np = {
	.name = "kgdboe",
	.dev_name = "eth0",
	.rx_hook = rx_hook,
	.local_port = 6443,
	.remote_port = 6442,
	.remote_mac = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
};
static int configured;

int eth_getDebugChar(void)
{
	int chr;

	while (atomic_read(&in_count) == 0)
		netpoll_poll(&np);

	chr = in_buf[in_tail++];
	in_tail &= (IN_BUF_SIZE - 1);
	atomic_dec(&in_count);
	return chr;
}

void eth_flushDebugChar(void)
{
	if(out_count && np.dev) {
		netpoll_send_udp(&np, out_buf, out_count);
		out_count = 0;
	}
}

void eth_putDebugChar(int chr)
{
	out_buf[out_count++] = chr;
	if(out_count == OUT_BUF_SIZE)
		eth_flushDebugChar();
}

static void rx_hook(struct netpoll *np, int port, char *msg, int len)
{
	int i;

	np->remote_port = port;

	/* Is this gdb trying to attach? */
	if (!netpoll_trap() && len == 8 && !strncmp(msg, "$Hc-1#09", 8))
		kgdb_schedule_breakpoint();

	for (i = 0; i < len; i++) {
		if (msg[i] == 3)
			kgdb_schedule_breakpoint();

		if (atomic_read(&in_count) >= IN_BUF_SIZE) {
			/* buffer overflow, clear it */
			in_head = in_tail = 0;
			atomic_set(&in_count, 0);
			break;
		}
		in_buf[in_head++] = msg[i];
		in_head &= (IN_BUF_SIZE - 1);
		atomic_inc(&in_count);
	}
}

static int option_setup(char *opt)
{
	configured = !netpoll_parse_options(&np, opt);
	return 0;
}
__setup("kgdboe=", option_setup);

static int init_kgdboe(void)
{
#ifdef CONFIG_SMP
	if (num_online_cpus() > CONFIG_NO_KGDB_CPUS) {
		printk("kgdb: too manu cpus. Cannot enable debugger with more than %d cpus\n", CONFIG_NO_KGDB_CPUS);
		return -1;
	}
#endif

	set_debug_traps();

	if(!configured || netpoll_setup(&np))
		return 1;

	kgdboe = 1;
	printk(KERN_INFO "kgdb: debugging over ethernet enabled\n");

	return 0;
}

module_init(init_kgdboe);
